@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -19,6 +19,7 @@ gi.require_version("Notify", "0.7")
 gi.require_version("TotemPlParser", "1.0")
 from gi.repository import Gtk, Gio, GLib, Gdk, Notify, TotemPlParser
 
+from threading import current_thread
 from pickle import dump
 from gettext import gettext as _
 
@@ -33,7 +34,7 @@ except Exception as e:
     print("$ sudo pip3 install pylast")
     LastFM = None
 
-from lollypop.utils import is_gnome, is_unity
+from lollypop.utils import is_gnome, is_unity, set_proxy_from_gnome
 from lollypop.define import Type, LOLLYPOP_DATA_PATH
 from lollypop.window import Window
 from lollypop.database import Database
@@ -71,6 +72,11 @@ class Application(Gtk.Application):
                             application_id="org.gnome.Lollypop",
                             flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
         self.__version = version
+        # Set main thread name
+        # We force it to current python 3.6 name, to be sure in case of
+        # change in python
+        current_thread().setName("MainThread")
+        set_proxy_from_gnome()
         self.set_property("register-session", True)
         GLib.setenv("PULSE_PROP_media.role", "music", True)
         GLib.setenv("PULSE_PROP_application.icon_name",
@@ -90,12 +96,10 @@ class Application(Gtk.Application):
         self.cursors = {}
         self.window = None
         self.notify = None
-        self.lastfm = None
-        self.librefm = None
+        self.scrobblers = []
         self.debug = False
         self.__fs = None
         self.__externals_count = 0
-        self.__init_proxy()
         GLib.set_application_name("Lollypop")
         GLib.set_prgname("lollypop")
         self.add_main_option("play-ids", b"a", GLib.OptionFlags.NONE,
@@ -125,10 +129,8 @@ class Application(Gtk.Application):
         self.connect("command-line", self.__on_command_line)
         self.connect("handle-local-options", self.__on_handle_local_options)
         self.connect("activate", self.__on_activate)
-        self.register(None)
-        if self.get_is_remote():
-            Gdk.notify_startup_complete()
-        self.__listen_to_gnome_sm()
+        self.connect("shutdown", lambda a: self.__save_state())
+        Gdk.notify_startup_complete()
 
     def init(self):
         """
@@ -149,12 +151,8 @@ class Application(Gtk.Application):
         except Exception as e:
             print("Application::init():", e)
 
-        if Gtk.get_minor_version() > 18:
-            cssProviderFile = Gio.File.new_for_uri(
+        cssProviderFile = Gio.File.new_for_uri(
                  "resource:///org/gnome/Lollypop/application.css")
-        else:
-            cssProviderFile = Gio.File.new_for_uri(
-                 "resource:///org/gnome/Lollypop/application-legacy.css")
         cssProvider = Gtk.CssProvider()
         cssProvider.load_from_file(cssProviderFile)
         screen = Gdk.Screen.get_default()
@@ -163,9 +161,6 @@ class Application(Gtk.Application):
                                              Gtk.STYLE_PROVIDER_PRIORITY_USER)
         self.db = Database()
         self.playlists = Playlists()
-        # We store cursors for main thread
-        SqlCursor.add(self.db)
-        SqlCursor.add(self.playlists)
         self.albums = AlbumsDatabase()
         self.artists = ArtistsDatabase()
         self.genres = GenresDatabase()
@@ -178,9 +173,11 @@ class Application(Gtk.Application):
         self.art.update_art_size()
         if self.settings.get_value("artist-artwork"):
             GLib.timeout_add(5000, self.art.cache_artists_info)
+        # Load lastfm if support available
         if LastFM is not None:
-            self.lastfm = LastFM("lastfm")
-            self.librefm = LastFM("librefm")
+            self.scrobblers = [LastFM("lastfm"),
+                               LastFM("librefm")]
+        self.load_listenbrainz()
         if not self.settings.get_value("disable-mpris"):
             from lollypop.mpris import MPRIS
             MPRIS(self)
@@ -196,8 +193,6 @@ class Application(Gtk.Application):
         self.add_action(self.settings.create_action("playback"))
         self.add_action(self.settings.create_action("shuffle"))
 
-        self.db.upgrade()
-
     def do_startup(self):
         """
             Init application
@@ -205,7 +200,7 @@ class Application(Gtk.Application):
         Gtk.Application.do_startup(self)
         Notify.init("Lollypop")
 
-        if not self.window:
+        if self.window is None:
             self.init()
             menu = self.__setup_app_menu()
             # If GNOME/Unity, add appmenu
@@ -216,7 +211,6 @@ class Application(Gtk.Application):
             if not is_gnome() and not is_unity():
                 self.window.setup_menu(menu)
             self.window.connect("delete-event", self.__hide_on_delete)
-            self.window.init_list_one()
             self.window.show()
             self.player.restore_state()
             # We add to mainloop as we want to run
@@ -229,8 +223,8 @@ class Application(Gtk.Application):
             Quit Lollypop
             @param vacuum as bool
         """
-        # First save state
-        self.__save_state()
+        if self.settings.get_value("save-state"):
+            self.window.container.save_view_state()
         # Then vacuum db
         if vacuum:
             self.__vacuum()
@@ -252,6 +246,34 @@ class Application(Gtk.Application):
         if self.window is not None:
             self.window.set_mini()
 
+    def load_listenbrainz(self):
+        """
+            Load listenbrainz support if needed
+        """
+        if self.settings.get_value("listenbrainz-user-token").get_string():
+            from lollypop.listenbrainz import ListenBrainz
+            for scrobbler in self.scrobblers:
+                if isinstance(scrobbler, ListenBrainz):
+                    return
+            listenbrainz = ListenBrainz()
+            self.scrobblers.append(listenbrainz)
+            self.settings.bind("listenbrainz-user-token", listenbrainz,
+                               "user_token", 0)
+
+    @property
+    def lastfm(self):
+        """
+            Get lastfm provider from scrobbler
+            @return LastFM/None
+        """
+        if LastFM is None:
+            return None
+        from pylast import LastFMNetwork
+        for scrobbler in self.scrobblers:
+            if isinstance(scrobbler, LastFMNetwork):
+                return scrobbler
+        return None
+
     @property
     def gtk_application_prefer_dark_theme(self):
         """
@@ -267,10 +289,7 @@ class Application(Gtk.Application):
         """
             Save window position and view
         """
-        if self.is_fullscreen():
-            return
         if self.settings.get_value("save-state"):
-            self.window.save_view_state()
             # Save current track
             if self.player.current_track.id is None:
                 track_id = -1
@@ -283,25 +302,23 @@ class Application(Gtk.Application):
                 track_id = self.player.current_track.id
                 # Save albums context
                 try:
-                    dump(self.player.context.genre_ids,
-                         open(LOLLYPOP_DATA_PATH + "/genre_ids.bin", "wb"))
-                    dump(self.player.context.artist_ids,
-                         open(LOLLYPOP_DATA_PATH + "/artist_ids.bin", "wb"))
                     self.player.shuffle_albums(False)
-                    dump(self.player.get_albums(),
-                         open(LOLLYPOP_DATA_PATH + "/albums.bin", "wb"))
+                    with open(LOLLYPOP_DATA_PATH + "/Albums.bin", "wb") as f:
+                        dump(list(self.player.albums), f)
                 except Exception as e:
                     print("Application::__save_state()", e)
             dump(track_id, open(LOLLYPOP_DATA_PATH + "/track_id.bin", "wb"))
             dump([self.player.is_playing, self.player.is_party],
                  open(LOLLYPOP_DATA_PATH + "/player.bin", "wb"))
+            dump(self.player.queue,
+                 open(LOLLYPOP_DATA_PATH + "/queue.bin", "wb"))
             # Save current playlist
             if self.player.current_track.id == Type.RADIOS:
                 playlist_ids = [Type.RADIOS]
-            elif not self.player.get_user_playlist_ids():
+            elif not self.player.get_playlist_ids():
                 playlist_ids = []
             else:
-                playlist_ids = self.player.get_user_playlist_ids()
+                playlist_ids = self.player.get_playlist_ids()
             dump(playlist_ids,
                  open(LOLLYPOP_DATA_PATH + "/playlist_ids.bin", "wb"))
         if self.player.current_track.id is not None:
@@ -310,7 +327,7 @@ class Application(Gtk.Application):
             position = 0
         dump(position, open(LOLLYPOP_DATA_PATH + "/position.bin", "wb"))
         self.player.stop_all()
-        self.window.stop_all()
+        self.window.container.stop_all()
 
     def __vacuum(self):
         """
@@ -352,22 +369,6 @@ class Application(Gtk.Application):
                   "https://github.com/gnumdk/lollypop-portal")
             print("Application::__preload_portal():", e)
 
-    def __init_proxy(self):
-        """
-            Init proxy setting env
-        """
-        try:
-            proxy = Gio.Settings.new("org.gnome.system.proxy")
-            https = Gio.Settings.new("org.gnome.system.proxy.https")
-            mode = proxy.get_value("mode").get_string()
-            if mode != "none":
-                h = https.get_value("host").get_string()
-                p = https.get_value("port").get_int32()
-                GLib.setenv("http_proxy", "http://%s:%s" % (h, p), True)
-                GLib.setenv("https_proxy", "http://%s:%s" % (h, p), True)
-        except Exception as e:
-            print("Application::__init_proxy()", e)
-
     def __on_handle_local_options(self, app, options):
         """
             Handle local options
@@ -405,17 +406,16 @@ class Application(Gtk.Application):
             try:
                 value = options.lookup_value("play-ids").get_string()
                 ids = value.split(";")
-                track_ids = []
+                tracks = []
                 for id in ids:
                     if id[0:2] == "a:":
                         album = Album(int(id[2:]))
-                        track_ids += album.track_ids
+                        tracks += album.tracks
                     else:
-                        track_ids.append(int(id[2:]))
-                track = Track(track_ids[0])
-                self.player.load(track)
-                self.player.populate_user_playlist_by_tracks(track_ids,
-                                                             [Type.SEARCH])
+                        tracks.append(Track(int(id[2:])))
+                self.player.load(tracks[0])
+                self.player.populate_playlist_by_tracks(tracks,
+                                                        [Type.SEARCH])
             except Exception as e:
                 print(e)
                 pass
@@ -424,7 +424,7 @@ class Application(Gtk.Application):
         elif options.contains("prev"):
             self.player.prev()
         elif options.contains("emulate-phone"):
-            self.window.add_fake_phone()
+            self.window.container.add_fake_phone()
         elif len(args) > 1:
             self.player.clear_externals()
             for uri in args[1:]:
@@ -435,24 +435,12 @@ class Application(Gtk.Application):
                 parser = TotemPlParser.Parser.new()
                 parser.connect("entry-parsed", self.__on_entry_parsed)
                 parser.parse_async(uri, True, None, None)
-        elif self.window is not None and self.window.is_visible():
-            self.window.present_with_time(Gtk.get_current_event_time())
         elif self.window is not None:
-            # self.window.setup_window()
-            # self.window.present()
-            # Horrible HACK: https://bugzilla.gnome.org/show_bug.cgi?id=774130
-            self.window.save_view_state()
-            self.window.destroy()
-            self.window = Window()
-            # If not GNOME/Unity add menu to toolbar
-            if not is_gnome() and not is_unity():
-                menu = self.__setup_app_menu()
-                self.window.setup_menu(menu)
-            self.window.connect("delete-event", self.__hide_on_delete)
-            self.window.init_list_one()
-            self.window.show()
-            self.player.emit("status-changed")
-            self.player.emit("current-changed")
+            self.window.setup_window()
+            if not self.window.is_visible():
+                self.window.present()
+                self.player.emit("status-changed")
+                self.player.emit("current-changed")
         return 0
 
     def __on_entry_parsed(self, parser, uri, metadata):
@@ -489,16 +477,7 @@ class Application(Gtk.Application):
         if self.window:
             helper = TaskHelper()
             helper.run(self.art.clean_all_cache)
-            self.window.update_db()
-
-    def __set_network(self, action, param):
-        """
-            Enable/disable network
-            @param action as Gio.SimpleAction
-            @param param as GLib.Variant
-        """
-        action.set_state(param)
-        self.settings.set_value("network-access", param)
+            self.scanner.update()
 
     def __fullscreen(self, action=None, param=None):
         """
@@ -530,22 +509,6 @@ class Application(Gtk.Application):
         """
         self.window.present()
 
-    def __listen_to_gnome_sm(self):
-        """
-            Save state on EndSession signal
-        """
-        try:
-            bus = self.get_dbus_connection()
-            bus.signal_subscribe(None,
-                                 "org.gnome.SessionManager.EndSessionDialog",
-                                 "ConfirmedLogout",
-                                 "/org/gnome/SessionManager/EndSessionDialog",
-                                 None,
-                                 Gio.DBusSignalFlags.NONE,
-                                 lambda a, b, c, d, e, f: self.__save_state())
-        except Exception as e:
-            print("Application::__listen_to_gnome_sm():", e)
-
     def __settings_dialog(self, action=None, param=None):
         """
             Show settings dialog
@@ -604,13 +567,6 @@ class Application(Gtk.Application):
         updateAction.connect("activate", self.__update_db)
         self.add_action(updateAction)
 
-        networkAction = Gio.SimpleAction.new_stateful(
-           "network",
-           None,
-           GLib.Variant.new_boolean(self.settings.get_value("network-access")))
-        networkAction.connect("change-state", self.__set_network)
-        self.add_action(networkAction)
-
         fsAction = Gio.SimpleAction.new("fullscreen", None)
         fsAction.connect("activate", self.__fullscreen)
         self.add_action(fsAction)
