@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,7 +15,7 @@ from gi.repository import Gio, GLib, GdkPixbuf
 from os import mkdir, path  # FIXME Use Gio
 
 from lollypop.utils import escape
-from lollypop.define import ArtSize, Lp
+from lollypop.define import ArtSize, App
 
 
 class InfoCache:
@@ -125,7 +125,7 @@ class InfoCache:
                                              extract,
                                              0, 0)
                     extract.savev(filepath_at_size, "jpeg",
-                                  ["quality"], [str(Lp().settings.get_value(
+                                  ["quality"], [str(App().settings.get_value(
                                                 "cover-quality").get_int32())])
                 return filepath_at_size
         except Exception as e:
@@ -191,7 +191,7 @@ class InfoCache:
                                                                None)
             stream.close()
             pixbuf.savev(filepath + ".jpg",
-                         "jpeg", ["quality"], [str(Lp().settings.get_value(
+                         "jpeg", ["quality"], [str(App().settings.get_value(
                                                "cover-quality").get_int32())])
 
     def remove(prefix, suffix):
