@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -17,7 +17,7 @@ from threading import Thread
 from time import time
 
 from lollypop.inotify import Inotify
-from lollypop.define import Lp
+from lollypop.define import App
 from lollypop.sqlcursor import SqlCursor
 from lollypop.tagreader import TagReader
 from lollypop.database_history import History
@@ -44,23 +44,30 @@ class CollectionScanner(GObject.GObject, TagReader):
 
         self.__thread = None
         self.__history = None
-        if Lp().settings.get_value("auto-update"):
+        self.__disable_compilations = True
+        if App().settings.get_value("auto-update"):
             self.__inotify = Inotify()
         else:
             self.__inotify = None
-        Lp().albums.update_max_count()
+        App().albums.update_max_count()
 
-    def update(self):
+    def update(self, *ignore):
         """
             Update database
         """
-        if not self.is_locked():
-            uris = Lp().settings.get_music_uris()
+        # Stop previous scan
+        if self.is_locked():
+            self.stop()
+            GLib.timeout_add(250, self.update)
+        else:
+            self.__disable_compilations = not App().settings.get_value(
+                                                        "show-compilations")
+            uris = App().settings.get_music_uris()
             if not uris:
                 return
 
-            Lp().window.progress.add(self)
-            Lp().window.progress.set_fraction(0.0, self)
+            App().window.container.progress.add(self)
+            App().window.container.progress.set_fraction(0.0, self)
 
             self.__thread = Thread(target=self.__scan, args=(uris,))
             self.__thread.daemon = True
@@ -138,19 +145,19 @@ class CollectionScanner(GObject.GObject, TagReader):
             Update progress bar status
             @param scanned items as int, total items as int
         """
-        Lp().window.progress.set_fraction(current / total, self)
+        App().window.container.progress.set_fraction(current / total, self)
 
     def __finish(self):
         """
             Notify from main thread when scan finished
         """
-        Lp().window.progress.set_fraction(1.0, self)
+        App().window.container.progress.set_fraction(1.0, self)
         self.stop()
         self.emit("scan-finished")
         # Update max count value
-        Lp().albums.update_max_count()
-        if Lp().settings.get_value("artist-artwork"):
-            Lp().art.cache_artists_info()
+        App().albums.update_max_count()
+        if App().settings.get_value("artist-artwork"):
+            App().art.cache_artists_info()
 
     def __scan(self, uris):
         """
@@ -160,16 +167,16 @@ class CollectionScanner(GObject.GObject, TagReader):
         """
         if self.__history is None:
             self.__history = History()
-        mtimes = Lp().tracks.get_mtimes()
+        mtimes = App().tracks.get_mtimes()
         (new_tracks, new_dirs, ignore_dirs) = self.__get_objects_for_uris(
                                                                          uris)
-        orig_tracks = Lp().tracks.get_uris(ignore_dirs)
+        orig_tracks = App().tracks.get_uris(ignore_dirs)
         was_empty = len(orig_tracks) == 0
 
         if ignore_dirs:
-            if Lp().notify is not None:
-                Lp().notify.send(_("Lollypop is detecting an empty folder."),
-                                 _("Check your music settings."))
+            if App().notify is not None:
+                App().notify.send(_("Lollypop is detecting an empty folder."),
+                                  _("Check your music settings."))
         count = len(new_tracks) + len(orig_tracks)
         # Add monitors on dirs
         if self.__inotify is not None:
@@ -177,57 +184,61 @@ class CollectionScanner(GObject.GObject, TagReader):
                 if d.startswith("file://"):
                     self.__inotify.add_monitor(d)
 
-        with SqlCursor(Lp().db) as sql:
-            i = 0
-            # Look for new files/modified files
-            try:
-                to_add = []
-                for uri in new_tracks:
-                    if self.__thread is None:
-                        return
-                    try:
-                        GLib.idle_add(self.__update_progress, i, count)
-                        f = Gio.File.new_for_uri(uri)
-                        info = f.query_info("time::modified",
-                                            Gio.FileQueryInfoFlags.NONE,
-                                            None)
-                        mtime = int(info.get_attribute_as_string(
-                                                             "time::modified"))
-                        # If songs exists and mtime unchanged, continue,
-                        # else rescan
-                        if uri in orig_tracks:
-                            orig_tracks.remove(uri)
+        i = 0
+        # Look for new files/modified files
+        try:
+            to_add = []
+            SqlCursor.add(App().db)
+            for uri in new_tracks:
+                if self.__thread is None:
+                    return
+                try:
+                    GLib.idle_add(self.__update_progress, i, count)
+                    f = Gio.File.new_for_uri(uri)
+                    # We do not use time::modified because many tag editors
+                    # just preserve this setting
+                    info = f.query_info("time::changed",
+                                        Gio.FileQueryInfoFlags.NONE,
+                                        None)
+                    mtime = int(info.get_attribute_as_string(
+                                                         "time::changed"))
+                    # If songs exists and mtime unchanged, continue,
+                    # else rescan
+                    if uri in orig_tracks:
+                        orig_tracks.remove(uri)
+                        i += 1
+                        if mtime <= mtimes.get(uri, mtime + 1):
                             i += 1
-                            if mtime <= mtimes.get(uri, mtime + 1):
-                                i += 1
-                                continue
-                            else:
-                                self.__del_from_db(uri)
-                        # On first scan, use modification time
-                        # Else, use current time
-                        if not was_empty:
-                            mtime = int(time())
-                        to_add.append((uri, mtime))
-                    except Exception as e:
-                        print("CollectionScanner::__scan(mtime):", e)
-                # Clean deleted files
-                # Now because we need to populate history
-                for uri in orig_tracks:
+                            continue
+                        else:
+                            SqlCursor.allow_main_thread_execution(App().db)
+                            self.__del_from_db(uri)
+                    # On first scan, use modification time
+                    # Else, use current time
+                    if not was_empty:
+                        mtime = int(time())
+                    to_add.append((uri, mtime))
+                except Exception as e:
+                    print("CollectionScanner::__scan(mtime):", e)
+            # Clean deleted files
+            # Now because we need to populate history
+            for uri in orig_tracks:
+                i += 1
+                GLib.idle_add(self.__update_progress, i, count)
+                self.__del_from_db(uri)
+            # Add files to db
+            for (uri, mtime) in to_add:
+                try:
+                    debug("Adding file: %s" % uri)
                     i += 1
                     GLib.idle_add(self.__update_progress, i, count)
-                    self.__del_from_db(uri)
-                # Add files to db
-                for (uri, mtime) in to_add:
-                    try:
-                        debug("Adding file: %s" % uri)
-                        i += 1
-                        GLib.idle_add(self.__update_progress, i, count)
-                        self.__add2db(uri, mtime)
-                    except Exception as e:
-                        print("CollectionScanner::__scan(add):", e, uri)
-                sql.commit()
-            except Exception as e:
-                print("CollectionScanner::__scan():", e)
+                    self.__add2db(uri, mtime)
+                    SqlCursor.allow_main_thread_execution(App().db)
+                except Exception as e:
+                    print("CollectionScanner::__scan(add):", e, uri)
+            SqlCursor.remove(App().db)
+        except Exception as e:
+            print("CollectionScanner::__scan():", e)
         GLib.idle_add(self.__finish)
         del self.__history
         self.__history = None
@@ -238,6 +249,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             @param uri as string
             @param mtime as int
             @return track id as int
+            @warning, be sure SqlCursor is available for App().db
         """
         f = Gio.File.new_for_uri(uri)
         debug("CollectionScanner::add2db(): Read tags")
@@ -252,6 +264,8 @@ class CollectionScanner(GObject.GObject, TagReader):
         aa_sortnames = self.get_album_artist_sortnames(tags)
         album_artists = self.get_album_artists(tags)
         album_name = self.get_album_name(tags)
+        mb_album_id = self.get_mb_album_id(tags)
+        mb_track_id = self.get_mb_track_id(tags)
         genres = self.get_genres(tags)
         discnumber = self.get_discnumber(tags)
         discname = self.get_discname(tags)
@@ -289,6 +303,10 @@ class CollectionScanner(GObject.GObject, TagReader):
               "Add album artists %s" % album_artists)
         album_artist_ids = self.add_album_artists(album_artists, aa_sortnames)
 
+        # User does not want compilations
+        if self.__disable_compilations and not album_artist_ids:
+            album_artist_ids = artist_ids
+
         new_artist_ids = list(set(album_artist_ids) | set(artist_ids))
 
         missing_artist_ids = list(set(album_artist_ids) - set(artist_ids))
@@ -300,7 +318,8 @@ class CollectionScanner(GObject.GObject, TagReader):
 
         debug("CollectionScanner::add2db(): Add album: "
               "%s, %s" % (album_name, album_artist_ids))
-        (album_id, new_album) = self.add_album(album_name, album_artist_ids,
+        (album_id, new_album) = self.add_album(album_name, mb_album_id,
+                                               album_artist_ids,
                                                uri, loved, album_pop,
                                                album_rate, mtime)
 
@@ -308,18 +327,17 @@ class CollectionScanner(GObject.GObject, TagReader):
 
         # Add track to db
         debug("CollectionScanner::add2db(): Add track")
-        track_id = Lp().tracks.add(title, uri, duration,
-                                   tracknumber, discnumber, discname,
-                                   album_id, year, track_pop, track_rate,
-                                   track_ltime, mtime)
+        track_id = App().tracks.add(title, uri, duration,
+                                    tracknumber, discnumber, discname,
+                                    album_id, year, track_pop, track_rate,
+                                    track_ltime, mtime, mb_track_id)
 
         debug("CollectionScanner::add2db(): Update track")
         self.update_track(track_id, artist_ids, genre_ids)
         debug("CollectionScanner::add2db(): Update album")
         self.update_album(album_id, album_artist_ids, genre_ids, year)
         if new_album:
-            with SqlCursor(Lp().db) as sql:
-                sql.commit()
+            SqlCursor.commit(App().db)
         for genre_id in genre_ids:
             GLib.idle_add(self.emit, "genre-updated", genre_id, True)
         for artist_id in new_artist_ids:
@@ -334,35 +352,41 @@ class CollectionScanner(GObject.GObject, TagReader):
         try:
             f = Gio.File.new_for_uri(uri)
             name = f.get_basename()
-            track_id = Lp().tracks.get_id_by_uri(uri)
-            album_id = Lp().tracks.get_album_id(track_id)
-            genre_ids = Lp().tracks.get_genre_ids(track_id)
-            album_artist_ids = Lp().albums.get_artist_ids(album_id)
-            artist_ids = Lp().tracks.get_artist_ids(track_id)
-            popularity = Lp().tracks.get_popularity(track_id)
-            rate = Lp().tracks.get_rate(track_id)
-            ltime = Lp().tracks.get_ltime(track_id)
-            mtime = Lp().tracks.get_mtime(track_id)
-            duration = Lp().tracks.get_duration(track_id)
-            album_popularity = Lp().albums.get_popularity(album_id)
-            album_rate = Lp().albums.get_rate(album_id)
-            loved = Lp().albums.get_loved(album_id)
-            uri = Lp().tracks.get_uri(track_id)
+            track_id = App().tracks.get_id_by_uri(uri)
+            album_id = App().tracks.get_album_id(track_id)
+            genre_ids = App().tracks.get_genre_ids(track_id)
+            album_artist_ids = App().albums.get_artist_ids(album_id)
+            artist_ids = App().tracks.get_artist_ids(track_id)
+            popularity = App().tracks.get_popularity(track_id)
+            rate = App().tracks.get_rate(track_id)
+            ltime = App().tracks.get_ltime(track_id)
+            mtime = App().tracks.get_mtime(track_id)
+            duration = App().tracks.get_duration(track_id)
+            album_popularity = App().albums.get_popularity(album_id)
+            album_rate = App().albums.get_rate(album_id)
+            loved = App().albums.get_loved(album_id)
+            uri = App().tracks.get_uri(track_id)
             self.__history.add(name, duration, popularity, rate,
                                ltime, mtime, loved, album_popularity,
                                album_rate)
-            Lp().tracks.remove(track_id)
-            Lp().tracks.clean(track_id)
-            deleted = Lp().albums.clean(album_id)
-            if deleted:
-                with SqlCursor(Lp().db) as sql:
-                    sql.commit()
-                GLib.idle_add(self.emit, "album-updated", album_id, True)
+            App().tracks.remove(track_id)
+            App().tracks.clean(track_id)
+            cleaned = App().albums.clean(album_id)
+            if cleaned:
+                SqlCursor.commit(App().db)
+                GLib.idle_add(self.emit, "album-updated",
+                              album_id, True)
             for artist_id in album_artist_ids + artist_ids:
-                Lp().artists.clean(artist_id)
-                GLib.idle_add(self.emit, "artist-updated", artist_id, False)
+                cleaned = App().artists.clean(artist_id)
+                # Force update even if not cleaned as artist may
+                # have been removed from a selected genre
+                GLib.idle_add(self.emit, "artist-updated",
+                              artist_id, False)
             for genre_id in genre_ids:
-                Lp().genres.clean(genre_id)
-                GLib.idle_add(self.emit, "genre-updated", genre_id, False)
+                cleaned = App().genres.clean(genre_id)
+                if cleaned:
+                    SqlCursor.commit(App().db)
+                    GLib.idle_add(self.emit, "genre-updated",
+                                  genre_id, False)
         except Exception as e:
             print("CollectionScanner::__del_from_db:", e)
