@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,10 +15,11 @@ from gi.repository import Gtk, Gio, GLib
 from gettext import gettext as _
 from urllib.parse import urlparse
 
-from lollypop.define import Lp, Type
+from lollypop.define import App, Type
 from lollypop.loader import Loader
 from lollypop.selectionlist import SelectionList
 from lollypop.view_container import ViewContainer
+from lollypop.view import View
 from lollypop.progressbar import ProgressBar
 
 
@@ -29,7 +30,7 @@ class Device:
     uri = None
 
 
-class Container:
+class Container(Gtk.Bin):
     """
         Container for main view
     """
@@ -37,23 +38,18 @@ class Container:
         """
             Init container
         """
+        Gtk.Bin.__init__(self)
         self.__pulse_timeout = None
         # Index will start at -VOLUMES
         self.__devices = {}
         self.__devices_index = Type.DEVICES
-        self.__show_genres = Lp().settings.get_value("show-genres")
+        self.__show_genres = App().settings.get_value("show-genres")
         self.__stack = ViewContainer(500)
         self.__stack.show()
 
         self.__setup_view()
         self.__setup_scanner()
 
-        (list_one_ids, list_two_ids) = self.__get_saved_view_state()
-        if list_one_ids and list_one_ids[0] != Type.NONE:
-            self.__list_one.select_ids(list_one_ids)
-        if list_two_ids and list_two_ids[0] != Type.NONE:
-            self.__list_two.select_ids(list_two_ids)
-
         # Volume manager
         self.__vm = Gio.VolumeMonitor.get()
         self.__vm.connect("mount-added", self.__on_mount_added)
@@ -61,32 +57,9 @@ class Container:
         for mount in self.__vm.get_mounts():
             self.__add_device(mount, False)
 
-        Lp().playlists.connect("playlists-changed",
-                               self.__update_playlists)
-
-    def update_db(self, *ignore):
-        """
-            Update db at startup only if needed
-        """
-        # Stop previous scan
-        if Lp().scanner.is_locked():
-            Lp().scanner.stop()
-            GLib.timeout_add(250, self.update_db)
-        else:
-            # Allow user to disable network access before first load
-            if Lp().tracks.count() == 0:
-                self.__show_first_run()
-            Lp().scanner.update()
-
-    def get_genre_id(self):
-        """
-            Return current selected genre
-            @return genre id as int
-        """
-        if self.__show_genres:
-            return self.__list_one.get_selected_id()
-        else:
-            return None
+        App().playlists.connect("playlists-changed",
+                                self.__update_playlists)
+        self.add(self.__paned_main_list)
 
     def init_list_one(self):
         """
@@ -94,32 +67,54 @@ class Container:
         """
         self.__update_list_one(None)
 
+    def restore_view_state(self):
+        """
+            Restore view state
+        """
+        # Get list one ids (always)
+        list_one_ids = []
+        ids = App().settings.get_value("list-one-ids")
+        for i in ids:
+            if isinstance(i, int):
+                list_one_ids.append(i)
+        if not list_one_ids:
+            list_one_ids = [Type.POPULARS]
+
+        # Get list two ids (only on save state)
+        list_two_ids = [Type.NONE]
+        if App().settings.get_value("save-state"):
+            list_two_ids = []
+            ids = App().settings.get_value("list-two-ids")
+            for i in ids:
+                if isinstance(i, int):
+                    list_two_ids.append(i)
+        if list_one_ids and list_one_ids[0] != Type.NONE:
+            self.__list_one.select_ids(list_one_ids)
+        if list_two_ids and list_two_ids[0] != Type.NONE:
+            self.__list_two.select_ids(list_two_ids)
+
     def save_view_state(self):
         """
             Save view state
         """
-        Lp().settings.set_value(
+        App().settings.set_value(
                             "list-one-ids",
                             GLib.Variant("ai",
                                          self.__list_one.selected_ids))
-        Lp().settings.set_value(
+        App().settings.set_value(
                             "list-two-ids",
                             GLib.Variant("ai",
                                          self.__list_two.selected_ids))
 
-    def show_playlist_manager(self, object_id, genre_ids,
-                              artist_ids, is_album):
+    def show_playlist_manager(self, object):
         """
             Show playlist manager for object_id
             Current view stay present in ViewContainer
-            @param object id as int
-            @param genre ids as [int]
-            @param artist ids as [int]
-            @param is_album as bool
+            @param object as Track/Album/Disc
         """
         from lollypop.view_playlists import PlaylistsManageView
         current = self.__stack.get_visible_child()
-        view = PlaylistsManageView(object_id, genre_ids, artist_ids, is_album)
+        view = PlaylistsManageView(object)
         view.populate()
         view.show()
         self.__stack.add(view)
@@ -177,37 +172,6 @@ class Container:
                 self.__stack.clean_old_views(child)
                 break
 
-    @property
-    def view(self):
-        """
-            Disable overlays
-        """
-        return self.__stack.get_visible_child()
-
-    @property
-    def progress(self):
-        """
-            Progress bar
-            @return ProgressBar
-        """
-        return self.__progress
-
-    def add_remove_from(self, value, list_one, add):
-        """
-            Add or remove value to list
-            @param value as (int, str)
-            @param list one as bool
-            @param add as bool
-        """
-        if list_one:
-            l = self.__list_one
-        else:
-            l = self.__list_two
-        if add:
-            l.add_value(value)
-        else:
-            l.remove_value(value[0])
-
     def reload_view(self):
         """
             Reload current view
@@ -239,13 +203,6 @@ class Container:
                 self.__pulse_timeout = None
                 self.__progress.hide()
 
-    def on_scan_finished(self, scanner):
-        """
-            Mark force scan as False, update lists
-            @param scanner as CollectionScanner
-        """
-        self.__update_lists(scanner)
-
     def add_fake_phone(self):
         """
             Emulate an Android Phone
@@ -274,9 +231,9 @@ class Container:
             # Get artist genres
             genre_ids = []
             for artist_id in artist_ids:
-                album_ids = Lp().artists.get_albums(artist_ids)
+                album_ids = App().artists.get_albums(artist_ids)
                 for album_id in album_ids:
-                    for genre_id in Lp().albums.get_genre_ids(album_id):
+                    for genre_id in App().albums.get_genre_ids(album_id):
                         if genre_id not in genre_ids:
                             genre_ids.append(genre_id)
             # Select genres on list one
@@ -287,12 +244,56 @@ class Container:
             # Select artists on list one
             GLib.idle_add(self.__list_one.select_ids, artist_ids)
 
-##############
-# PROTECTED  #
-##############
-    def _hide_pane(self):
+    def go_back(self):
         """
-            Hide navigation pane
+            Go back in stack
+        """
+        visible_child = self.__stack.get_visible_child()
+        if visible_child == self.__list_two:
+            self.__stack.set_visible_child(self.__list_one)
+            App().window.toolbar.playback.show_back(True)
+        elif self.__list_two.is_visible():
+            self.__stack.set_visible_child(self.__list_two)
+        else:
+            self.__stack.set_visible_child(self.__list_one)
+            App().window.toolbar.playback.show_back(True)
+
+    def save_internals(self):
+        """
+            Save paned position
+        """
+        main_pos = self.__paned_main_list.get_position()
+        listview_pos = self.__paned_list_view.get_position()
+        listview_pos = listview_pos if listview_pos > 100 else 100
+        App().settings.set_value("paned-mainlist-width",
+                                 GLib.Variant("i",
+                                              main_pos))
+        App().settings.set_value("paned-listview-width",
+                                 GLib.Variant("i",
+                                              listview_pos))
+
+    def paned_stack(self, b):
+        """
+            Enable paned stack
+            @param bool as b
+        """
+        if b and not self.is_paned_stack:
+            self.__paned_list_view.remove(self.__list_two)
+            self.__paned_main_list.remove(self.__list_one)
+            self.__stack.add(self.__list_one)
+            self.__stack.add(self.__list_two)
+            App().window.toolbar.playback.show_back(True)
+            self.__stack.set_visible_child(self.__list_one)
+        elif not b and self.is_paned_stack:
+            self.__stack.remove(self.__list_two)
+            self.__stack.remove(self.__list_one)
+            self.__paned_list_view.add1(self.__list_two)
+            self.__paned_main_list.add1(self.__list_one)
+            App().window.toolbar.playback.show_back(False)
+
+    def hide_paned(self):
+        """
+            Hide navigation paned
             Internally hide list one and list two
         """
         if self.__list_one.is_visible():
@@ -302,9 +303,38 @@ class Container:
             self.__list_one.show()
             if self.__list_two.was_visible:
                 self.__list_two.show()
-        Lp().settings.set_value("show-navigation-list",
-                                GLib.Variant("b",
-                                             self.__list_one.is_visible()))
+        App().settings.set_value("show-navigation-list",
+                                 GLib.Variant("b",
+                                              self.__list_one.is_visible()))
+
+    @property
+    def view(self):
+        """
+            Disable overlays
+        """
+        view = self.__stack.get_visible_child()
+        if view is not None and not isinstance(view, View):
+            for child in self.__stack.get_children():
+                if isinstance(child, View):
+                    view = child
+                    break
+        return view
+
+    @property
+    def progress(self):
+        """
+            Progress bar
+            @return ProgressBar
+        """
+        return self.__progress
+
+    @property
+    def is_paned_stack(self):
+        """
+            Return True if stack contains paned
+            return True
+        """
+        return self.__list_one in self.__stack.get_children()
 
 ############
 # PRIVATE  #
@@ -314,25 +344,13 @@ class Container:
             Make progress bar pulse while visible
             @param pulse as bool
         """
-        if self.__progress.is_visible() and not Lp().scanner.is_locked():
+        if self.__progress.is_visible() and not App().scanner.is_locked():
             self.__progress.pulse()
             return True
         else:
             self.__progress.set_fraction(0.0, self)
             return False
 
-    def __show_first_run(self):
-        """
-            Show first run view
-        """
-        from lollypop.view_first_run import FirstRunView
-        self.__stop_current_view()
-        view = FirstRunView()
-        view.show()
-        self.__stack.add(view)
-        self.__stack.set_visible_child(view)
-        self.__stack.clean_old_views(view)
-
     def __setup_view(self):
         """
             Setup window main view:
@@ -340,13 +358,13 @@ class Container:
                 - artist list
                 - main view as artist view or album view
         """
-        self._paned_main_list = Gtk.Paned.new(Gtk.Orientation.HORIZONTAL)
-        self._paned_list_view = Gtk.Paned.new(Gtk.Orientation.HORIZONTAL)
+        self.__paned_main_list = Gtk.Paned.new(Gtk.Orientation.HORIZONTAL)
+        self.__paned_list_view = Gtk.Paned.new(Gtk.Orientation.HORIZONTAL)
         vgrid = Gtk.Grid()
         vgrid.set_orientation(Gtk.Orientation.VERTICAL)
 
         self.__list_one = SelectionList()
-        if Lp().settings.get_value("show-navigation-list"):
+        if App().settings.get_value("show-navigation-list"):
             self.__list_one.show()
         self.__list_two = SelectionList()
         self.__list_one.connect("item-selected", self.__on_list_one_selected)
@@ -362,48 +380,26 @@ class Container:
         vgrid.add(self.__progress)
         vgrid.show()
 
-        self._paned_list_view.add1(self.__list_two)
-        self._paned_list_view.add2(vgrid)
-        self._paned_main_list.add1(self.__list_one)
-        self._paned_main_list.add2(self._paned_list_view)
-        self._paned_main_list.set_position(
-            Lp().settings.get_value("paned-mainlist-width").get_int32())
-        self._paned_list_view.set_position(
-            Lp().settings.get_value("paned-listview-width").get_int32())
-        self._paned_main_list.show()
-        self._paned_list_view.show()
-
-    def __get_saved_view_state(self):
-        """
-            Get save view state
-            @return (list one id, list two id)
-        """
-        # Get list one ids (always)
-        list_one_ids = []
-        ids = Lp().settings.get_value("list-one-ids")
-        for i in ids:
-            if isinstance(i, int):
-                list_one_ids.append(i)
-        if not list_one_ids:
-            list_one_ids = [Type.POPULARS]
-        list_two_ids = [Type.NONE]
-        # Get list two ids (only on save state)
-        if Lp().settings.get_value("save-state"):
-            list_two_ids = []
-            ids = Lp().settings.get_value("list-two-ids")
-            for i in ids:
-                if isinstance(i, int):
-                    list_two_ids.append(i)
-        return (list_one_ids, list_two_ids)
+        self.__paned_list_view.add1(self.__list_two)
+        self.__paned_list_view.add2(vgrid)
+        self.__paned_main_list.add1(self.__list_one)
+        self.__paned_main_list.add2(self.__paned_list_view)
+        self.__paned_main_list.set_position(
+            App().settings.get_value("paned-mainlist-width").get_int32())
+        self.__paned_list_view.set_position(
+            App().settings.get_value("paned-listview-width").get_int32())
+        self.__paned_main_list.show()
+        self.__paned_list_view.show()
 
     def __setup_scanner(self):
         """
             Run collection update if needed
             @return True if hard scan is running
         """
-        Lp().scanner.connect("scan-finished", self.on_scan_finished)
-        Lp().scanner.connect("genre-updated", self.__on_genre_updated)
-        Lp().scanner.connect("artist-updated", self.__on_artist_updated)
+        App().scanner.connect("scan-finished",
+                              lambda x: self.__update_lists(App().scanner))
+        App().scanner.connect("genre-updated", self.__on_genre_updated)
+        App().scanner.connect("artist-updated", self.__on_artist_updated)
 
     def __update_playlists(self, playlists, playlist_id):
         """
@@ -413,9 +409,9 @@ class Container:
         """
         ids = self.__list_one.selected_ids
         if ids and ids[0] == Type.PLAYLISTS:
-            if Lp().playlists.exists(playlist_id):
+            if App().playlists.exists(playlist_id):
                 self.__list_two.update_value(playlist_id,
-                                             Lp().playlists.get_name(
+                                             App().playlists.get_name(
                                                                   playlist_id))
             else:
                 self.__list_two.remove_value(playlist_id)
@@ -459,7 +455,7 @@ class Container:
             @thread safe
         """
         def load():
-            genres = Lp().genres.get()
+            genres = App().genres.get()
             return genres
 
         def setup(genres):
@@ -484,8 +480,8 @@ class Container:
             @thread safe
         """
         def load():
-            artists = Lp().artists.get(genre_ids)
-            compilations = Lp().albums.get_compilation_ids(genre_ids)
+            artists = App().artists.get(genre_ids)
+            compilations = App().albums.get_compilation_ids(genre_ids)
             return (artists, compilations)
 
         def setup(artists, compilations):
@@ -520,7 +516,7 @@ class Container:
             @thread safe
         """
         items = self.__list_two.get_pl_headers()
-        items += Lp().playlists.get()
+        items += App().playlists.get()
         if update:
             self.__list_two.update_values(items)
         else:
@@ -536,44 +532,55 @@ class Container:
             if hasattr(child, "stop"):
                 child.stop()
 
-    def __update_view_device(self, device_id):
+    def __get_view_device(self, device_id):
         """
-            Update current view with device view,
+            Get device view for id
             Use existing view if available
             @param device id as int
+            @return View
         """
         from lollypop.view_device import DeviceView, DeviceLocked
         self.__stop_current_view()
         device = self.__devices[device_id]
-        child = self.__stack.get_child_by_name(device.uri)
-        if child is None:
+        device_view = None
+
+        # Search a device child with uri
+        for child in self.__stack.get_children():
+            if isinstance(child, DeviceView):
+                if child.device.uri == device.uri:
+                    device_view = child
+                    break
+            elif isinstance(child, DeviceLocked):
+                device_view = child
+                break
+
+        # If no view available, get a new one
+        if device_view is None:
             files = DeviceView.get_files(device.uri)
             if files:
-                if child is None:
-                    child = DeviceView(device)
-                    self.__stack.add_named(child, device.uri)
+                device_view = DeviceView(device)
+                device_view.populate()
+                self.__stack.add_named(child, device.uri)
             else:
-                child = DeviceLocked()
+                device_view = DeviceLocked()
                 self.__stack.add(child)
-            child.show()
-        child.populate()
-        self.__stack.set_visible_child(child)
-        self.__stack.clean_old_views(child)
+            device_view.show()
+        return device_view
 
-    def __update_view_artists(self, genre_ids, artist_ids):
+    def __get_view_artists(self, genre_ids, artist_ids):
         """
-            Update current view with artists view
+            Get artists view for genres/artists
             @param genre ids as [int]
             @param artist ids as [int]
         """
         def load():
             if genre_ids and genre_ids[0] == Type.ALL:
-                albums = Lp().albums.get_ids(artist_ids, [])
+                albums = App().albums.get_ids(artist_ids, [])
             else:
                 albums = []
                 if artist_ids and artist_ids[0] == Type.COMPILATIONS:
-                    albums += Lp().albums.get_compilation_ids(genre_ids)
-                albums += Lp().albums.get_ids(artist_ids, genre_ids)
+                    albums += App().albums.get_compilation_ids(genre_ids)
+                albums += App().albums.get_ids(artist_ids, genre_ids)
             return albums
         from lollypop.view_artist import ArtistView
         self.__stop_current_view()
@@ -581,13 +588,11 @@ class Container:
         loader = Loader(target=load, view=view)
         loader.start()
         view.show()
-        self.__stack.add(view)
-        self.__stack.set_visible_child(view)
-        self.__stack.clean_old_views(view)
+        return view
 
-    def __update_view_albums(self, genre_ids, artist_ids):
+    def __get_view_albums(self, genre_ids, artist_ids):
         """
-            Update current view with albums view
+            Get albums view for genres/artists
             @param genre ids as [int]
             @param is compilation as bool
         """
@@ -596,64 +601,65 @@ class Container:
             is_compilation = artist_ids and artist_ids[0] == Type.COMPILATIONS
             if genre_ids and genre_ids[0] == Type.ALL:
                 if is_compilation or\
-                        Lp().settings.get_value("show-compilations"):
-                    items = Lp().albums.get_compilation_ids()
+                        App().settings.get_value(
+                                            "show-compilations-in-album-view"):
+                    items = App().albums.get_compilation_ids()
                 if not is_compilation:
-                    items += Lp().albums.get_ids()
+                    items += App().albums.get_ids()
             elif genre_ids and genre_ids[0] == Type.POPULARS:
-                items = Lp().albums.get_rated()
+                items = App().albums.get_rated()
                 count = 100 - len(items)
-                for album in Lp().albums.get_populars(count):
+                for album in App().albums.get_populars(count):
                     if album not in items:
                         items.append(album)
             elif genre_ids and genre_ids[0] == Type.LOVED:
-                items = Lp().albums.get_loves()
+                items = App().albums.get_loves()
             elif genre_ids and genre_ids[0] == Type.RECENTS:
-                items = Lp().albums.get_recents()
+                items = App().albums.get_recents()
             elif genre_ids and genre_ids[0] == Type.NEVER:
-                items = Lp().albums.get_never_listened_to()
+                items = App().albums.get_never_listened_to()
             elif genre_ids and genre_ids[0] == Type.RANDOMS:
-                items = Lp().albums.get_randoms()
+                items = App().albums.get_randoms()
             else:
                 if is_compilation or\
-                        Lp().settings.get_value("show-compilations"):
-                    items = Lp().albums.get_compilation_ids(genre_ids)
+                        App().settings.get_value(
+                                            "show-compilations-in-album-view"):
+                    items = App().albums.get_compilation_ids(genre_ids)
                 if not is_compilation:
-                    items += Lp().albums.get_ids([], genre_ids)
+                    items += App().albums.get_ids([], genre_ids)
             return items
 
-        from lollypop.view_albums import AlbumsView
+        from lollypop.view_albums_box import AlbumsBoxView
         self.__stop_current_view()
-        view = AlbumsView(genre_ids, artist_ids)
+        view = AlbumsBoxView(genre_ids, artist_ids)
         loader = Loader(target=load, view=view)
         loader.start()
         view.show()
-        self.__stack.add(view)
-        self.__stack.set_visible_child(view)
-        self.__stack.clean_old_views(view)
+        return view
 
-    def __update_view_playlists(self, playlist_ids=[]):
+    def __get_view_playlists(self, playlist_ids=[]):
         """
-            Update current view with playlist view
+            Get playlits view for playlists
             @param playlist ids as [int]
+            @return View
         """
         def load():
             track_ids = []
             for playlist_id in playlist_ids:
                 if playlist_id == Type.POPULARS:
-                    tracks = Lp().tracks.get_rated()
-                    for track in Lp().tracks.get_populars():
+                    tracks = App().tracks.get_rated()
+                    for track in App().tracks.get_populars():
                         tracks.append(track)
                 elif playlist_id == Type.RECENTS:
-                    tracks = Lp().tracks.get_recently_listened_to()
+                    tracks = App().tracks.get_recently_listened_to()
                 elif playlist_id == Type.NEVER:
-                    tracks = Lp().tracks.get_never_listened_to()
+                    tracks = App().tracks.get_never_listened_to()
                 elif playlist_id == Type.RANDOMS:
-                    tracks = Lp().tracks.get_randoms()
+                    tracks = App().tracks.get_randoms()
                 elif playlist_id == Type.LOVED:
-                    tracks = Lp().playlists.get_track_ids_sorted(playlist_id)
+                    tracks = App().playlists.get_track_ids_sorted(playlist_id)
                 else:
-                    tracks = Lp().playlists.get_track_ids(playlist_id)
+                    tracks = App().playlists.get_track_ids(playlist_id)
                 for track_id in tracks:
                     if track_id not in track_ids:
                         track_ids.append(track_id)
@@ -668,25 +674,21 @@ class Container:
             loader.start()
         else:
             from lollypop.view_playlists import PlaylistsManageView
-            view = PlaylistsManageView(Type.NONE, [], [], False)
+            view = PlaylistsManageView(None)
             view.populate()
         view.show()
-        self.__stack.add(view)
-        self.__stack.set_visible_child(view)
-        self.__stack.clean_old_views(view)
+        return view
 
-    def __update_view_radios(self):
+    def __get_view_radios(self):
         """
-            Update current view with radios view
+            Get radios view
         """
         from lollypop.view_radios import RadiosView
         self.__stop_current_view()
         view = RadiosView()
         view.populate()
         view.show()
-        self.__stack.add(view)
-        self.__stack.set_visible_child(view)
-        self.__stack.clean_old_views(view)
+        return view
 
     def __add_device(self, mount, show=False):
         """
@@ -702,7 +704,7 @@ class Container:
         parsed = urlparse(uri)
         is_removable = parsed.scheme == "file" and\
             mount.can_eject() and\
-            Lp().settings.get_value("sync-to-usb-disks")
+            App().settings.get_value("sync-to-usb-disks")
         if is_removable or parsed.scheme == "mtp":
             self.__devices_index -= 1
             dev = Device()
@@ -733,43 +735,58 @@ class Container:
             Update view based on selected object
             @param list as SelectionList
         """
+        view = None
         selected_ids = self.__list_one.selected_ids
         if not selected_ids:
             return
         if selected_ids[0] == Type.PLAYLISTS:
-            if Lp().settings.get_value("show-navigation-list"):
+            if App().settings.get_value("show-navigation-list"):
                 self.__list_two.show()
             if not self.__list_two.will_be_selected():
-                self.__update_view_playlists()
+                view = self.__get_view_playlists()
             self.__update_list_playlists(False)
         elif Type.DEVICES - 999 < selected_ids[0] < Type.DEVICES:
             self.__list_two.hide()
             if not self.__list_two.will_be_selected():
-                self.__update_view_device(selected_ids[0])
+                view = self.__get_view_device(selected_ids[0])
         elif selected_ids[0] in [Type.POPULARS,
                                  Type.LOVED,
                                  Type.RECENTS,
                                  Type.NEVER,
                                  Type.RANDOMS]:
             self.__list_two.hide()
-            self.__update_view_albums(selected_ids, [])
+            view = self.__get_view_albums(selected_ids, [])
         elif selected_ids[0] == Type.RADIOS:
             self.__list_two.hide()
-            self.__update_view_radios()
+            view = self.__get_view_radios()
         elif selection_list.is_marked_as_artists():
             self.__list_two.hide()
             if selected_ids[0] == Type.ALL:
-                self.__update_view_albums(selected_ids, [])
+                view = self.__get_view_albums(selected_ids, [])
             elif selected_ids[0] == Type.COMPILATIONS:
-                self.__update_view_albums([], selected_ids)
+                view = self.__get_view_albums([], selected_ids)
             else:
-                self.__update_view_artists([], selected_ids)
+                view = self.__get_view_artists([], selected_ids)
         else:
             self.__update_list_artists(self.__list_two, selected_ids, False)
-            if Lp().settings.get_value("show-navigation-list"):
+            if App().settings.get_value("show-navigation-list"):
                 self.__list_two.show()
             if not self.__list_two.will_be_selected():
-                self.__update_view_albums(selected_ids, [])
+                view = self.__get_view_albums(selected_ids, [])
+        if view is not None:
+            if self.is_paned_stack:
+                # Just to make it sensitive
+                App().window.toolbar.playback.show_back(True, True)
+            if view not in self.__stack.get_children():
+                self.__stack.add(view)
+            # If we are in paned stack mode, show list two if wanted
+            if self.is_paned_stack and\
+                    self.__list_two.is_visible() and (
+                    selected_ids[0] >= 0 or selected_ids[0] == Type.PLAYLISTS):
+                self.__stack.set_visible_child(self.__list_two)
+            else:
+                self.__stack.set_visible_child(view)
+            self.__stack.clean_old_views(view)
 
     def __on_list_populated(self, selection_list):
         """
@@ -789,11 +806,14 @@ class Container:
         if not selected_ids or not genre_ids:
             return
         if genre_ids[0] == Type.PLAYLISTS:
-            self.__update_view_playlists(selected_ids)
+            view = self.__get_view_playlists(selected_ids)
         elif selected_ids[0] == Type.COMPILATIONS:
-            self.__update_view_albums(genre_ids, selected_ids)
+            view = self.__get_view_albums(genre_ids, selected_ids)
         else:
-            self.__update_view_artists(genre_ids, selected_ids)
+            view = self.__get_view_artists(genre_ids, selected_ids)
+        self.__stack.add(view)
+        self.__stack.set_visible_child(view)
+        self.__stack.clean_old_views(view)
 
     def __on_pass_focus(self, selection_list):
         """
@@ -815,28 +835,26 @@ class Container:
         """
         if self.__show_genres:
             if add:
-                genre_name = Lp().genres.get_name(genre_id)
+                genre_name = App().genres.get_name(genre_id)
                 self.__list_one.add_value((genre_id, genre_name))
             else:
-                genre_ids = Lp().genres.get_ids()
-                if genre_id not in genre_ids:
-                    self.__list_one.remove_value(genre_id)
+                self.__list_one.remove_value(genre_id)
 
     def __on_artist_updated(self, scanner, artist_id, add):
         """
-            Add artist to artist list
+            Add/remove artist to/from list
             @param scanner as CollectionScanner
             @param artist id as int
             @param add as bool
         """
-        artist_name = Lp().artists.get_name(artist_id)
-        sortname = Lp().artists.get_sortname(artist_id)
+        artist_name = App().artists.get_name(artist_id)
+        sortname = App().artists.get_sortname(artist_id)
         if self.__show_genres:
             l = self.__list_two
-            artist_ids = Lp().artists.get_ids(self.__list_one.selected_ids)
+            artist_ids = App().artists.get_ids(self.__list_one.selected_ids)
         else:
             l = self.__list_one
-            artist_ids = Lp().artists.get_ids()
+            artist_ids = App().artists.get_ids()
         if add:
             if artist_id in artist_ids:
                 l.add_value((artist_id, artist_name, sortname))
