@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,7 +15,7 @@ from gettext import gettext as _
 import itertools
 
 from lollypop.sqlcursor import SqlCursor
-from lollypop.define import Lp
+from lollypop.define import App
 from lollypop.utils import noaccents
 
 
@@ -31,8 +31,8 @@ class TracksDatabase:
         """
         pass
 
-    def add(self, name, uri, duration, tracknumber, discnumber,
-            discname, album_id, year, popularity, rate, ltime, mtime):
+    def add(self, name, uri, duration, tracknumber, discnumber, discname,
+            album_id, year, popularity, rate, ltime, mtime, mb_track_id):
         """
             Add a new track to database
             @param name as string
@@ -48,15 +48,16 @@ class TracksDatabase:
             @param rate as int
             @param ltime as int
             @param mtime as int
+            @param mb_track_id as str
             @return inserted rowid as int
             @warning: commit needed
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute(
                 "INSERT INTO tracks (name, uri, duration, tracknumber,\
                 discnumber, discname, album_id,\
-                year, popularity, rate, ltime, mtime) VALUES\
-                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (
+                year, popularity, rate, ltime, mtime, mb_track_id) VALUES\
+                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (
                                                         name,
                                                         uri,
                                                         duration,
@@ -68,7 +69,8 @@ class TracksDatabase:
                                                         popularity,
                                                         rate,
                                                         ltime,
-                                                        mtime))
+                                                        mtime,
+                                                        mb_track_id))
             return result.lastrowid
 
     def add_artist(self, track_id, artist_id):
@@ -78,7 +80,7 @@ class TracksDatabase:
             @param artist id as int
             @warning: commit needed
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             artists = self.get_artist_ids(track_id)
             if artist_id not in artists:
                 sql.execute("INSERT INTO "
@@ -92,7 +94,7 @@ class TracksDatabase:
             @param genre id as int
             @warning: commit needed
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             genres = self.get_genre_ids(track_id)
             if genre_id not in genres:
                 sql.execute("INSERT INTO\
@@ -105,7 +107,7 @@ class TracksDatabase:
             Return all internal track ids
             @return track ids as [int]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT rowid FROM tracks")
             return list(itertools.chain(*result))
 
@@ -115,7 +117,7 @@ class TracksDatabase:
             @param name as str
             @return track id as [int]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT rowid\
                                   FROM tracks WHERE name=?\
                                   COLLATE NOCASE COLLATE LOCALIZED",
@@ -128,7 +130,7 @@ class TracksDatabase:
             @param uri as str
             @return track id as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT rowid FROM tracks WHERE uri=?",
                                  (uri,))
             v = result.fetchone()
@@ -143,7 +145,7 @@ class TracksDatabase:
             @param album id as int
             @return track id as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             filters = (name, album_id) + tuple(artist_ids)
             request = "SELECT tracks.rowid FROM tracks\
                        WHERE name = ? COLLATE NOCASE\
@@ -168,7 +170,7 @@ class TracksDatabase:
             @param Track id as int
             @return Name as string
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT name FROM tracks WHERE rowid=?",
                                  (track_id,))
             v = result.fetchone()
@@ -182,7 +184,7 @@ class TracksDatabase:
             @param track id as int
             @return track year as string
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT year FROM tracks WHERE rowid=?",
                                  (track_id,))
             v = result.fetchone()
@@ -197,7 +199,7 @@ class TracksDatabase:
             @param album id as int
             @return int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT year, COUNT(year) AS occurrence\
                                   FROM tracks\
                                   WHERE tracks.album_id=?\
@@ -215,7 +217,7 @@ class TracksDatabase:
             @param track id as int
             @return rate as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT rate FROM tracks WHERE rowid=?",
                                  (track_id,))
             v = result.fetchone()
@@ -229,7 +231,7 @@ class TracksDatabase:
             @param Track id as int
             @return uri as string
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT uri FROM tracks WHERE rowid=?",
                                  (track_id,))
             v = result.fetchone()
@@ -243,11 +245,10 @@ class TracksDatabase:
             @param Track id as int
             @param uri as string
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             sql.execute("UPDATE tracks SET uri=?\
                          WHERE rowid=?",
                         (uri, track_id))
-            sql.commit()
             if uri.startswith("http") or uri.startswith("https"):
                 self.set_duration(track_id, 0)
 
@@ -257,11 +258,10 @@ class TracksDatabase:
             @param Track id as int
             @param rate as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             sql.execute("UPDATE tracks SET rate=?\
                          WHERE rowid=?",
                         (rate, track_id))
-            sql.commit()
 
     def get_album_id(self, track_id):
         """
@@ -269,7 +269,7 @@ class TracksDatabase:
             @param track id as int
             @return album id as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT album_id FROM tracks WHERE rowid=?",
                                  (track_id,))
             v = result.fetchone()
@@ -277,13 +277,27 @@ class TracksDatabase:
                 return v[0]
             return -1
 
+    def get_mb_track_id(self, track_id):
+        """
+            Get MusicBrainz recording id for track id
+            @param track id as int
+            @return recording id as int
+        """
+        with SqlCursor(App().db) as sql:
+            result = sql.execute("SELECT mb_track_id FROM tracks\
+                                  WHERE rowid=?", (track_id,))
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return -1
+
     def get_album_name(self, track_id):
         """
             Get album name for track id
             @param track id as int
             @return album name as str
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT albums.name from albums,tracks\
                                   WHERE tracks.rowid=? AND\
                                   tracks.album_id=albums.rowid", (track_id,))
@@ -298,7 +312,7 @@ class TracksDatabase:
             @param track id as int
             @return artist ids as [int]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT artist_id FROM track_artists\
                                   WHERE track_id=?", (track_id,))
             return list(itertools.chain(*result))
@@ -309,7 +323,7 @@ class TracksDatabase:
             @param track id as int
             @return artists as [str]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT name FROM artists, track_artists\
                                   WHERE track_artists.track_id=?\
                                   AND track_artists.artist_id=artists.rowid",
@@ -322,7 +336,7 @@ class TracksDatabase:
             @param track id as int
             @return genre ids as [int]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT genre_id FROM track_genres\
                                   WHERE track_id=?", (track_id,))
             return list(itertools.chain(*result))
@@ -333,7 +347,7 @@ class TracksDatabase:
             @param track id as int
             @return [str]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT name FROM genres, track_genres\
                                   WHERE track_genres.track_id=?\
                                   AND track_genres.genre_id=genres.rowid",
@@ -347,7 +361,7 @@ class TracksDatabase:
             @param None
             @return dict of {uri as string: mtime as int}
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             mtimes = {}
             result = sql.execute("SELECT DISTINCT uri, mtime\
                                   FROM tracks")
@@ -361,7 +375,7 @@ class TracksDatabase:
             @param exclude as [str]
             @return Array of uri as string
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             filters = ()
             for e in exclude:
                 filters += ("%" + e + "%",)
@@ -378,7 +392,7 @@ class TracksDatabase:
             @param track id as int
             @return position as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT tracknumber FROM tracks\
                                   WHERE rowid=?", (track_id,))
             v = result.fetchone()
@@ -392,7 +406,7 @@ class TracksDatabase:
             @param Track id as int
             @return duration as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT duration FROM tracks\
                                   WHERE rowid=?", (track_id,))
             v = result.fetchone()
@@ -406,31 +420,30 @@ class TracksDatabase:
             @param Track id as int
             @param duration as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             sql.execute("UPDATE tracks\
                          SET duration=?\
                          WHERE rowid=?", (duration, track_id,))
-            sql.commit()
 
     def is_empty(self):
         """
             Return True if no tracks in db
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT COUNT(1) FROM tracks  LIMIT 1")
             v = result.fetchone()
             if v is not None:
                 return v[0] == 0
             return True
 
-    def get_as_non_album_artist(self, artist_id):
+    def get_for_artist(self, artist_id):
         """
-            Get tracks for artist_id where artist_id isn"t main artist
+            Get tracks for artist_id where artist_id isn't main artist
             @param artist id as int
             @return list of [tracks id as int, track name as string]
         """
-        with SqlCursor(Lp().db) as sql:
-            result = sql.execute("SELECT tracks.rowid, tracks.name\
+        with SqlCursor(App().db) as sql:
+            result = sql.execute("SELECT tracks.rowid\
                                  FROM tracks, track_artists, album_artists\
                                  WHERE album_artists.album_id=tracks.album_id\
                                  AND track_artists.artist_id=?\
@@ -441,7 +454,7 @@ class TracksDatabase:
                                   WHERE artist_id=track_artists.artist_id\
                                   AND album_id=tracks.album_id)",
                                  (artist_id,))
-            return list(result)
+            return list(itertools.chain(*result))
 
     def get_rated(self, limit=100):
         """
@@ -449,7 +462,7 @@ class TracksDatabase:
             @param limit as int
             @return tracks as [int]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT rowid FROM tracks\
                                   WHERE rate >= 4\
                                   ORDER BY popularity DESC LIMIT ?",
@@ -462,7 +475,7 @@ class TracksDatabase:
             @param limit as int
             @return tracks as [int]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT rowid FROM tracks\
                                   WHERE popularity!=0\
                                   ORDER BY popularity DESC LIMIT ?",
@@ -474,7 +487,7 @@ class TracksDatabase:
             Get higher available popularity
             @return int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT popularity\
                                   FROM tracks\
                                   ORDER BY POPULARITY DESC LIMIT 1")
@@ -488,7 +501,7 @@ class TracksDatabase:
             Return avarage popularity
             @return avarage popularity as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT AVG(popularity)\
                                   FROM (SELECT popularity\
                                         FROM tracks\
@@ -504,7 +517,7 @@ class TracksDatabase:
             @param track id as int
             @raise sqlite3.OperationalError on db update
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT popularity from tracks WHERE rowid=?",
                                  (track_id,))
             pop = result.fetchone()
@@ -515,7 +528,6 @@ class TracksDatabase:
             current += 1
             sql.execute("UPDATE tracks set popularity=? WHERE rowid=?",
                         (current, track_id))
-            sql.commit()
 
     def set_listened_at(self, track_id, time):
         """
@@ -523,17 +535,16 @@ class TracksDatabase:
             @param track id as int
             @param time as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             sql.execute("UPDATE tracks set ltime=? WHERE rowid=?",
                         (time, track_id))
-            sql.commit()
 
     def get_never_listened_to(self):
         """
             Return random tracks never listened to
             @return tracks as [int]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT tracks.rowid\
                                   FROM tracks\
                                   WHERE ltime=0\
@@ -545,7 +556,7 @@ class TracksDatabase:
             Return tracks listened recently
             @return tracks as [int]
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT tracks.rowid\
                                   FROM tracks\
                                   WHERE ltime!=0\
@@ -558,7 +569,7 @@ class TracksDatabase:
             @param track id as int
             @return int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT rowid FROM tracks\
                                   WHERE persistent=?", (track_id,))
             v = result.fetchone()
@@ -572,48 +583,42 @@ class TracksDatabase:
             @param track id as int
             @param persistent as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             sql.execute("UPDATE tracks\
                          SET persistent=?\
                          WHERE rowid=?", (persistent, track_id,))
-            sql.commit()
 
     def get_randoms(self):
         """
             Return random tracks
             @return array of track ids as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT tracks.rowid\
                                   FROM tracks\
                                   ORDER BY random() LIMIT 100")
             return list(itertools.chain(*result))
 
-    def set_popularity(self, track_id, popularity, commit=False):
+    def set_popularity(self, track_id, popularity):
         """
             Set popularity
             @param track id as int
             @param popularity as int
-            @warning: commit needed
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             try:
                 sql.execute("UPDATE tracks set popularity=? WHERE rowid=?",
                             (popularity, track_id))
-                if commit:
-                    sql.commit()
             except:  # Database is locked
                 pass
 
     def get_popularity(self, track_id):
-        """ème a déjà été remonté en début de semaine et semble ancien.
-Nous traitons le problème dès que possible (salle très occupée).
-
+        """
             Get popularity
             @param track id  as int
             @return popularity as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT popularity FROM tracks WHERE\
                                  rowid=?", (track_id,))
             v = result.fetchone()
@@ -627,7 +632,7 @@ Nous traitons le problème dès que possible (salle très occupée).
             @param track id  as int
             @return listen time as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT ltime FROM tracks WHERE\
                                  rowid=?", (track_id,))
             v = result.fetchone()
@@ -641,7 +646,7 @@ Nous traitons le problème dès que possible (salle très occupée).
             @param track_id as int
             @return modification time as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             request = "SELECT mtime FROM tracks\
                        WHERE tracks.rowid=?"
             result = sql.execute(request, (track_id,))
@@ -655,7 +660,7 @@ Nous traitons le problème dès que possible (salle très occupée).
             Count tracks
             @return int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT COUNT(1) FROM tracks")
             v = result.fetchone()
             if v is not None:
@@ -668,7 +673,7 @@ Nous traitons le problème dès que possible (salle très occupée).
             @param track_id as int
             @warning commit needed
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             sql.execute("DELETE FROM track_artists\
                          WHERE track_id = ?", (track_id,))
             sql.execute("DELETE FROM track_genres\
@@ -680,12 +685,12 @@ Nous traitons le problème dès que possible (salle très occupée).
             @param searched as string
             return: list of [id as int, name as string]
         """
-        with SqlCursor(Lp().db) as sql:
-            result = sql.execute("SELECT tracks.rowid, tracks.name\
+        with SqlCursor(App().db) as sql:
+            result = sql.execute("SELECT tracks.rowid\
                                   FROM tracks\
                                   WHERE noaccents(name) LIKE ? LIMIT 25",
                                  ("%" + noaccents(searched) + "%",))
-            return list(result)
+            return list(itertools.chain(*result))
 
     def search_track(self, artist, title):
         """
@@ -698,13 +703,13 @@ Nous traitons le problème dès que possible (salle très occupée).
         """
         track_ids = self.get_ids_for_name(title)
         for track_id in track_ids:
-            album_id = Lp().tracks.get_album_id(track_id)
-            artist_ids = set(Lp().albums.get_artist_ids(album_id)) &\
-                set(Lp().tracks.get_artist_ids(track_id))
+            album_id = App().tracks.get_album_id(track_id)
+            artist_ids = set(App().albums.get_artist_ids(album_id)) &\
+                set(App().tracks.get_artist_ids(track_id))
             for artist_id in artist_ids:
-                if artist == Lp().artists.get_name(artist_id):
+                if artist == App().artists.get_name(artist_id):
                     return track_id
-            if ", ".join(Lp().tracks.get_artists(track_id)) == artist:
+            if ", ".join(App().tracks.get_artists(track_id)) == artist:
                 return track_id
         return None
 
@@ -713,7 +718,7 @@ Nous traitons le problème dès que possible (salle très occupée).
             Remove track
             @param track id as int
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(App().db) as sql:
             sql.execute("DELETE FROM track_genres\
                          WHERE track_id=?", (track_id,))
             sql.execute("DELETE FROM track_artists\
