@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -19,7 +19,7 @@ from lollypop.sqlcursor import SqlCursor
 from lollypop.utils import translate_artist_name
 from lollypop.database_history import History
 from lollypop.radios import Radios
-from lollypop.define import Lp
+from lollypop.define import App, Type
 
 
 class DatabaseUpgrade:
@@ -27,15 +27,13 @@ class DatabaseUpgrade:
         Manage database schema upgrades
     """
 
-    def __init__(self, version):
+    def __init__(self):
         """
             Init object
-            @param version as int
         """
-        self._version = version
         # Here are schema upgrade, key is database version,
         # value is sql request
-        self._UPGRADES = {
+        self.__UPGRADES = {
             1: "UPDATE tracks SET duration=CAST(duration as INTEGER);",
             2: "UPDATE albums SET artist_id=-2001 where artist_id=-999;",
             3: self.__upgrade_3,
@@ -59,40 +57,57 @@ class DatabaseUpgrade:
             21: self.__upgrade_21,
             22: self.__upgrade_22,
             23: self.__upgrade_23,
-                         }
+            24: "ALTER TABLE albums ADD album_id TEXT",
+            25: "ALTER TABLE tracks ADD mb_track_id TEXT",
+            26: self.__upgrade_26,
+            27: "UPDATE tracks SET duration=CAST(duration AS INT)",
+            28: self.__upgrade_28,
+        }
 
-    """
-        Return upgrade count
-        @return int
-    """
-    def count(self):
-        return len(self._UPGRADES)
+    def upgrade(self, db):
+        """
+            Upgrade db
+            @param db as Database
+        """
+        # Migration from gsettings
+        gsettings_version = App().settings.get_value("db-version").get_int32()
+        if gsettings_version != -1:
+            with SqlCursor(db) as sql:
+                sql.execute("PRAGMA user_version=%s" % gsettings_version)
+                App().settings.set_value("db-version",
+                                         GLib.Variant("i", -1))
+        version = 0
+        with SqlCursor(db) as sql:
+            result = sql.execute("PRAGMA user_version")
+            v = result.fetchone()
+            if v is not None:
+                version = v[0]
+            if version < self.version:
+                for i in range(version+1, self.version + 1):
+                    try:
+                        if isinstance(self.__UPGRADES[i], str):
+                            sql.execute(self.__UPGRADES[i])
+                        else:
+                            self.__UPGRADES[i](db)
+                    except Exception as e:
+                        print("History DB upgrade %s failed: %s" % (i, e))
+                sql.execute("PRAGMA user_version=%s" % self.version)
 
-    """
-        Upgrade database based on version
-        @return new db version as int
-    """
-    def do_db_upgrade(self):
-        with SqlCursor(Lp().db) as sql:
-            for i in range(self._version+1, len(self._UPGRADES)+1):
-                try:
-                    if isinstance(self._UPGRADES[i], str):
-                        sql.execute(self._UPGRADES[i])
-                        sql.commit()
-                    else:
-                        self._UPGRADES[i]()
-                except Exception as e:
-                    print("Database upgrade failed: ", e)
-            return len(self._UPGRADES)
+    @property
+    def version(self):
+        """
+            Current wanted version
+        """
+        return len(self.__UPGRADES)
 
 #######################
 # PRIVATE             #
 #######################
-    def __upgrade_3(self):
+    def __upgrade_3(self, db):
         """
             Add a sorted field to artists
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(db) as sql:
             sql.execute("ALTER TABLE artists ADD sortname TEXT")
             result = sql.execute("SELECT DISTINCT artists.rowid,\
                                   artists.name\
@@ -103,13 +118,12 @@ class DatabaseUpgrade:
                             (translated, row[0]))
                 sql.execute("UPDATE artists SET sortname=? WHERE rowid=?",
                             (row[1], row[0]))
-            sql.commit()
 
-    def __upgrade_4(self):
+    def __upgrade_4(self, db):
         """
             Add album artists table
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(db) as sql:
             sql.execute("CREATE TABLE album_artists (\
                                                 album_id INT NOT NULL,\
                                                 artist_id INT NOT NULL)")
@@ -158,13 +172,12 @@ class DatabaseUpgrade:
                                popularity,\
                                mtime FROM backup")
             sql.execute("DROP TABLE backup")
-            sql.commit()
 
-    def __upgrade_13(self):
+    def __upgrade_13(self, db):
         """
             Convert tracks filepath column to uri
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(db) as sql:
             sql.execute("ALTER TABLE tracks RENAME TO tmp_tracks")
             sql.execute("""CREATE TABLE tracks (id INTEGER PRIMARY KEY,
                                               name TEXT NOT NULL,
@@ -200,8 +213,7 @@ class DatabaseUpgrade:
                         uri = GLib.filename_to_uri(uri)
                         sql.execute("UPDATE tracks set uri=? WHERE rowid=?",
                                     (uri, track_id))
-            sql.commit()
-        with SqlCursor(Lp().playlists) as sql:
+        with SqlCursor(App().playlists) as sql:
             sql.execute("ALTER TABLE tracks RENAME TO tmp_tracks")
             sql.execute("""CREATE TABLE tracks (playlist_id INT NOT NULL,
                                                 uri TEXT NOT NULL)""")
@@ -214,32 +226,31 @@ class DatabaseUpgrade:
                     uri = GLib.filename_to_uri(path)
                     sql.execute("UPDATE tracks set uri=? WHERE uri=?",
                                 (uri, path))
-            sql.commit()
 
-    def __upgrade_15(self):
+    def __upgrade_15(self, db):
         """
             Fix broken 0.9.208 release
         """
-        if Lp().notify:
-            Lp().notify.send("Please wait while upgrading db...")
-        with SqlCursor(Lp().db) as sql:
+        if App().notify:
+            App().notify.send("Please wait while upgrading db...")
+        with SqlCursor(db) as sql:
             result = sql.execute("SELECT tracks.rowid FROM tracks\
                                   WHERE NOT EXISTS (\
                                                  SELECT track_id\
                                                  FROM track_genres\
                                                  WHERE track_id=tracks.rowid)")
-            Lp().db.del_tracks(list(itertools.chain(*result)))
+            db.del_tracks(list(itertools.chain(*result)))
 
-    def __upgrade_16(self):
+    def __upgrade_16(self, db):
         """
             Get ride of paths
         """
-        paths = Lp().settings.get_value("music-path")
+        paths = App().settings.get_value("music-path")
         uris = []
         for path in paths:
             uris.append(GLib.filename_to_uri(path))
-        Lp().settings.set_value("music-uris", GLib.Variant("as", uris))
-        with SqlCursor(Lp().db) as sql:
+        App().settings.set_value("music-uris", GLib.Variant("as", uris))
+        with SqlCursor(db) as sql:
             sql.execute("ALTER TABLE albums RENAME TO tmp_albums")
             sql.execute("""CREATE TABLE albums (
                                               id INTEGER PRIMARY KEY,
@@ -263,18 +274,16 @@ class DatabaseUpgrade:
                     uri = GLib.filename_to_uri(uri)
                     sql.execute("UPDATE albums set uri=? WHERE rowid=?",
                                 (uri, rowid))
-            sql.commit()
 
-    def __upgrade_18(self):
+    def __upgrade_18(self, db):
         """
             Upgrade history
         """
         with SqlCursor(History()) as sql:
             sql.execute("ALTER TABLE history ADD loved_album\
                         INT NOT NULL DEFAULT 0")
-            sql.commit()
 
-    def __upgrade_19(self):
+    def __upgrade_19(self, db):
         """
             Upgrade history
         """
@@ -284,22 +293,20 @@ class DatabaseUpgrade:
                             INT NOT NULL DEFAULT -1")
                 sql.execute("ALTER TABLE history ADD rate\
                             INT NOT NULL DEFAULT -1")
-                sql.commit()
             except:
                 pass  # May fails if History was non existent
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(db) as sql:
             sql.execute("ALTER TABLE tracks ADD rate\
                         INT NOT NULL DEFAULT -1")
             sql.execute("ALTER TABLE albums ADD rate\
                         INT NOT NULL DEFAULT -1")
-            sql.commit()
 
-    def __upgrade_20(self):
+    def __upgrade_20(self, db):
         """
             Add mtimes tables
         """
         mtime = int(time())
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(db) as sql:
             sql.execute("ALTER TABLE album_genres\
                          ADD mtime INT NOT NULL DEFAULT %s" % mtime)
             sql.execute("ALTER TABLE track_genres\
@@ -406,28 +413,26 @@ class DatabaseUpgrade:
                                    ltime,\
                                    persistent FROM backup")
             sql.execute("DROP TABLE backup")
-            sql.commit()
 
-    def __upgrade_21(self):
+    def __upgrade_21(self, db):
         """
             Add rate to radios
         """
         with SqlCursor(Radios()) as sql:
             sql.execute("ALTER TABLE radios ADD rate\
                          INT NOT NULL DEFAULT -1")
-            sql.commit()
 
-    def __upgrade_22(self):
+    def __upgrade_22(self, db):
         """
             Remove Charts/Web entries
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(db) as sql:
             result = sql.execute("SELECT rowid FROM tracks\
                                   WHERE persistent=0 OR\
                                   persistent=2 OR\
                                   persistent=3")
             track_ids = list(itertools.chain(*result))
-            Lp().db.del_tracks(track_ids)
+            db.del_tracks(track_ids)
             # Remove persistent from tracks table
             sql.execute("CREATE TEMPORARY TABLE backup(\
                                           id INTEGER PRIMARY KEY,\
@@ -483,13 +488,12 @@ class DatabaseUpgrade:
                                    rate,\
                                    ltime FROM backup")
             sql.execute("DROP TABLE backup")
-            sql.commit()
 
-    def __upgrade_23(self):
+    def __upgrade_23(self, db):
         """
             Restore back mtime in tracks
         """
-        with SqlCursor(Lp().db) as sql:
+        with SqlCursor(db) as sql:
             sql.execute("ALTER TABLE tracks ADD mtime INT")
             sql.execute("ALTER TABLE albums ADD mtime INT")
 
@@ -500,7 +504,6 @@ class DatabaseUpgrade:
             sql.execute("UPDATE albums SET mtime = (\
                             SELECT mtime FROM album_genres\
                             WHERE album_genres.album_id=albums.rowid)")
-            sql.commit()
             # Remove mtime from album_genres table
             sql.execute("CREATE TABLE album_genres2 (\
                                                 album_id INT NOT NULL,\
@@ -520,4 +523,45 @@ class DatabaseUpgrade:
                                    genre_id FROM track_genres")
             sql.execute("DROP TABLE track_genres")
             sql.execute("ALTER TABLE track_genres2 RENAME TO track_genres")
-            sql.commit()
+
+    def __upgrade_26(self, db):
+        """
+            Rename album_id to mb_album_id in albums
+        """
+        with SqlCursor(db) as sql:
+            sql.execute("ALTER TABLE albums RENAME TO tmp_albums")
+            sql.execute("""CREATE TABLE albums (
+                               id INTEGER PRIMARY KEY,
+                               name TEXT NOT NULL,
+                               mb_album_id TEXT,
+                               no_album_artist BOOLEAN NOT NULL,
+                               year INT,
+                               uri TEXT NOT NULL,
+                               popularity INT NOT NULL,
+                               rate INT NOT NULL,
+                               loved INT NOT NULL,
+                               mtime INT NOT NULL,
+                               synced INT NOT NULL)""")
+
+            sql.execute("""INSERT INTO albums (id, name, mb_album_id,
+                            no_album_artist, year, uri, popularity, rate,
+                            loved, mtime, synced) SELECT id, name, album_id,
+                            no_album_artist, year, uri, popularity, rate,
+                            loved, mtime, synced FROM tmp_albums""")
+            sql.execute("DROP TABLE tmp_albums")
+
+    def __upgrade_28(self, db):
+        """
+            Upgrade setting based on db
+            https://gitlab.gnome.org/gnumdk/lollypop/issues/1368
+        """
+        with SqlCursor(db) as sql:
+            result = sql.execute("SELECT albums.rowid\
+                                  FROM albums, album_artists\
+                                  WHERE album_artists.artist_id=?\
+                                  AND album_artists.album_id=albums.rowid\
+                                  LIMIT 1",
+                                 (Type.COMPILATIONS,))
+            if list(itertools.chain(*result)):
+                App().settings.set_value("show-compilations",
+                                         GLib.Variant("b", True))
