@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -14,8 +14,8 @@ from gi.repository import Gtk, Gdk, GLib, Gio, Gst
 
 from datetime import datetime
 
-from lollypop.define import Lp, ArtSize, Type
-from lollypop.pop_next import NextPopover
+from lollypop.define import App, ArtSize, ResponsiveType
+from lollypop.view_albums_list import AlbumsListView
 from lollypop.controllers import InfoController, PlaybackController
 from lollypop.controllers import ProgressController
 
@@ -37,10 +37,8 @@ class FullScreen(Gtk.Window, InfoController,
         PlaybackController.__init__(self)
         ProgressController.__init__(self)
         self.set_application(app)
-        self.__timeout1 = None
-        self.__timeout2 = None
-        self.__signal1_id = None
-        self.__signal2_id = None
+        self.__timeout1 = self.__timeout2 = None
+        self.__signal1_id = self.__signal2_id = self.__signal3_id = None
         self.set_decorated(False)
         self.__parent = parent
 
@@ -57,50 +55,56 @@ class FullScreen(Gtk.Window, InfoController,
             artsize = int(ArtSize.FULLSCREEN*geometry.height/1080)
         else:
             artsize = int(ArtSize.FULLSCREEN*geometry.width/1920)
-        InfoController.__init__(self, artsize)
+        self.__font_size = int(14 * geometry.height / 1080)
+        InfoController.__init__(self, artsize, self.__font_size)
         widget = builder.get_object("widget")
+        grid = builder.get_object("grid")
         self._play_btn = builder.get_object("play_btn")
         self._next_btn = builder.get_object("next_btn")
         self._prev_btn = builder.get_object("prev_btn")
-        self._next_popover = NextPopover()
-        self._next_popover.set_position(Gtk.PositionType.RIGHT)
         self._play_image = builder.get_object("play_image")
         self._pause_image = builder.get_object("pause_image")
         close_btn = builder.get_object("close_btn")
         preferences = Gio.Settings.new("org.gnome.desktop.wm.preferences")
         layout = preferences.get_value("button-layout").get_string()
         if layout.split(":")[0] == "close":
-            widget.attach(close_btn, 0, 0, 1, 1)
+            grid.attach(close_btn, 0, 0, 1, 1)
             close_btn.set_property("halign", Gtk.Align.START)
         else:
-            widget.attach(close_btn, 2, 0, 1, 1)
+            grid.attach(close_btn, 2, 0, 1, 1)
             close_btn.set_property("halign", Gtk.Align.END)
         self._cover = builder.get_object("cover")
         self._title_label = builder.get_object("title")
         self._artist_label = builder.get_object("artist")
         self._album_label = builder.get_object("album")
-
+        self.__revealer = builder.get_object("reveal")
         self._datetime = builder.get_object("datetime")
-
         self._progress = builder.get_object("progress_scale")
         self._timelabel = builder.get_object("playback")
         self._total_time_label = builder.get_object("duration")
         self.connect("key-release-event", self.__on_key_release_event)
+
+        # Add an AlbumListView on the right
+        self.__view = AlbumsListView(ResponsiveType.DND)
+        self.__view.get_style_context().add_class("background-opacity")
+        self.__view.show()
+        self.__revealer.add(self.__view)
         self.add(widget)
 
     def do_show(self):
         """
             Init signals, set color and go party mode if nothing is playing
         """
-        self.__signal1_id = Lp().player.connect("current-changed",
-                                                self.on_current_changed)
-        self.__signal2_id = Lp().player.connect("status-changed",
-                                                self.on_status_changed)
-        if Lp().player.current_track.id is None:
-            Lp().player.set_party(True)
+        self.__signal1_id = App().player.connect("current-changed",
+                                                 self.on_current_changed)
+        self.__signal2_id = App().player.connect("status-changed",
+                                                 self.on_status_changed)
+        if App().player.current_track.id is None:
+            App().player.set_party(True)
         else:
-            self.on_status_changed(Lp().player)
-            self.on_current_changed(Lp().player)
+            self.on_status_changed(App().player)
+            self.on_current_changed(App().player)
+        self.__view.populate(App().player.albums)
         if self.__timeout1 is None:
             self.__timeout1 = GLib.timeout_add(1000, self._update_position)
         Gtk.Window.do_show(self)
@@ -116,38 +120,39 @@ class FullScreen(Gtk.Window, InfoController,
             self.__timeout2 = GLib.timeout_add(1000,
                                                self.__update_datetime,
                                                show_seconds)
-        self._update_position(Lp().player.position/Gst.SECOND)
+        self._update_position(App().player.position/Gst.SECOND)
         screen = Gdk.Screen.get_default()
         monitor = screen.get_monitor_at_window(self.__parent.get_window())
         self.fullscreen_on_monitor(screen, monitor)
-        self._next_popover.set_relative_to(self._next_btn)
-        if Lp().player.next_track.id != Type.RADIOS:
-            self._next_popover.show()
 
         # Disable screensaver (idle)
-        Lp().inhibitor.manual_inhibit(suspend=False, idle=True)
+        App().inhibitor.manual_inhibit(suspend=False, idle=True)
+        self.__signal3_id = App().player.connect("party-changed",
+                                                 self.__on_party_changed)
 
     def do_hide(self):
         """
             Remove signals and unset color
         """
+        self.__view.stop()
         self.__parent.set_skip_pager_hint(False)
         self.__parent.set_skip_taskbar_hint(False)
         Gtk.Window.do_hide(self)
         if self.__signal1_id is not None:
-            Lp().player.disconnect(self.__signal1_id)
+            App().player.disconnect(self.__signal1_id)
             self.__signal1_id = None
         if self.__signal2_id is not None:
-            Lp().player.disconnect(self.__signal2_id)
+            App().player.disconnect(self.__signal2_id)
             self.__signal2_id = None
+        if self.__signal3_id is not None:
+            App().player.disconnect(self.__signal3_id)
+            self.__signal3_id = None
         if self.__timeout1 is not None:
             GLib.source_remove(self.__timeout1)
             self.__timeout1 = None
         if self.__timeout2 is not None:
             GLib.source_remove(self.__timeout2)
-        self._next_popover.set_relative_to(None)
-        self._next_popover.hide()
-        Lp().inhibitor.manual_uninhibit()
+        App().inhibitor.manual_uninhibit()
 
     def show_hide_volume_control(self):
         """
@@ -167,14 +172,10 @@ class FullScreen(Gtk.Window, InfoController,
             album_name = player.current_track.album.name
             if player.current_track.year:
                 album_name += " (%s)" % player.current_track.year
-            self._album_label.set_text(album_name)
-        # Do not show next popover non internal tracks as
-        # tags will be readed on the fly
-        if player.next_track.id is not None and player.next_track.id >= 0:
-            self._next_popover.update()
-            self._next_popover.show()
-        else:
-            self._next_popover.hide()
+            self._album_label.set_markup(
+                                        "<span font='%s'>%s</span>" %
+                                        (self.__font_size - 1,
+                                         GLib.markup_escape_text(album_name)))
 
 #######################
 # PROTECTED           #
@@ -184,8 +185,18 @@ class FullScreen(Gtk.Window, InfoController,
             Destroy self
             @param widget as Gtk.Button
         """
+        self.__view.stop()
         self.destroy()
 
+    def _on_motion_notify_event(self, widget, event):
+        """
+            Show/Hide track list if mouse on the right
+            @param widget as Gtk.Widget
+            @param event as Gdk.Event
+        """
+        reveal = event.x_root > widget.get_allocated_width() / 2
+        self.__revealer.set_reveal_child(reveal)
+
 #######################
 # PRIVATE             #
 #######################
@@ -204,6 +215,14 @@ class FullScreen(Gtk.Window, InfoController,
             return False
         return True
 
+    def __on_party_changed(self, player, party):
+        """
+            Populate view again
+            @param player as Player
+            @param party as bool
+        """
+        self.__view.populate(player.albums)
+
     def __on_key_release_event(self, widget, event):
         """
             Destroy window if Esc
