@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
 
 from gi.repository import Gio, GLib
 
-from lollypop.define import Lp
+from lollypop.define import App
 from lollypop.utils import is_audio
 
 
@@ -53,12 +53,17 @@ class Inotify:
 #######################
     def __on_dir_changed(self, monitor, changed_file, other_file, event):
         """
-            Prepare thread to handle changes
+            Stop collection scanner if running
+            Delayed update by default
+            @param monitor as Gio.FileMonitor
+            @param changed_file as Gio.File/None
+            @param other_file as Gio.File/None
+            @param event as Gio.FileMonitorEvent
         """
         update = False
         # Stop collection scanner and wait
-        if Lp().scanner.is_locked():
-            Lp().scanner.stop()
+        if App().scanner.is_locked():
+            App().scanner.stop()
             GLib.timeout_add(self.__TIMEOUT,
                              self.__on_dir_changed,
                              monitor,
@@ -92,4 +97,4 @@ class Inotify:
             Run a collection update
         """
         self.__timeout = None
-        Lp().window.update_db()
+        App().scanner.update()
