@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -23,12 +23,6 @@ except Exception as e:
     print(_("Last.fm authentication disabled"))
     Secret = None
 
-try:
-    gi.require_version("Goa", "1.0")
-    from gi.repository import Goa
-except:
-    pass
-
 from pylast import LastFMNetwork, LibreFMNetwork, md5, WSError
 from pylast import SessionKeyGenerator
 from gettext import gettext as _
@@ -36,9 +30,10 @@ from locale import getdefaultlocale
 import re
 
 from lollypop.helper_task import TaskHelper
-from lollypop.define import Lp, Type
+from lollypop.define import App, Type
 from lollypop.objects import Track
-from lollypop.utils import debug, get_network_available
+from lollypop.utils import debug
+from lollypop.goa import GoaSyncedAccount
 
 
 class LastFM(LastFMNetwork, LibreFMNetwork):
@@ -60,17 +55,19 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
         """
         self.__name = name
         self.__login = ""
-        self.__is_auth = False
+        self.session_key = ""
         self.__password = None
         self.__goa = None
-        self.__check_for_proxy()
         if name == "librefm":
             LibreFMNetwork.__init__(self)
         else:
-            self.__goa = self.__get_goa_oauth()
-            if self.__goa is not None:
-                self.__API_KEY = self.__goa.props.client_id
-                self.__API_SECRET = self.__goa.props.client_secret
+            self.__goa = GoaSyncedAccount("Last.fm")
+            self.__goa.connect("account-switched",
+                               self.on_goa_account_switched)
+            if self.is_goa:
+                auth = self.__goa.oauth2_based
+                self.__API_KEY = auth.props.client_id
+                self.__API_SECRET = auth.props.client_secret
             else:
                 self.__API_KEY = "7a9619a850ccf7377c46cf233c51e3c6"
                 self.__API_SECRET = "9254319364d73bec6c59ace485a95c98"
@@ -85,10 +82,10 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             @param full_sync as bool
             @param callback as function
         """
-        if self.__goa is not None:
+        if self.is_goa:
             helper = TaskHelper()
             helper.run(self.__connect, full_sync)
-        elif get_network_available():
+        elif Gio.NetworkMonitor.get_default().get_network_available():
             from lollypop.helper_passwords import PasswordsHelper
             helper = PasswordsHelper()
             helper.get(self.__name,
@@ -103,7 +100,7 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             @param artist as str
             @return (url as str, content as str)
         """
-        if not get_network_available():
+        if not Gio.NetworkMonitor.get_default().get_network_available():
             return (None, None, None)
         last_artist = self.get_artist(artist)
         try:
@@ -115,32 +112,36 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
         url = last_artist.get_cover_image(3)
         return (url, content.encode(encoding="UTF-8"))
 
-    def do_scrobble(self, artist, album, title, timestamp):
+    def listen(self, track, timestamp):
         """
-            Scrobble track
-            @param artist as str
-            @param title as str
-            @param album as str
+            Submit a listen for a track (scrobble)
+            @param track as Track
             @param timestamp as int
-            @param duration as int
         """
-        if get_network_available() and\
-           self.__is_auth and Secret is not None:
+        if Gio.NetworkMonitor.get_default().get_network_available() and\
+                self.available:
             helper = TaskHelper()
-            helper.run(self.__scrobble, artist, album, title, timestamp)
+            helper.run(self.__scrobble,
+                       ", ".join(track.artists),
+                       track.album_name,
+                       track.title,
+                       timestamp,
+                       track.mb_track_id)
 
-    def now_playing(self, artist, album, title, duration):
+    def playing_now(self, track):
         """
-            Now playing track
-            @param artist as str
-            @param title as str
-            @param album as str
-            @param duration as int
+            Submit a playing now notification for a track
+            @param track as Track
         """
-        if get_network_available() and\
-           self.__is_auth and Secret is not None:
+        if Gio.NetworkMonitor.get_default().get_network_available() and\
+                track.id > 0 and self.available:
             helper = TaskHelper()
-            helper.run(self.__now_playing, artist, album, title, duration)
+            helper.run(self.__now_playing,
+                       ", ".join(track.artists),
+                       track.album_name,
+                       track.title,
+                       int(track.duration),
+                       track.mb_track_id)
 
     def love(self, artist, title):
         """
@@ -150,8 +151,8 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             @thread safe
         """
         # Love the track on lastfm
-        if get_network_available() and\
-           self.__is_auth:
+        if Gio.NetworkMonitor.get_default().get_network_available() and\
+                self.available:
             track = self.get_track(artist, title)
             try:
                 track.love()
@@ -166,8 +167,8 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             @thread safe
         """
         # Love the track on lastfm
-        if get_network_available() and\
-           self.__is_auth:
+        if Gio.NetworkMonitor.get_default().get_network_available() and\
+                self.available:
             track = self.get_track(artist, title)
             try:
                 track.unlove()
@@ -189,72 +190,71 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             pass
         return artists
 
+    def on_goa_account_switched(self, obj):
+        """
+            Callback for GoaSyncedAccount signal "account-switched"
+            @param obj as GoaSyncedAccount
+        """
+        self.connect()
+
+    def set_loved(self, track, loved):
+        """
+            Add or remove track from loved playlist on Last.fm
+            @param track as Track
+            @param loved Add to loved playlist if `True`; remove if `False`
+        """
+        if Gio.NetworkMonitor.get_default().get_network_available() and\
+                self.available:
+            if loved:
+                self.love(",".join(track.artists), track.name)
+            else:
+                self.unlove(",".join(track.artists), track.name)
+
     @property
-    def is_auth(self):
+    def can_love(self):
         """
-            True if valid authentication send
+            True if engine can love
+            @return bool
         """
-        return self.__is_auth
+        return True
 
     @property
     def is_goa(self):
         """
             True if using Gnome Online Account
         """
-        return self.__goa is not None
+        return self.__goa is not None and self.__goa.has_account
+
+    @property
+    def available(self):
+        """
+            Return True if Last.fm/Libre.fm submission is available
+            @return bool
+        """
+        if not self.session_key:
+            return False
+        if self.is_goa:
+            music_disabled = self.__goa.account.props.music_disabled
+            debug("Last.fm GOA scrobbling disabled: %s" % music_disabled)
+            return not music_disabled
+        return True
 
 #######################
 # PRIVATE             #
 #######################
-    def __get_goa_oauth(self):
-        """
-            Init Gnome Online Account
-            @return get_oauth2_based()/None
-        """
-        try:
-            c = Goa.Client.new_sync()
-            for proxy in c.get_accounts():
-                if proxy.get_account().props.provider_name == "Last.fm":
-                    return proxy.get_oauth2_based()
-        except:
-            pass
-        return None
-
-    def __check_for_proxy(self):
-        """
-            Enable proxy if needed
-        """
-        try:
-            proxy = Gio.Settings.new("org.gnome.system.proxy")
-            https = Gio.Settings.new("org.gnome.system.proxy.https")
-            mode = proxy.get_value("mode").get_string()
-            if mode != "none":
-                h = https.get_value("host").get_string()
-                p = https.get_value("port").get_int32()
-                if h != "" and p != 0:
-                    self.enable_proxy(host=h, port=p)
-            else:
-                self.disable_proxy()
-        except:
-            pass
-
     def __connect(self, full_sync=False):
         """
             Connect service
             @param full_sync as bool
             @thread safe
         """
-        if self.__goa is not None or (self.__password != "" and
-                                      self.__login != ""):
-            self.__is_auth = True
-        else:
-            self.__is_auth = False
         try:
             self.session_key = ""
-            self.__check_for_proxy()
-            if self.__goa is not None:
-                self.session_key = self.__goa.call_get_access_token_sync(
-                                                                      None)[0]
+            if self.is_goa:
+                auth = self.__goa.oauth2_based
+                self.api_key = auth.props.client_id
+                self.api_secret = auth.props.client_secret
+                self.session_key = auth.call_get_access_token_sync(None)[0]
             else:
                 skg = SessionKeyGenerator(self)
                 self.session_key = skg.get_session_key(
@@ -265,9 +265,9 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
                 helper.run(self.__populate_loved_tracks)
         except Exception as e:
             debug("LastFM::__connect(): %s" % e)
-            self.__is_auth = False
 
-    def __scrobble(self, artist, album, title, timestamp, first=True):
+    def __scrobble(self, artist, album, title, timestamp, mb_track_id,
+                   first=True):
         """
             Scrobble track
             @param artist as str
@@ -275,18 +275,21 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             @param album_title as str
             @param timestamp as int
             @param duration as int
+            @param mb_track_id as str
             @param first is internal
             @thread safe
         """
-        debug("LastFM::__scrobble(): %s, %s, %s, %s" % (artist,
-                                                        album,
-                                                        title,
-                                                        timestamp))
+        debug("LastFM::__scrobble(): %s, %s, %s, %s, %s" % (artist,
+                                                            album,
+                                                            title,
+                                                            timestamp,
+                                                            mb_track_id))
         try:
             self.scrobble(artist=artist,
                           album=album,
                           title=title,
-                          timestamp=timestamp)
+                          timestamp=timestamp,
+                          mbid=mb_track_id)
         except WSError:
             pass
         except Exception as e:
@@ -294,9 +297,11 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             # Scrobble sometimes fails
             if first:
                 self.__connect()
-                self.__scrobble(artist, album, title, timestamp, False)
+                self.__scrobble(artist, album, title, timestamp,
+                                mb_track_id, False)
 
-    def __now_playing(self, artist, album, title, duration, first=True):
+    def __now_playing(self, artist, album, title, duration, mb_track_id,
+                      first=True):
         """
             Now playing track
             @param artist as str
@@ -310,11 +315,10 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             self.update_now_playing(artist=artist,
                                     album=album,
                                     title=title,
-                                    duration=duration)
-            debug("LastFM::__now_playing(): %s, %s, %s, %s" % (artist,
-                                                               album,
-                                                               title,
-                                                               duration))
+                                    duration=duration,
+                                    mbid=mb_track_id)
+            debug("LastFM::__now_playing(): %s, %s, %s, %s, %s" % (
+                   artist, album, title, duration, mb_track_id))
         except WSError:
             pass
         except Exception as e:
@@ -322,24 +326,25 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             # now playing sometimes fails
             if first:
                 self.__connect()
-                self.__now_playing(artist, album, title, duration, False)
+                self.__now_playing(artist, album, title, duration,
+                                   mb_track_id, False)
 
     def __populate_loved_tracks(self):
         """
             Populate loved tracks playlist
         """
-        if not self.__is_auth:
+        if not self.available:
             return
         try:
             tracks = []
             user = self.get_user(self.__login)
             for loved in user.get_loved_tracks():
-                track_id = Lp().tracks.search_track(
+                track_id = App().tracks.search_track(
                                                   str(loved.track.artist),
                                                   str(loved.track.title))
                 if track_id is not None:
                     tracks.append(Track(track_id))
-            Lp().playlists.add_tracks(Type.LOVED, tracks)
+            App().playlists.add_tracks(Type.LOVED, tracks)
         except Exception as e:
                 print("LastFM::__populate_loved_tracks: %s" % e)
 
@@ -357,6 +362,6 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             return
         self.__login = attributes["login"]
         self.__password = password
-        if get_network_available():
+        if Gio.NetworkMonitor.get_default().get_network_available():
             helper = TaskHelper()
             helper.run(self.__connect, full_sync, callback=(callback, *args))
