@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # Copyright (c) 2015 Jean-Philippe Braun <eon@patapon.info>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
 
 from gi.repository import GLib
 
+from lollypop.helper_task import TaskHelper
 from lollypop.radios import Radios
-from lollypop.define import Lp, Type
-from lollypop.sqlcursor import SqlCursor
+from lollypop.define import App, Type
 
 
 class Base:
@@ -32,6 +32,11 @@ class Base:
         return super(Base, self).__dir__(*args, **kwargs) +\
             list(self.DEFAULTS.keys())
 
+    # Used by pickle
+    def __getstate__(self): return self.__dict__
+
+    def __setstate__(self, d): self.__dict__.update(d)
+
     def __getattr__(self, attr):
         # Lazy DB calls of attributes
         if attr in list(self.DEFAULTS.keys()):
@@ -83,7 +88,7 @@ class Base:
                 best_popularity = self.db.get_higher_popularity()
                 if new_rate == 5:
                     popularity = (popularity + best_popularity) / 2
-                self.db.set_popularity(self.id, popularity, True)
+                self.db.set_popularity(self.id, popularity)
             elif self.id == Type.RADIOS:
                 radios = Radios()
                 avg_popularity = radios.get_avg_popularity()
@@ -121,7 +126,7 @@ class Base:
             radios.set_rate(self._album_artists[0], rate)
         else:
             self.db.set_rate(self.id, rate)
-            Lp().player.emit("rate-changed", (self.id, rate))
+            App().player.emit("rate-changed", (self.id, rate))
 
 
 class Disc:
@@ -130,47 +135,70 @@ class Disc:
     """
 
     def __init__(self, album, disc_number):
-        self.db = Lp().albums
-        self.album = album
-        self.number = disc_number
-        self._track_ids = []
+        self.db = App().albums
+        self.__tracks = []
+        self.__album = album
+        self.__number = disc_number
+
+    def set_tracks(self, tracks):
+        """
+            Set disc tracks
+            @param tracks as [Track]
+        """
+        self.__tracks = tracks
 
     @property
-    def name(self):
+    def number(self):
         """
-            Disc name
-            @return disc name as str
+            Get disc number
         """
+        return self.__number
+
+    @property
+    def album(self):
+        """
+            Get disc album
+            @return Album
+        """
+        return self.__album
 
     @property
     def track_ids(self):
         """
-            Get all tracks ids of the disc
-            @return list of int
+            Get disc track ids
+            @return [int]
         """
-        if not self._track_ids:
-            self._track_ids = self.db.get_disc_tracks(self.album.id,
-                                                      self.album.genre_ids,
-                                                      self.album.artist_ids,
-                                                      self.number)
-            # If user tagged track with an artist not present in album
-            if not self._track_ids:
-                print("%s missing an album artist in artists" %
-                      self.album.name)
-                self._track_ids = self.db.get_disc_tracks(self.album.id,
-                                                          self.album.genre_ids,
-                                                          [],
-                                                          self.number)
-        return self._track_ids
+        return [track.id for track in self.tracks]
+
+    @property
+    def track_uris(self):
+        """
+            Get disc track uris
+            @return [str]
+        """
+        return [track.uri for track in self.tracks]
 
     @property
     def tracks(self):
         """
-            Get all tracks of the disc
-
-            @return list of Track
+            Get disc tracks
+            @return [Track]
         """
-        return [Track(id) for id in self.track_ids]
+        if not self.__tracks and self.album.id is not None:
+            self.__tracks = [Track(track_id, self.album)
+                             for track_id in self.db.get_disc_track_ids(
+                                                      self.album.id,
+                                                      self.album.genre_ids,
+                                                      self.album.artist_ids,
+                                                      self.number)]
+            if not self.__tracks:
+                self.__tracks = [Track(track_id, self.album)
+                                 for track_id in self.db.get_disc_track_ids(
+                                                          self.album.id,
+                                                          self.album.genre_ids,
+                                                          [],
+                                                          self.number)]
+        return self.__tracks
 
 
 class Album(Base):
@@ -182,6 +210,7 @@ class Album(Base):
                 "artist_ids": [],
                 "year": None,
                 "uri": "",
+                "tracks_count": 0,
                 "duration": 0,
                 "mtime": 0,
                 "synced": False,
@@ -193,14 +222,81 @@ class Album(Base):
             @param album_id as int
             @param genre_ids as [int]
         """
-        Base.__init__(self, Lp().albums)
+        Base.__init__(self, App().albums)
         self.id = album_id
         self.genre_ids = genre_ids
-        self._track_ids = None
+        self._tracks = []
+        self._discs = []
         # Use artist ids from db else
         if artist_ids:
             self.artist_ids = artist_ids
 
+    def merge_discs(self):
+        """
+            Merge discs into one
+        """
+        tracks = self.tracks
+        self._discs = [Disc(self, -1)]
+        self._discs[0].set_tracks(tracks)
+
+    def move_track(self, track, index):
+        """
+            Move track to index
+            @param track as Track
+            @param index
+        """
+        if track in self._tracks:
+            self._tracks.remove(track)
+            self._tracks.insert(index, track)
+
+    def set_tracks(self, tracks):
+        """
+            Set album tracks
+            @param tracks as [Track]
+        """
+        self._tracks = tracks
+        for track in tracks:
+            track.set_album(self)
+
+    def add_track(self, track):
+        """
+            Add track to album
+            @param track as Track
+        """
+        self._tracks.append(track)
+        track.set_album(self)
+
+    def remove_track(self, track):
+        """
+            Remove track from album
+            @param track as Track
+        """
+        if track in self.tracks:
+            self._tracks.remove(track)
+
+    def clear_tracks(self):
+        """
+            Clear album tracks
+        """
+        self._tracks = []
+
+    def disc_names(self, disc):
+        """
+            Disc names
+            @param disc as int
+            @return disc names as [str]
+        """
+        return self.db.get_disc_names(self.id, disc)
+
+    def set_loved(self, loved):
+        """
+            Mark album as loved
+            @param loved as bool
+        """
+        if self.id >= 0:
+            App().albums.set_loved(self.id, loved)
+            self.loved = loved
+
     @property
     def title(self):
         """
@@ -213,38 +309,29 @@ class Album(Base):
     def track_ids(self):
         """
             Get album track ids
-            @return list of int
+            @return [int]
         """
-        if self._track_ids is None:
-            self._track_ids = self.db.get_track_ids(self.id,
-                                                    self.genre_ids,
-                                                    self.artist_ids)
-            # If user tagged track with an artist not present in album
-            if not self._track_ids:
-                print("%s missing an album artist in artists" % self.name)
-                self._track_ids = self.db.get_track_ids(self.id,
-                                                        self.genre_ids,
-                                                        [])
-        return self._track_ids
+        return [track.id for track in self.tracks]
+
+    @property
+    def track_uris(self):
+        """
+            Get album track uris
+            @return [str]
+        """
+        return [track.uri for track in self.tracks]
 
     @property
     def tracks(self):
         """
             Get album tracks
-            @return list of Track
+            @return [Track]
         """
-        if not self._tracks and self.track_ids:
-            self._tracks = [Track(track_id) for track_id in self.track_ids]
+        if not self._tracks and self.id is not None:
+            for disc in self.discs:
+                self._tracks += disc.tracks
         return self._tracks
 
-    def disc_names(self, disc):
-        """
-            Disc names
-            @param disc as int
-            @return disc names as [str]
-        """
-        return self.db.get_disc_names(self.id, disc)
-
     @property
     def discs(self):
         """
@@ -252,47 +339,9 @@ class Album(Base):
             @return [Disc]
         """
         if not self._discs:
-            self._discs = self.db.get_discs(self.id, self.genre_ids)
-        return [Disc(self, number) for number in self._discs]
-
-    def set_loved(self, loved):
-        """
-            Mark album as loved
-            @param loved as bool
-        """
-        if self.id >= 0:
-            Lp().albums.set_loved(self.id, loved)
-
-    def remove(self):
-        """
-            Remove album
-        """
-        artist_ids = []
-        # We want all tracks
-        album = Album(self.id)
-        for track_id in self.track_ids:
-            artist_ids += Lp().tracks.get_artist_ids(track_id)
-            uri = Lp().tracks.get_uri(track_id)
-            Lp().playlists.remove(uri)
-            Lp().tracks.remove(track_id)
-            Lp().tracks.clean(track_id)
-            art_file = Lp().art.get_album_cache_name(self)
-            Lp().art.clean_store(art_file)
-        artist_ids += album.artist_ids
-        genre_ids = Lp().albums.get_genre_ids(album.id)
-        deleted = Lp().albums.clean(self.id)
-        for artist_id in list(set(artist_ids)):
-            Lp().artists.clean(artist_id)
-            # Do not check clean return
-            GLib.idle_add(Lp().scanner.emit, "artist-updated",
-                          artist_id, False)
-        for genre_id in genre_ids:
-            Lp().genres.clean(genre_id)
-            GLib.idle_add(Lp().scanner.emit, "genre-updated",
-                          genre_id, False)
-        with SqlCursor(Lp().db) as sql:
-            sql.commit()
-        GLib.idle_add(Lp().scanner.emit, "album-updated", self.id, deleted)
+            disc_numbers = self.db.get_discs(self.id, self.genre_ids)
+            self._discs = [Disc(self, number) for number in disc_numbers]
+        return self._discs
 
 
 class Track(Base):
@@ -311,83 +360,39 @@ class Track(Base):
                 "duration": 0,
                 "number": 0,
                 "year": None,
-                "mtime": 0}
+                "mtime": 0,
+                "mb_track_id": None}
 
-    def __init__(self, track_id=None):
+    def __init__(self, track_id=None, album=None):
         """
             Init track
             @param track_id as int
+            @param album as Album
         """
-        Base.__init__(self, Lp().tracks)
+        Base.__init__(self, App().tracks)
         self.id = track_id
         self._uri = None
+        self._number = 0
 
-    def get_featuring_ids(self, album_artist_ids):
+        if album is None:
+            self.__album = Album(self.album_id)
+        else:
+            self.__album = album
+        self.__featuring_ids = []
+
+    def set_featuring_ids(self, album_artist_ids):
         """
-            Get featuring artist ids
+            Set featuring artist ids
             @param artist ids as [int]
             @return featuring artist ids as [int]
         """
         artist_ids = self.db.get_artist_ids(self.id)
         album_id = self.db.get_album_id(self.id)
         if not album_artist_ids:
-            db_album_artist_ids = Lp().albums.get_artist_ids(album_id)
+            db_album_artist_ids = App().albums.get_artist_ids(album_id)
             if len(db_album_artist_ids) == 1:
                 artist_ids = list(set(artist_ids) - set(db_album_artist_ids))
-        return list(set(artist_ids) - set(album_artist_ids))
-
-    @property
-    def title(self):
-        """
-            Get track name
-            Alias to Track.name
-        """
-        return self.name
-
-    @property
-    def uri(self):
-        """
-            Get track file uri
-            @return str
-        """
-        if self._uri is None:
-            self._uri = Lp().tracks.get_uri(self.id)
-        return self._uri
-
-    @property
-    def path(self):
-        """
-            Get track file path
-            Alias to Track.path
-            @return str
-        """
-        return GLib.filename_from_uri(self.uri)[0]
-
-    @property
-    def album(self):
-        """
-            Get track"s album
-            @return Album
-        """
-        return Album(self.album_id)
-
-    @property
-    def album_artists(self):
-        """
-            Get track album artists, can be != than album.artists as track
-            may not have any album (radio, externals, ...)
-            @return str
-        """
-        if getattr(self, "_album_artists") is None:
-            self._album_artists = self.album.artists
-        return self._album_artists
-
-    def set_duration(self, duration):
-        """
-            Set duration
-            @param duration as in
-        """
-        self._duration = duration
+        self.__featuring_ids = list(set(artist_ids) - set(album_artist_ids))
 
     def set_album_artists(self, artists):
         """
@@ -396,6 +401,13 @@ class Track(Base):
         """
         self._album_artists = artists
 
+    def set_album(self, album):
+        """
+            Set track album
+            @param album as Album
+        """
+        self.__album = album
+
     def set_uri(self, uri):
         """
             Set uri
@@ -413,28 +425,122 @@ class Track(Base):
         self._album_artists = [name]
         self._uri = uri
 
-    def remove(self):
+    def set_number(self, number):
         """
-            Remove track
+            Set number
+            @param number as int
         """
-        artist_ids = []
-        album = self.album
-        artist_ids = Lp().tracks.get_artist_ids(self.id)
-        Lp().playlists.remove(self.uri)
-        Lp().tracks.remove(self.id)
-        Lp().tracks.clean(self.id)
-        artist_ids += album.artist_ids
-        genre_ids = Lp().tracks.get_genre_ids(self.id)
-        deleted = Lp().albums.clean(album.id)
-        for artist_id in list(set(artist_ids)):
-            Lp().artists.clean(artist_id)
-            # Do not check clean return
-            GLib.idle_add(Lp().scanner.emit, "artist-updated",
-                          artist_id, False)
-        for genre_id in genre_ids:
-            Lp().genres.clean(genre_id)
-            GLib.idle_add(Lp().scanner.emit, "genre-updated",
-                          genre_id, False)
-        with SqlCursor(Lp().db) as sql:
-            sql.commit()
-        GLib.idle_add(Lp().scanner.emit, "album-updated", album.id, deleted)
+        self._number = number
+
+    def set_loved(self, loved):
+        """
+            Add or remove track from loved playlist
+            @param loved Add to loved playlist if `True`; remove if `False`
+        """
+        scrobbler_love = True
+        if not self.loved:
+            if loved:
+                App().playlists.add_tracks(Type.LOVED, [self])
+        elif not loved:
+            scrobbler_love = False
+            App().playlists.remove_tracks(Type.LOVED, [self])
+        for scrobbler in App().scrobblers:
+            if scrobbler.can_love:
+                helper = TaskHelper()
+                helper.run(scrobbler.set_loved, self, scrobbler_love)
+
+    @property
+    def loved(self):
+        """
+            True if track loved
+            @return bool
+        """
+        return App().playlists.exists_track(Type.LOVED,
+                                            self)
+
+    @property
+    def featuring_artist_ids(self):
+        """
+            Get featuring artist ids
+            @return [int]
+        """
+        return self.__featuring_ids
+
+    @property
+    def position(self):
+        """
+            Get track position for album
+            @return int
+        """
+        i = 0
+        for track_id in self.__album.track_ids:
+            if track_id == self.id:
+                break
+            i += 1
+        return i
+
+    @property
+    def first(self):
+        """
+            Is track first for album
+            @return bool
+        """
+        tracks = self.__album.tracks
+        return tracks and self.id == tracks[0].id
+
+    @property
+    def last(self):
+        """
+            Is track last for album
+            @return bool
+        """
+        tracks = self.__album.tracks
+        return tracks and self.id == tracks[-1].id
+
+    @property
+    def title(self):
+        """
+            Get track name
+            Alias to Track.name
+        """
+        return self.name
+
+    @property
+    def uri(self):
+        """
+            Get track file uri
+            @return str
+        """
+        if self._uri is None:
+            self._uri = App().tracks.get_uri(self.id)
+        return self._uri
+
+    @property
+    def path(self):
+        """
+            Get track file path
+            Alias to Track.path
+            @return str
+        """
+        return GLib.filename_from_uri(self.uri)[0]
+
+    @property
+    def album(self):
+        """
+            Get track"s album
+            @return Album
+        """
+        if self.__album is None:
+            self.__album = Album(self._album_id)
+        return self.__album
+
+    @property
+    def album_artists(self):
+        """
+            Get track album artists, can be != than album.artists as track
+            may not have any album (radio, externals, ...)
+            @return str
+        """
+        if getattr(self, "_album_artists") is None:
+            self._album_artists = self.album.artists
+        return self._album_artists
