@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -21,13 +21,13 @@ from lollypop.player_linear import LinearPlayer
 from lollypop.player_shuffle import ShufflePlayer
 from lollypop.player_radio import RadioPlayer
 from lollypop.player_externals import ExternalsPlayer
-from lollypop.player_userplaylist import UserPlaylistPlayer
+from lollypop.player_playlist import PlaylistPlayer
 from lollypop.radios import Radios
 from lollypop.objects import Track, Album
-from lollypop.define import Lp, Type, NextContext, LOLLYPOP_DATA_PATH, Shuffle
+from lollypop.define import App, Type, NextContext, LOLLYPOP_DATA_PATH, Shuffle
 
 
-class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
+class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
              LinearPlayer, ShufflePlayer, ExternalsPlayer):
     """
         Player object used to manage playback and playlists
@@ -41,33 +41,12 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         QueuePlayer.__init__(self)
         LinearPlayer.__init__(self)
         ShufflePlayer.__init__(self)
-        UserPlaylistPlayer.__init__(self)
+        PlaylistPlayer.__init__(self)
         RadioPlayer.__init__(self)
         ExternalsPlayer.__init__(self)
         self.update_crossfading()
         self.__do_not_update_next = False
-        Lp().settings.connect("changed::playback", self.__on_playback_changed)
-
-    @property
-    def next_track(self):
-        """
-            Current track
-        """
-        return self._next_track
-
-    @property
-    def prev_track(self):
-        """
-            Current track
-        """
-        return self._prev_track
-
-    @property
-    def context(self):
-        """
-            Current track
-        """
-        return self._context
+        App().settings.connect("changed::playback", self.__on_playback_changed)
 
     def prev(self):
         """
@@ -75,7 +54,7 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         """
         if self._locked:
             return
-        smart_prev = Lp().settings.get_value("smart-previous")
+        smart_prev = App().settings.get_value("smart-previous")
         if self._prev_track.id is not None:
             if smart_prev and self.position / Gst.SECOND > 2:
                 self.seek(0)
@@ -103,7 +82,7 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             @param play as bool, ignored for radios
         """
         if track.id == Type.RADIOS:
-            RadioPlayer.load(self, track)
+            RadioPlayer.load(self, track, play)
         else:
             if play:
                 # Do not update next if user clicked on a track
@@ -114,43 +93,34 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
                 BinPlayer._load_track(self, track)
                 self.emit("current-changed")
 
-    def add_album(self, album):
+    def add_album(self, album, index=-1):
         """
             Add album
             @param album as Album
+            @param index as int
         """
         # We are not playing a user playlist anymore
-        self._user_playlist = []
-        self._user_playlist_ids = []
-        self.shuffle_albums(False)
-        # If album already exists, merge genres/artists
-        if album.id in self._albums:
-            genre_ids = self._context.genre_ids[album.id]
-            for genre_id in album.genre_ids:
-                if genre_id >= 0 and genre_id not in genre_ids:
-                    self._context.genre_ids[album.id].append(genre_id)
-            artist_ids = self._context.artist_ids[album.id]
-            for artist_id in album.artist_ids:
-                if artist_id >= 0 and artist_id not in artist_ids:
-                    self._context.artist_ids[album.id].append(artist_id)
+        self._playlist_track_ids = []
+        self._playlist_ids = []
+        # We do not shuffle when user add an album
+        self._albums_backup = []
+        if index == -1:
+            self._albums.append(album)
         else:
-            self._albums.append(album.id)
-            self._context.genre_ids[album.id] = list(album.genre_ids)
-            self._context.artist_ids[album.id] = list(album.artist_ids)
-        self.shuffle_albums(True)
+            self._albums.insert(index, album)
         if self._current_track.id is not None and self._current_track.id > 0:
             if not self.is_party:
                 self.set_next()
             self.set_prev()
         self.emit("album-added", album.id)
 
-    def move_album(self, album_id, position):
+    def move_album(self, album, position):
         """
             Move album to position
-            @param album id as int
+            @param album as Album
             @param position as int
         """
-        index = self._albums.index(album_id)
+        index = self._albums.index(album)
         self._albums.insert(position, self._albums.pop(index))
 
     def remove_album(self, album):
@@ -159,22 +129,9 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             @param album as Album
         """
         try:
-            # Remove genre ids from context
-            genre_ids = self._context.genre_ids[album.id]
-            for genre_id in album.genre_ids:
-                if genre_id in genre_ids:
-                    genre_ids.remove(genre_id)
-            artist_ids = self._context.artist_ids[album.id]
-            # Remove artist ids from context
-            for artist_id in album.artist_ids:
-                if artist_id in artist_ids:
-                    artist_ids.remove(artist_id)
-            if not genre_ids or not artist_ids:
-                self._context.genre_ids.pop(album.id, None)
-                self._context.artist_ids.pop(album.id, None)
-                self._albums.remove(album.id)
-                if album.id in self._albums_backup:
-                    self._albums_backup.remove(album.id)
+            self._albums.remove(album)
+            if album in self._albums_backup:
+                self._albums_backup.remove(album)
             if not self.is_party or self._next_track.album_id == album.id:
                 self.set_next()
             self.set_prev()
@@ -182,52 +139,6 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         except Exception as e:
             print("Player::remove_album():", e)
 
-    def get_genre_ids(self, album_id):
-        """
-            Return genre ids for album
-            @param album id as int
-            @return genre ids as [int]
-        """
-        if album_id in self._context.genre_ids.keys():
-            return self._context.genre_ids[album_id]
-        else:
-            return []
-
-    def get_artist_ids(self, album_id):
-        """
-            Return artist ids for album
-            @param album id as int
-            @return artist ids as [int]
-        """
-        if album_id in self._context.artist_ids.keys():
-            return self._context.artist_ids[album_id]
-        else:
-            return []
-
-    def has_album(self, album):
-        """
-            Check if player has album
-            @param album as Album
-            @return bool
-        """
-        is_genres = True
-        is_artists = True
-        if album.id in self._albums:
-            for genre_id in album.genre_ids:
-                if album.id in self._context.genre_ids.keys() and\
-                   self._context.genre_ids[album.id] and\
-                   genre_id not in self._context.genre_ids[album.id]:
-                    is_genres = False
-            for artist_id in album.artist_ids:
-                if album.id in self._context.artist_ids.keys() and\
-                   self._context.artist_ids[album.id] and\
-                   artist_id not in self._context.artist_ids[album.id]:
-                    is_artists = False
-        else:
-            is_genres = False
-            is_artists = False
-        return is_genres and is_artists
-
     def play_album(self, album):
         """
             Play album
@@ -237,106 +148,82 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             self.set_party(False)
         self.reset_history()
         # We are not playing a user playlist anymore
-        self._user_playlist = []
-        self._user_playlist_ids = []
-        self._context.genre_ids = {}
-        self._context.artist_ids = {}
-        self._context.genre_ids[album.id] = []
-        self._context.artist_ids[album.id] = []
-        for genre_id in album.genre_ids:
-            if genre_id >= 0:
-                self._context.genre_ids[album.id].append(genre_id)
-        for artist_id in album.artist_ids:
-            if artist_id >= 0:
-                self._context.artist_ids[album.id].append(artist_id)
-        if Lp().settings.get_enum("shuffle") == Shuffle.TRACKS:
+        self._playlist_track_ids = []
+        self._playlist_ids = []
+        if App().settings.get_enum("shuffle") == Shuffle.TRACKS:
             track = choice(album.tracks)
         else:
             track = album.tracks[0]
         self.load(track)
-        self._albums = [album.id]
+        self._albums = [album]
 
-    def set_albums(self, track_id, artist_ids, genre_ids):
+    def play_albums(self, track, genre_ids, artist_ids):
         """
-            Set album list (for next/prev)
-            @param track id as int
-            @param artist id as int
-            @param genre id as int
+            Play albums related to track/genre_ids/artist_ids
+            @param track as Track
+            @param genre_ids as [int]
+            @param artist_ids as [int]
         """
-        # Invalid track
-        if track_id is None:
-            return
         self._albums = []
-        self._context.genre_ids = {}
-        self._context.aritst_ids = {}
+        album_ids = []
         ShufflePlayer.reset_history(self)
 
         # We are not playing a user playlist anymore
-        self._user_playlist = []
-        self._user_playlist_ids = []
+        self._playlist_track_ids = []
+        self._playlist_ids = []
         # We are in all artists
         if (genre_ids and genre_ids[0] == Type.ALL) or\
            (artist_ids and artist_ids[0] == Type.ALL):
             # Genres: all, Artists: compilations
             if artist_ids and artist_ids[0] == Type.COMPILATIONS:
-                self._albums += Lp().albums.get_compilation_ids()
+                album_ids = App().albums.get_compilation_ids()
             # Genres: all, Artists: ids
             elif artist_ids and artist_ids[0] != Type.ALL:
-                self._albums += Lp().albums.get_ids(artist_ids)
+                album_ids += App().albums.get_ids(artist_ids)
             # Genres: all, Artists: all
             else:
-                if Lp().settings.get_value("show-compilations"):
-                    self._albums += Lp().albums.get_compilation_ids()
-                self._albums += Lp().albums.get_ids()
+                if App().settings.get_value("show-compilations-in-album-view"):
+                    album_ids += App().albums.get_compilation_ids()
+                album_ids += App().albums.get_ids()
         # We are in populars view, add popular albums
         elif genre_ids and genre_ids[0] == Type.POPULARS:
-            self._albums = Lp().albums.get_populars()
+            album_ids = App().albums.get_populars()
         # We are in loved view, add loved albums
         elif genre_ids and genre_ids[0] == Type.LOVED:
-            self._albums = Lp().albums.get_loves()
+            album_ids = App().albums.get_loves()
         # We are in recents view, add recent albums
         elif genre_ids and genre_ids[0] == Type.RECENTS:
-            self._albums = Lp().albums.get_recents()
+            album_ids = App().albums.get_recents()
         # We are in randoms view, add random albums
         elif genre_ids and genre_ids[0] == Type.RANDOMS:
-            self._albums = Lp().albums.get_cached_randoms()
+            album_ids = App().albums.get_cached_randoms()
         # We are in compilation view without genre
         elif genre_ids and genre_ids[0] == Type.COMPILATIONS:
-            self._albums = Lp().albums.get_compilation_ids()
+            album_ids = App().albums.get_compilation_ids()
         # Add albums for artists/genres
         else:
             # If we are not in compilation view and show compilation is on,
             # add compilations
             if artist_ids and artist_ids[0] == Type.COMPILATIONS:
-                self._albums += Lp().albums.get_compilation_ids(genre_ids)
+                album_ids += App().albums.get_compilation_ids(genre_ids)
             else:
                 if not artist_ids and\
-                        Lp().settings.get_value("show-compilations"):
-                    self._albums += Lp().albums.get_compilation_ids(genre_ids)
-                self._albums += Lp().albums.get_ids(artist_ids, genre_ids)
-
-        # We do not store genre_ids for ALL/POPULARS/...
-        if genre_ids and genre_ids[0] < 0:
-            genre_ids = []
-        # Set context for each album
-        for album_id in self._albums:
-            self._context.genre_ids[album_id] = []
-            self._context.artist_ids[album_id] = []
-            for genre_id in genre_ids:
-                if genre_id >= 0:
-                    self._context.genre_ids[album_id].append(genre_id)
-            for artist_id in artist_ids:
-                if artist_id >= 0:
-                    self._context.artist_ids[album_id].append(artist_id)
+                        App().settings.get_value(
+                                            "show-compilations-in-album-view"):
+                    album_ids += App().albums.get_compilation_ids(genre_ids)
+                album_ids += App().albums.get_ids(artist_ids, genre_ids)
+        # Create album objects
+        for album_id in album_ids:
+            album = Album(album_id, genre_ids, artist_ids)
+            self._albums.append(album)
+            # Get track from album
+            # to make Player.current_track present in Player.albums
+            if album.id == track.album.id:
+                index = album.track_ids.index(track.id)
+                track = album.tracks[index]
         # Shuffle album list if needed
         self.shuffle_albums(True)
-
-    def get_albums(self):
-        """
-            Return albums
-            @return albums as [int]
-        """
-        return self._albums
+        self.load(track)
 
     def clear_albums(self):
         """
@@ -361,67 +248,54 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             Restore player state
         """
         try:
-            if Lp().settings.get_value("save-state"):
+            if App().settings.get_value("save-state"):
                 track_id = load(open(LOLLYPOP_DATA_PATH +
-                                     "/track_id.bin",
-                                     "rb"))
+                                     "/track_id.bin", "rb"))
+                self.set_queue(load(open(LOLLYPOP_DATA_PATH +
+                               "/queue.bin", "rb")))
                 playlist_ids = load(open(LOLLYPOP_DATA_PATH +
-                                         "/playlist_ids.bin",
-                                         "rb"))
+                                         "/playlist_ids.bin", "rb"))
                 (is_playing, was_party) = load(open(LOLLYPOP_DATA_PATH +
-                                                    "/player.bin",
-                                                    "rb"))
+                                                    "/player.bin", "rb"))
                 if playlist_ids and playlist_ids[0] == Type.RADIOS:
                     radios = Radios()
                     track = Track()
                     name = radios.get_name(track_id)
                     url = radios.get_url(name)
                     track.set_radio(name, url)
-                    self.load(track)
-                elif Lp().tracks.get_uri(track_id) != "":
+                    self.load(track, is_playing)
+                elif App().tracks.get_uri(track_id) != "":
                     track = Track(track_id)
                     self._load_track(track)
                     # We set this initial state
                     # because seek while failed otherwise
                     self.pause()
                     if playlist_ids:
-                        pids = []
-                        for playlist_id in playlist_ids:
-                            pids.append(int(playlist_id))
                         track_ids = []
                         for playlist_id in playlist_ids:
                             if playlist_id == Type.POPULARS:
-                                tracks = Lp().tracks.get_populars()
+                                track_ids = App().tracks.get_populars()
                             elif playlist_id == Type.RECENTS:
-                                tracks = Lp().tracks.get_recently_listened_to()
+                                track_ids = App().tracks.\
+                                    get_recently_listened_to()
                             elif playlist_id == Type.NEVER:
-                                tracks = Lp().tracks.get_never_listened_to()
+                                track_ids = App().tracks.\
+                                    get_never_listened_to()
                             elif playlist_id == Type.RANDOMS:
-                                tracks = Lp().tracks.get_randoms()
+                                track_ids = App().tracks.get_randoms()
                             else:
-                                tracks = Lp().playlists.get_track_ids(
+                                track_ids = App().playlists.get_track_ids(
                                                                    playlist_id)
-                            for track_id in tracks:
-                                if track_id not in track_ids:
-                                    track_ids.append(track_id)
-                            self.populate_user_playlist_by_tracks(track_ids,
-                                                                  pids)
+                            self.populate_playlist_by_track_ids(
+                                                          list(set(track_ids)),
+                                                          playlist_ids)
                     else:
                         if was_party:
                             self.emit("party-changed", True)
                         else:
                             self._albums = load(open(
                                                 LOLLYPOP_DATA_PATH +
-                                                "/albums.bin",
-                                                "rb"))
-                            self.shuffle_albums(True)
-                            self._context.genre_ids = load(open(
-                                                LOLLYPOP_DATA_PATH +
-                                                "/genre_ids.bin",
-                                                "rb"))
-                            self._context.artist_ids = load(open(
-                                                LOLLYPOP_DATA_PATH +
-                                                "/artist_ids.bin",
+                                                "/Albums.bin",
                                                 "rb"))
                     self.set_next()
                     self.set_prev()
@@ -450,7 +324,7 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             Set previous track
         """
         try:
-            if Lp().settings.get_enum("playback") == NextContext.REPEAT_TRACK:
+            if App().settings.get_enum("playback") == NextContext.REPEAT_TRACK:
                 self._prev_track = self._current_track
             else:
                 # Look at externals
@@ -462,7 +336,7 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
 
             # Look at user playlist then
             if self._prev_track.id is None:
-                self._prev_track = UserPlaylistPlayer.prev(self)
+                self._prev_track = PlaylistPlayer.prev(self)
 
             # Look at shuffle
             if self._prev_track.id is None:
@@ -485,7 +359,7 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
             # Reset finished context
             self._next_context = NextContext.NONE
 
-            if Lp().settings.get_enum("playback") == NextContext.REPEAT_TRACK:
+            if App().settings.get_enum("playback") == NextContext.REPEAT_TRACK:
                 next_track = self._current_track
             else:
                 # Look at externals
@@ -501,7 +375,7 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
 
             # Look at user playlist then
             if next_track.id is None:
-                next_track = UserPlaylistPlayer.next(self, force)
+                next_track = PlaylistPlayer.next(self, force)
 
             # Get a random album/track then
             if next_track.id is None:
@@ -521,27 +395,78 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         """
         # In party or shuffle, just update next track
         if self.is_party or\
-                Lp().settings.get_enum("shuffle") == Shuffle.TRACKS:
+                App().settings.get_enum("shuffle") == Shuffle.TRACKS:
             self.set_next()
             # We send this signal to update next popover
             self.emit("queue-changed")
         elif self._current_track.id is not None:
-            pos = self._albums.index(self._current_track.album.id)
+            pos = self._albums.index(self._current_track.album)
             if pos + 1 >= len(self._albums):
                 next_album = self._albums[0]
             else:
                 next_album = self._albums[pos + 1]
-            self.load(Album(next_album).tracks[0])
+            self.load(next_album.tracks[0])
 
     def update_crossfading(self):
         """
             Calculate if crossfading is needed
         """
-        mix = Lp().settings.get_value("mix")
-        party_mix = Lp().settings.get_value("party-mix")
+        mix = App().settings.get_value("smooth-transitions")
+        party_mix = App().settings.get_value("party-mix")
         self._crossfading = (mix and not party_mix) or\
                             (mix and party_mix and self.is_party)
 
+    def track_in_playback(self, track):
+        """
+            True if track present in current playback
+            @param track as Track
+            @return bool
+        """
+        for album in self._albums:
+            if album.id == track.album.id:
+                for track_id in album.track_ids:
+                    if track.id == track_id:
+                        return True
+        return False
+
+    def object_by_name(self, track_name, album_name):
+        """
+            Get track by object name
+            @track_name as str
+            @param album_name as str
+            @return Album is track_name is None, else Track or None
+        """
+        for album in self._albums:
+            if str(album) == album_name:
+                if track_name is None:
+                    return album
+                for track in album.tracks:
+                    if str(track) == track_name:
+                        return track
+        return None
+
+    @property
+    def next_track(self):
+        """
+            Current track
+        """
+        return self._next_track
+
+    @property
+    def prev_track(self):
+        """
+            Current track
+        """
+        return self._prev_track
+
+    @property
+    def albums(self):
+        """
+            Return albums
+            @return albums as [Album]
+        """
+        return self._albums
+
 #######################
 # PROTECTED           #
 #######################
@@ -558,8 +483,8 @@ class Player(BinPlayer, QueuePlayer, UserPlaylistPlayer, RadioPlayer,
         """
             On stream start, set next and previous track
         """
-        if not Lp().scanner.is_locked():
-            Lp().window.pulse(False)
+        if not App().scanner.is_locked():
+            App().window.container.pulse(False)
         if self._current_track.id is not None and self._current_track.id >= 0:
             ShufflePlayer._on_stream_start(self, bus, message)
         if self.track_in_queue(self._current_track):
