@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,9 +16,10 @@ from gettext import gettext as _
 import itertools
 import sqlite3
 from datetime import datetime
+from threading import Lock
 
 from lollypop.database import Database
-from lollypop.define import Lp, Type
+from lollypop.define import App, Type
 from lollypop.objects import Track
 from lollypop.sqlcursor import SqlCursor
 from lollypop.localized import LocalizedCollation
@@ -50,6 +51,7 @@ class Playlists(GObject.GObject):
         """
             Init playlists manager
         """
+        self.thread_lock = Lock()
         GObject.GObject.__init__(self)
         self.LOVED = _("Loved tracks")
         # Create db schema
@@ -57,7 +59,6 @@ class Playlists(GObject.GObject):
             with SqlCursor(self) as sql:
                 sql.execute(self.__create_playlists)
                 sql.execute(self.__create_tracks)
-                sql.commit()
         except:
             pass
 
@@ -71,7 +72,6 @@ class Playlists(GObject.GObject):
             result = sql.execute("INSERT INTO playlists (name, mtime)"
                                  " VALUES (?, ?)",
                                  (name, datetime.now().strftime("%s")))
-            sql.commit()
             GLib.idle_add(self.emit, "playlists-changed", result.lastrowid)
 
     def exists(self, playlist_id):
@@ -103,7 +103,6 @@ class Playlists(GObject.GObject):
                         SET name=?\
                         WHERE name=?",
                         (new_name, old_name))
-            sql.commit()
             GLib.idle_add(self.emit, "playlists-changed", playlist_id)
 
     def delete(self, name):
@@ -119,7 +118,6 @@ class Playlists(GObject.GObject):
             sql.execute("DELETE FROM tracks\
                         WHERE playlist_id=?",
                         (playlist_id,))
-            sql.commit()
             GLib.idle_add(self.emit, "playlists-changed", playlist_id)
 
     def remove(self, uri):
@@ -131,7 +129,6 @@ class Playlists(GObject.GObject):
             sql.execute("DELETE FROM tracks\
                         WHERE uri=?",
                         (uri,))
-            sql.commit()
 
     def get(self):
         """
@@ -294,7 +291,6 @@ class Playlists(GObject.GObject):
         with SqlCursor(self) as sql:
             sql.execute("DELETE FROM tracks\
                          WHERE playlist_id=?", (playlist_id,))
-            sql.commit()
             if notify:
                 GLib.idle_add(self.emit, "playlist-del", playlist_id, None)
 
@@ -308,7 +304,7 @@ class Playlists(GObject.GObject):
         with SqlCursor(self) as sql:
             changed = False
             for track in tracks:
-                if not self.exists_track(playlist_id, track.id):
+                if not self.exists_track(playlist_id, track):
                     changed = True
                     sql.execute("INSERT INTO tracks"
                                 " VALUES (?, ?)",
@@ -320,7 +316,6 @@ class Playlists(GObject.GObject):
                 sql.execute("UPDATE playlists SET mtime=?\
                              WHERE rowid=?", (datetime.now().strftime("%s"),
                                               playlist_id))
-                sql.commit()
 
     def remove_tracks(self, playlist_id, tracks, notify=True):
         """
@@ -336,15 +331,14 @@ class Playlists(GObject.GObject):
                 if notify:
                     GLib.idle_add(self.emit, "playlist-del",
                                   playlist_id, track.id)
-            sql.commit()
 
-    def import_uri(self, playlist_id, uri, start=None, up=False):
+    def import_uri(self, playlist_id, uri, start=None, down=True):
         """
             Import uri in playlist
             @param playlist id as int
             @param uri as str
             @param start track id as int
-            @param up as bool
+            @param down as bool
         """
         try:
             uri = uri.strip("\n\r")
@@ -370,12 +364,12 @@ class Playlists(GObject.GObject):
                             if info.get_file_type() == Gio.FileType.DIRECTORY:
                                 walk_uris.append(f.get_uri())
                             else:
-                                track_id = Lp().tracks.get_id_by_uri(
+                                track_id = App().tracks.get_id_by_uri(
                                                                    f.get_uri())
                                 if track_id is not None:
                                     track_ids.append(track_id)
                 else:
-                    track_id = Lp().tracks.get_id_by_uri(uri)
+                    track_id = App().tracks.get_id_by_uri(uri)
                     track_ids = [track_id]
                 tracks = []
                 if start is None:
@@ -386,7 +380,7 @@ class Playlists(GObject.GObject):
                     # Insert at wanted position
                     playlist_track_ids = self.get_track_ids(playlist_id)
                     start_idx = playlist_track_ids.index(start)
-                    if not up:
+                    if down:
                         start_idx += 1
                     for track_id in track_ids:
                         playlist_track_ids.insert(start_idx, track_id)
@@ -415,7 +409,7 @@ class Playlists(GObject.GObject):
             i += 1
         return i
 
-    def exists_track(self, playlist_id, track_id):
+    def exists_track(self, playlist_id, track):
         """
             Check if track id exist in playlist
             @param playlist id as int
@@ -429,27 +423,22 @@ class Playlists(GObject.GObject):
                                   AND playlist_id=?\
                                   AND music.tracks.uri=\
                                   main.tracks.uri",
-                                 (track_id, playlist_id))
+                                 (track.id, playlist_id))
             v = result.fetchone()
             if v is not None:
                 return True
             return False
 
-    def exists_album(self, playlist_id, album_id, genre_ids, artist_ids):
+    def exists_album(self, playlist_id, album):
         """
             Return True if object_id is already present in playlist
             @param playlist id as int
-            @param album id as int
-            @param genre ids as [int]
-            @param artist ids as [int]
-            @param sql as sqlite cursor
+            @param album as Album/Disc
             @return bool
         """
         # We do not use Album object for performance reasons
         playlist_uris = self.get_tracks(playlist_id)
-        track_uris = Lp().albums.get_track_uris(album_id,
-                                                genre_ids,
-                                                artist_ids)
+        track_uris = album.track_uris
         return len(set(playlist_uris) & set(track_uris)) == len(track_uris)
 
     def get_cursor(self):
