@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,9 +16,8 @@ from gettext import gettext as _
 
 from lollypop.widgets_rating import RatingWidget
 from lollypop.widgets_loved import LovedWidget
-from lollypop.define import Lp, Type
+from lollypop.define import App, Type, TAG_EDITORS
 from lollypop.objects import Track, Album
-from lollypop.utils import set_loved
 from lollypop.helper_dbus import DBusHelper
 from lollypop.helper_task import TaskHelper
 
@@ -39,7 +38,7 @@ class BaseMenu(Gio.Menu):
 
 class ArtistMenu(BaseMenu):
     """
-        Contextual menu for queue
+        Contextual menu for artist
     """
 
     def __init__(self, object):
@@ -48,17 +47,29 @@ class ArtistMenu(BaseMenu):
             @param object as Album/Track
         """
         BaseMenu.__init__(self, object)
-        self.__set_artists_actions()
+        if self.__filter_artist_ids():
+            self.__set_artists_actions()
 
 #######################
 # PRIVATE             #
 #######################
+    def __filter_artist_ids(self):
+        """
+            Remove static entries
+            @return [int]
+        """
+        artist_ids = []
+        for artist_id in self._object.artist_ids:
+            if artist_id > 0:
+                artist_ids.append(artist_id)
+        return artist_ids
+
     def __set_artists_actions(self):
         """
             Set queue actions
         """
         go_artist_action = Gio.SimpleAction(name="go_artist_action")
-        Lp().add_action(go_artist_action)
+        App().add_action(go_artist_action)
         go_artist_action.connect("activate",
                                  self.__go_to_artists)
         self.append(_("Show albums from artist"), "app.go_artist_action")
@@ -69,7 +80,7 @@ class ArtistMenu(BaseMenu):
             @param SimpleAction
             @param GVariant
         """
-        Lp().window.show_artists_albums(self._object.artist_ids)
+        App().window.container.show_artists_albums(self.__filter_artist_ids())
 
 
 class QueueMenu(BaseMenu):
@@ -77,12 +88,12 @@ class QueueMenu(BaseMenu):
         Contextual menu for queue
     """
 
-    def __init__(self, object):
+    def __init__(self, track):
         """
             Init playlists menu
-            @param object as Album/Track
+            @param track as Track
         """
-        BaseMenu.__init__(self, object)
+        BaseMenu.__init__(self, track)
         self.__set_queue_actions()
 
 #######################
@@ -92,48 +103,15 @@ class QueueMenu(BaseMenu):
         """
             Set queue actions
         """
-        queue = Lp().player.queue
-        append = True
-        prepend = True
-        delete = True
-        if not queue:
-            append = False
-        if not isinstance(self._object, Album):
-            if self._object.id in queue:
-                if queue and queue[0] == self._object.id:
-                    prepend = False
-                append = False
-            else:
-                delete = False
-        else:
-            tracks = Lp().albums.get_track_ids(self._object.id,
-                                               self._object.genre_ids,
-                                               self._object.artist_ids)
-            union = set(queue) & set(tracks)
-            if len(union) == len(tracks):
-                append = False
-                prepend = False
-            elif not bool(union):
-                delete = False
-
-        append_queue_action = Gio.SimpleAction(name="append_queue_action")
-        Lp().add_action(append_queue_action)
-        prepend_queue_action = Gio.SimpleAction(name="prepend_queue_action")
-        Lp().add_action(prepend_queue_action)
-        del_queue_action = Gio.SimpleAction(name="del_queue_action")
-        Lp().add_action(del_queue_action)
-        if append:
+        if self._object.id not in App().player.queue:
+            append_queue_action = Gio.SimpleAction(name="append_queue_action")
+            App().add_action(append_queue_action)
             append_queue_action.connect("activate",
                                         self.__append_to_queue)
             self.append(_("Add to queue"), "app.append_queue_action")
-        if prepend:
-            prepend_queue_action.connect("activate",
-                                         self.__insert_in_queue)
-            if isinstance(self._object, Album):
-                self.append(_("Next tracks"), "app.prepend_queue_action")
-            else:
-                self.append(_("Next track"), "app.prepend_queue_action")
-        if delete:
+        else:
+            del_queue_action = Gio.SimpleAction(name="del_queue_action")
+            App().add_action(del_queue_action)
             del_queue_action.connect("activate",
                                      self.__del_from_queue)
             self.append(_("Remove from queue"), "app.del_queue_action")
@@ -144,30 +122,8 @@ class QueueMenu(BaseMenu):
             @param SimpleAction
             @param GVariant
         """
-        if isinstance(self._object, Album):
-            for track_id in Lp().albums.get_track_ids(self._object.id,
-                                                      self._object.genre_ids,
-                                                      self._object.artist_ids):
-                Lp().player.append_to_queue(track_id, False)
-        else:
-            Lp().player.append_to_queue(self._object.id, False)
-        Lp().player.emit("queue-changed")
-
-    def __insert_in_queue(self, action, variant):
-        """
-            Prepend track id to queue
-            @param SimpleAction
-            @param GVariant
-        """
-        if isinstance(self._object, Album):
-            for track_id in reversed(Lp().albums.get_track_ids(
-                                                     self._object.id,
-                                                     self._object.genre_ids,
-                                                     self._object.artist_ids)):
-                Lp().player.insert_in_queue(track_id, 0, False)
-        else:
-            Lp().player.insert_in_queue(self._object.id, 0, False)
-        Lp().player.emit("queue-changed")
+        App().player.append_to_queue(self._object.id, False)
+        App().player.emit("queue-changed")
 
     def __del_from_queue(self, action, variant):
         """
@@ -175,14 +131,8 @@ class QueueMenu(BaseMenu):
             @param SimpleAction
             @param GVariant
         """
-        if isinstance(self._object, Album):
-            for track_id in Lp().albums.get_track_ids(self._object.id,
-                                                      self._object.genre_ids,
-                                                      self._object.artist_ids):
-                Lp().player.del_from_queue(track_id, False)
-        else:
-            Lp().player.del_from_queue(self._object.id, False)
-        Lp().player.emit("queue-changed")
+        App().player.del_from_queue(self._object.id, False)
+        App().player.emit("queue-changed")
 
 
 class PlaylistsMenu(BaseMenu):
@@ -206,21 +156,19 @@ class PlaylistsMenu(BaseMenu):
             Set playlist actions
         """
         playlist_action = Gio.SimpleAction(name="playlist_action")
-        Lp().add_action(playlist_action)
+        App().add_action(playlist_action)
         playlist_action.connect("activate",
                                 self.__add_to_playlists)
         self.append(_("Add to others"), "app.playlist_action")
 
         playlist_action = Gio.SimpleAction(name="playlist_not_in_party")
-        Lp().add_action(playlist_action)
+        App().add_action(playlist_action)
         if isinstance(self._object, Album):
-            exists = Lp().playlists.exists_album(Type.NOPARTY,
-                                                 self._object.id,
-                                                 self._object.genre_ids,
-                                                 self._object.artist_ids)
+            exists = App().playlists.exists_album(Type.NOPARTY,
+                                                  self._object)
         else:
-            exists = Lp().playlists.exists_track(Type.NOPARTY,
-                                                 self._object.id)
+            exists = App().playlists.exists_track(Type.NOPARTY,
+                                                  self._object)
         if exists:
             self.append(_('Remove from "Not in party"'),
                         "app.playlist_not_in_party")
@@ -233,17 +181,15 @@ class PlaylistsMenu(BaseMenu):
                                     self.__add_to_playlist, Type.NOPARTY)
 
         i = 0
-        for playlist in Lp().playlists.get_last():
+        for playlist in App().playlists.get_last():
             action = Gio.SimpleAction(name="playlist%s" % i)
-            Lp().add_action(action)
+            App().add_action(action)
             if isinstance(self._object, Album):
-                exists = Lp().playlists.exists_album(playlist[0],
-                                                     self._object.id,
-                                                     self._object.genre_ids,
-                                                     self._object.artist_ids)
+                exists = App().playlists.exists_album(playlist[0],
+                                                      self._object)
             else:
-                exists = Lp().playlists.exists_track(playlist[0],
-                                                     self._object.id)
+                exists = App().playlists.exists_track(playlist[0],
+                                                      self._object)
             if exists:
                 action.connect("activate",
                                self.__remove_from_playlist,
@@ -264,10 +210,7 @@ class PlaylistsMenu(BaseMenu):
             @param SimpleAction
             @param GVariant
         """
-        Lp().window.show_playlist_manager(self._object.id,
-                                          self._object.genre_ids,
-                                          self._object.artist_ids,
-                                          isinstance(self._object, Album))
+        App().window.container.show_playlist_manager(self._object)
 
     def __add_to_playlist(self, action, variant, playlist_id):
         """
@@ -279,17 +222,17 @@ class PlaylistsMenu(BaseMenu):
         def add(playlist_id):
             tracks = []
             if isinstance(self._object, Album):
-                track_ids = Lp().albums.get_track_ids(self._object.id,
-                                                      self._object.genre_ids,
-                                                      self._object.artist_ids)
+                track_ids = App().albums.get_track_ids(self._object.id,
+                                                       self._object.genre_ids,
+                                                       self._object.artist_ids)
                 for track_id in track_ids:
                     tracks.append(Track(track_id))
             else:
                 tracks = [Track(self._object.id)]
-            Lp().playlists.add_tracks(playlist_id, tracks)
-            if playlist_id in Lp().player.get_user_playlist_ids():
-                Lp().player.update_user_playlist(
-                                     Lp().playlists.get_track_ids(playlist_id))
+            App().playlists.add_tracks(playlist_id, tracks)
+            if playlist_id in App().player.get_playlist_ids():
+                App().player.update_playlist(
+                                    App().playlists.get_track_ids(playlist_id))
         helper = TaskHelper()
         helper.run(add, playlist_id)
 
@@ -305,17 +248,17 @@ class PlaylistsMenu(BaseMenu):
         def remove(playlist_id):
             tracks = []
             if isinstance(self._object, Album):
-                track_ids = Lp().albums.get_track_ids(self._object.id,
-                                                      self._object.genre_ids,
-                                                      self._object.artist_ids)
+                track_ids = App().albums.get_track_ids(self._object.id,
+                                                       self._object.genre_ids,
+                                                       self._object.artist_ids)
                 for track_id in track_ids:
                     tracks.append(Track(track_id))
             else:
                 tracks = [Track(self._object.id)]
-            Lp().playlists.remove_tracks(playlist_id, tracks)
-            if playlist_id in Lp().player.get_user_playlist_ids():
-                Lp().player.update_user_playlist(
-                                     Lp().playlists.get_track_ids(playlist_id))
+            App().playlists.remove_tracks(playlist_id, tracks)
+            if playlist_id in App().player.get_playlist_ids():
+                App().player.update_playlist(
+                                    App().playlists.get_track_ids(playlist_id))
         helper = TaskHelper()
         helper.run(remove, playlist_id)
 
@@ -326,7 +269,7 @@ class PlaylistsMenu(BaseMenu):
             @param GVariant
             @param playlist name as str
         """
-        set_loved(self._object.id, True)
+        self._object.set_loved(True)
 
     def __del_from_loved(self, action, variant):
         """
@@ -335,7 +278,7 @@ class PlaylistsMenu(BaseMenu):
             @param GVariant
             @param playlist name as str
         """
-        set_loved(self._object.id, False)
+        self._object.set_loved(False)
 
 
 class EditMenu(BaseMenu):
@@ -348,6 +291,13 @@ class EditMenu(BaseMenu):
             Init edit menu
             @param object as Album/Track
         """
+        # Search for available tag editors
+        self.__editor = App().settings.get_value("tag-editor").get_string()
+        if not self.__editor:
+            for tag_editor in TAG_EDITORS:
+                if GLib.find_program_in_path(tag_editor) is not None:
+                    self.__editor = tag_editor
+                    break
         # Ignore genre_ids/artist_ids
         if isinstance(object, Album):
             obj = Album(object.id)
@@ -356,7 +306,8 @@ class EditMenu(BaseMenu):
         BaseMenu.__init__(self, obj)
 
         dbus_helper = DBusHelper()
-        dbus_helper.call("CanLaunchTagEditor", None,
+        dbus_helper.call("CanLaunchTagEditor",
+                         GLib.Variant("(s)", (self.__editor,)),
                          self.__on_can_launch_tag_editor, None)
 
 #######################
@@ -367,18 +318,10 @@ class EditMenu(BaseMenu):
             Set edit actions
         """
         edit_tag_action = Gio.SimpleAction(name="edit_tag_action")
-        Lp().add_action(edit_tag_action)
+        App().add_action(edit_tag_action)
         edit_tag_action.connect("activate", self.__edit_tag)
         self.append(_("Modify information"), "app.edit_tag_action")
 
-    def __remove_object(self, action, variant):
-        """
-            Remove album
-            @param SimpleAction
-            @param GVariant
-        """
-        self._object.remove()
-
     def __edit_tag(self, action, variant):
         """
             Run tag editor
@@ -387,7 +330,8 @@ class EditMenu(BaseMenu):
         """
         path = GLib.filename_from_uri(self._object.uri)[0]
         dbus_helper = DBusHelper()
-        dbus_helper.call("LaunchTagEditor", GLib.Variant("(s)", (path,)),
+        dbus_helper.call("LaunchTagEditor",
+                         GLib.Variant("(ss)", (self.__editor, path)),
                          None, None)
 
     def __on_can_launch_tag_editor(self, source, result, data):
@@ -398,7 +342,8 @@ class EditMenu(BaseMenu):
             @param data as object
         """
         try:
-            if source.call_finish(result)[0]:
+            source_result = source.call_finish(result)
+            if source_result is not None and source_result[0]:
                 self.__set_edit_actions()
         except Exception as e:
             print("EditMenu::__on_can_launch_tag_editor():", e)
@@ -419,8 +364,6 @@ class AlbumMenu(Gio.Menu):
         if show_artist:
             self.insert_section(0, _("Artist"),
                                 ArtistMenu(album))
-        self.insert_section(1, _("Queue"),
-                            QueueMenu(album))
         self.insert_section(2, _("Playlists"),
                             PlaylistsMenu(album))
         self.insert_section(3, _("Edit"),
@@ -432,15 +375,19 @@ class TrackMenu(Gio.Menu):
         Contextual menu for a track
     """
 
-    def __init__(self, track):
+    def __init__(self, track, show_artist=False):
         """
             Init menu model
             @param track as Track
+            @param show artist menu as bool
         """
         Gio.Menu.__init__(self)
-        self.insert_section(0, _("Queue"),
+        if show_artist:
+            self.insert_section(0, _("Artist"),
+                                ArtistMenu(track))
+        self.insert_section(1, _("Queue"),
                             QueueMenu(track))
-        self.insert_section(1, _("Playlists"),
+        self.insert_section(2, _("Playlists"),
                             PlaylistsMenu(track))
         self.insert_section(3, _("Edit"),
                             EditMenu(track))
@@ -523,4 +470,4 @@ class TrackMenuPopover(Gtk.Popover):
             @param edit as Gtk.Edit
             @param track id as int
         """
-        Lp().tracks.set_uri(track_id, edit.get_text())
+        App().tracks.set_uri(track_id, edit.get_text())
