@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
 
 from gi.repository import Gtk, GLib, Gdk, Pango, GObject
 
-from lollypop.define import Lp, ArtSize, Type
+from lollypop.define import App, ArtSize, Type
 from lollypop.objects import Track, Album
 
 
@@ -60,9 +60,6 @@ class QueueRow(Gtk.ListBoxRow):
         self.__menu_button = Gtk.Button.new_from_icon_name(
                                                          "user-trash-symbolic",
                                                          Gtk.IconSize.MENU)
-        # Here a hack to make old Gtk version support min-height css attribute
-        # min-height = 24px, borders = 2px
-        self.__menu_button.set_property("height-request", 26)
         self.__menu_button.set_relief(Gtk.ReliefStyle.NONE)
         self.__menu_button.get_style_context().add_class("menu-button")
         self.__menu_button.get_style_context().add_class("track-menu-button")
@@ -196,11 +193,11 @@ class QueueRow(Gtk.ListBoxRow):
         """
         height = self.get_allocated_height()
         if y > height/2:
-            self.get_style_context().add_class("drag-up")
-            self.get_style_context().remove_class("drag-down")
-        else:
             self.get_style_context().remove_class("drag-up")
             self.get_style_context().add_class("drag-down")
+        else:
+            self.get_style_context().add_class("drag-up")
+            self.get_style_context().remove_class("drag-down")
 
     def __on_drag_leave(self, widget, context, time):
         """
@@ -217,7 +214,7 @@ class QueueRow(Gtk.ListBoxRow):
             Delete track from queue
             @param button as Gtk.Button
         """
-        Lp().player.del_from_queue(self.__id)
+        App().player.del_from_queue(self.__id)
         self.destroy()
 
     def __on_query_tooltip(self, widget, x, y, keyboard, tooltip):
@@ -278,9 +275,9 @@ class QueuePopover(Gtk.Popover):
         """
             Populate widget with queue rows
         """
-        if Lp().player.queue:
+        if App().player.queue:
             self.__clear_button.set_sensitive(True)
-        self.__add_items(list(Lp().player.queue))
+        self.__add_items(list(App().player.queue))
 
 #######################
 # PROTECTED           #
@@ -304,7 +301,7 @@ class QueuePopover(Gtk.Popover):
         for child in self.__view.get_children():
             child.destroy()
         if clear_queue:
-            Lp().player.clear_queue()
+            App().player.clear_queue()
 
     def __add_items(self, items, prev_album_id=None):
         """
@@ -313,10 +310,10 @@ class QueuePopover(Gtk.Popover):
         """
         if items and not self._stop:
             track_id = items.pop(0)
-            album_id = Lp().tracks.get_album_id(track_id)
+            album_id = App().tracks.get_album_id(track_id)
             row = self.__row_for_track_id(track_id)
             if album_id != prev_album_id:
-                surface = Lp().art.get_album_artwork(
+                surface = App().art.get_album_artwork(
                                         Album(album_id),
                                         ArtSize.MEDIUM,
                                         self.get_scale_factor())
@@ -342,11 +339,11 @@ class QueuePopover(Gtk.Popover):
             @param widget as Gtk.Widget
         """
         self._stop = False
-        height = Lp().window.get_size()[1]
+        height = App().window.get_size()[1]
         self.set_size_request(400, height*0.7)
         self.populate()
-        self._signal_id1 = Lp().player.connect("current-changed",
-                                               self.__on_current_changed)
+        self._signal_id1 = App().player.connect("current-changed",
+                                                self.__on_current_changed)
 
     def __on_unmap(self, widget):
         """
@@ -356,7 +353,7 @@ class QueuePopover(Gtk.Popover):
         self.__clear()
         self._stop = True
         if self._signal_id1 is not None:
-            Lp().player.disconnect(self._signal_id1)
+            App().player.disconnect(self._signal_id1)
             self._signal_id1 = None
 
     def __on_current_changed(self, player):
@@ -380,7 +377,7 @@ class QueuePopover(Gtk.Popover):
                 child.set_cover(None)
                 child.show_header(False)
             else:
-                surface = Lp().art.get_album_artwork(
+                surface = App().art.get_album_artwork(
                                         Album(track.album.id),
                                         ArtSize.MEDIUM,
                                         self.get_scale_factor())
@@ -402,8 +399,8 @@ class QueuePopover(Gtk.Popover):
             @param widget as Gtk.ListBox
             @param row as QueueRow
         """
-        if not Lp().player.locked:
-            Lp().player.load(Track(row.id))
+        if not App().player.locked:
+            App().player.load(Track(row.id))
             GLib.idle_add(row.destroy)
 
     def __on_track_moved(self, row, src, x, y):
@@ -429,7 +426,7 @@ class QueuePopover(Gtk.Popover):
             if child == row:
                 row_index = i
             if child.id == src:
-                Lp().player.del_from_queue(src, False)
+                App().player.del_from_queue(src, False)
                 child.disconnect_by_func(self.__on_child_destroyed)
                 child.destroy()
             else:
@@ -440,7 +437,7 @@ class QueuePopover(Gtk.Popover):
             if not up:
                 row_index += 1
             self.__view.insert(src_row, row_index)
-            Lp().player.insert_in_queue(src, row_index)
+            App().player.insert_in_queue(src, row_index)
         self.__update_headers()
 
     def __on_drag_data_received(self, widget, context, x, y, data, info, time):
