@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
 
 from gi.repository import Gtk, Gst
 
-from lollypop.define import Lp, WindowSize, LOLLYPOP_DATA_PATH
+from lollypop.define import App, WindowSize, LOLLYPOP_DATA_PATH
 from lollypop.toolbar_playback import ToolbarPlayback
 from lollypop.toolbar_info import ToolbarInfo
 from lollypop.toolbar_title import ToolbarTitle
@@ -45,10 +45,10 @@ class Toolbar(Gtk.HeaderBar):
         self.set_custom_title(self.__toolbar_title)
         self.pack_end(self.__toolbar_end)
 
-        Lp().player.connect("status-changed", self.__on_status_changed)
-        Lp().player.connect("current-changed", self.__on_current_changed)
-        Lp().player.connect("next-changed", self.__on_next_changed)
-        Lp().player.connect("prev-changed", self.__on_prev_changed)
+        App().player.connect("status-changed", self.__on_status_changed)
+        App().player.connect("current-changed", self.__on_current_changed)
+        App().player.connect("next-changed", self.__on_next_changed)
+        App().player.connect("prev-changed", self.__on_prev_changed)
 
     def do_get_preferred_height(self):
         """
@@ -78,26 +78,17 @@ class Toolbar(Gtk.HeaderBar):
             @param window width as int
         """
         width = self.__toolbar_playback.get_preferred_width()[1]
-        if window_width < WindowSize.MONSTER:
-            self.__toolbar_info.hide()
-        else:
-            self.__toolbar_info.show()
-        if window_width < WindowSize.BIG:
-            self.__toolbar_end.hide()
-        else:
-            width += self.__toolbar_end.get_preferred_width()[1]
-            self.__toolbar_end.show()
+        width += self.__toolbar_end.get_preferred_width()[1]
         window = self.get_window()
         if window is not None:
             available = window.get_width() - width
             if available > 0:
-                if window_width >= WindowSize.MONSTER:
-                    title = available/2
+                if window_width >= WindowSize.MEDIUM:
+                    title = available / 2
                 else:
                     title = available
                 self.__toolbar_title.set_width(title)
-                if window_width >= WindowSize.MEDIUM:
-                    self.__toolbar_info.set_width((available-title)/2)
+                self.__toolbar_info.set_width((available-title)/2)
             self.__width = window.get_width()
 
     def update_position(self, value=None):
@@ -120,7 +111,7 @@ class Toolbar(Gtk.HeaderBar):
             Mark toolbar with previously saved position
         """
         try:
-            if Lp().settings.get_value("save-state"):
+            if App().settings.get_value("save-state"):
                 from pickle import load
                 position = load(open(LOLLYPOP_DATA_PATH + "/position.bin",
                                 "rb"))
@@ -128,25 +119,37 @@ class Toolbar(Gtk.HeaderBar):
         except Exception as e:
             print("Toolbar::restore_state():", e)
 
-    def search(self, search):
+    @property
+    def end(self):
         """
-            Search item
-            @param search as str
+            Return end toolbar
+            @return ToolbarEnd
         """
-        self.__toolbar_end.search(search)
-
-    def show_hide_volume_control(self):
-        """
-            Show/Hide volume control
-        """
-        self.__toolbar_title.show_hide_volume_control()
+        return self.__toolbar_end
 
     @property
-    def artsize(self):
+    def info(self):
         """
-            Art size as int
+            Return info toolbar
+            @return ToolbarInfo
         """
-        return self.__toolbar_info.artsize
+        return self.__toolbar_info
+
+    @property
+    def title(self):
+        """
+            Return title toolbar
+            @return ToolbarTitle
+        """
+        return self.__toolbar_title
+
+    @property
+    def playback(self):
+        """
+            Return playback toolbar
+            @return ToolbarPlayback
+        """
+        return self.__toolbar_playback
 
 #######################
 # PRIVATE             #
@@ -156,7 +159,7 @@ class Toolbar(Gtk.HeaderBar):
             Update toolbar
             @param player as Player
         """
-        debug("Toolbar::_on_current_changed")
+        debug("Toolbar::_on_current_changed()")
         self.__toolbar_playback.on_current_changed(player)
         self.__toolbar_info.on_current_changed(player)
         self.__toolbar_title.on_current_changed(player)
