@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,7 +16,7 @@ from gettext import gettext as _
 
 from lollypop.pop_next import NextPopover
 from lollypop.touch_helper import TouchHelper
-from lollypop.define import Lp, Shuffle, Type, NextContext
+from lollypop.define import App, Shuffle, Type, NextContext
 
 
 class PartyPopover(Gtk.Popover):
@@ -43,14 +43,14 @@ class PartyPopover(Gtk.Popover):
         scrolled.add(party_grid)
         scrolled.show()
         self.add(scrolled)
-        size = Lp().window.get_size()
+        size = App().window.get_size()
         self.set_size_request(-1,
                               size[1]*0.6)
 
-        genres = Lp().genres.get()
+        genres = App().genres.get()
         genres.insert(0, (Type.POPULARS, _("Populars")))
         genres.insert(1, (Type.RECENTS, _("Recently added")))
-        ids = Lp().player.get_party_ids()
+        ids = App().player.get_party_ids()
         i = 0
         x = 0
         for genre_id, genre in genres:
@@ -86,7 +86,7 @@ class PartyPopover(Gtk.Popover):
             @param widget as Gtk.Switch
             @param state as bool, genre id as int
         """
-        ids = Lp().player.get_party_ids()
+        ids = App().player.get_party_ids()
         if state:
             try:
                 ids.append(genre_id)
@@ -97,9 +97,9 @@ class PartyPopover(Gtk.Popover):
                 ids.remove(genre_id)
             except:
                 pass
-        Lp().settings.set_value("party-ids",  GLib.Variant("ai", ids))
-        Lp().player.set_party_ids()
-        Lp().player.set_next()
+        App().settings.set_value("party-ids",  GLib.Variant("ai", ids))
+        App().player.set_party_ids()
+        App().player.set_next()
 
 
 class ToolbarEnd(Gtk.Bin):
@@ -115,9 +115,7 @@ class ToolbarEnd(Gtk.Bin):
         self.connect("show", self.__on_show)
         self.connect("hide", self.__on_hide)
         self.set_hexpand(True)
-        self.__next_popover = NextPopover()
         self.__search = None
-        self.__next_was_inhibited = False
         self.__timeout_id = None
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Lollypop/ToolbarEnd.ui")
@@ -125,22 +123,22 @@ class ToolbarEnd(Gtk.Bin):
 
         self.add(builder.get_object("end"))
 
-        self.__grid_next = builder.get_object("grid-next")
-
         self.__shuffle_button = builder.get_object("shuffle-button")
         self.__shuffle_image = builder.get_object("shuffle-button-image")
         shuffleAction = Gio.SimpleAction.new("shuffle-button", None)
         shuffleAction.connect("activate", self.__activate_shuffle_button)
-        Lp().add_action(shuffleAction)
-        Lp().set_accels_for_action("app.shuffle-button", ["<Control>r"])
-        Lp().settings.connect("changed::shuffle", self.__on_playback_changed)
-        Lp().settings.connect("changed::playback", self.__on_playback_changed)
+        App().add_action(shuffleAction)
+        App().set_accels_for_action("app.shuffle-button", ["<Control>r"])
+        App().settings.connect("changed::shuffle", self.__on_playback_changed)
+        App().settings.connect("changed::playback", self.__on_playback_changed)
 
         self.__party_button = builder.get_object("party-button")
         party_action = Gio.SimpleAction.new("party", None)
         party_action.connect("activate", self.__activate_party_button)
-        Lp().add_action(party_action)
-        Lp().set_accels_for_action("app.party", ["<Control>p"])
+        App().add_action(party_action)
+        App().set_accels_for_action("app.party", ["<Control>p"])
+        self.__next_popover = NextPopover()
+        self.__next_popover.set_relative_to(self.__party_button)
 
         self.__search_button = builder.get_object("search-button")
         self.__helper = TouchHelper(self.__search_button,
@@ -155,8 +153,21 @@ class ToolbarEnd(Gtk.Bin):
         self.__list_button.connect("query-tooltip",
                                    self.__on_list_button_query_tooltip)
         self.__list_popover = None
-        Lp().player.connect("party-changed", self.__on_party_changed)
-        Lp().player.connect("lock-changed", self.__on_lock_changed)
+        App().player.connect("party-changed", self.__on_party_changed)
+        App().player.connect("lock-changed", self.__on_lock_changed)
+
+    def set_minimal(self, b):
+        """
+            Set minimal, hide some widget for minimal player
+            @param b as bool
+        """
+        if b:
+            self.__list_button.hide()
+            self.__next_popover.hide()
+        else:
+            self.__list_button.show()
+            if self.__next_popover.should_be_shown():
+                self.__next_popover.show()
 
     def setup_menu(self, menu):
         """
@@ -176,22 +187,15 @@ class ToolbarEnd(Gtk.Bin):
             # via Fullscreen class, so check button state
             self.__party_button.set_active(player.is_party)
 
-    def on_next_changed(self, player, force=False):
+    def on_next_changed(self, player):
         """
             Show next popover
             @param player as Player
-            @param force to show the popover
         """
-        # Do not show popover is this menu is active
-        # or if we are hidden
-        if self.__shuffle_button.get_active() or\
-           not self.__grid_next.is_visible():
-            return
-        if self.__next_popover.should_be_shown() or force:
+        if self.__next_popover.should_be_shown():
             if self.__next_popover.is_visible():
                 self.__next_popover.update()
             else:
-                self.__next_popover.set_relative_to(self.__grid_next)
                 self.__next_popover.show()
         else:
             self.__next_popover.hide()
@@ -204,22 +208,30 @@ class ToolbarEnd(Gtk.Bin):
         self.__on_search_short([])
         self.__search.set_text(search)
 
+    def show_list_popover(self, button):
+        """
+            Show a popover with current playlist
+            @param button as Gtk.Button
+        """
+        if App().player.current_track.id == Type.EXTERNALS:
+            from lollypop.pop_externals import ExternalsPopover
+            popover = ExternalsPopover()
+        elif App().player.queue:
+            from lollypop.pop_queue import QueuePopover
+            popover = QueuePopover()
+        elif App().player.get_playlist_ids():
+            from lollypop.pop_playlists import PlaylistsPopover
+            popover = PlaylistsPopover()
+        else:
+            from lollypop.pop_albums import AlbumsPopover
+            popover = AlbumsPopover()
+        popover.set_relative_to(button)
+        popover.show()
+        return popover
+
 #######################
 # PROTECTED           #
 #######################
-    def _on_shuffle_button_toggled(self, button):
-        """
-            Hide next popover
-            @param button as Gtk.Button
-        """
-        if button.get_active():
-            self.__next_popover.hide()
-            self.__next_popover.inhibit(True)
-        else:
-            self.__next_popover.inhibit(False)
-            if self.__next_popover.should_be_shown():
-                self.__next_popover.show()
-
     def _on_party_button_toggled(self, button):
         """
             Set party mode on if party button active
@@ -227,12 +239,12 @@ class ToolbarEnd(Gtk.Bin):
         """
         active = self.__party_button.get_active()
         self.__shuffle_button.set_sensitive(not active)
-        if not Lp().gtk_application_prefer_dark_theme and\
-                not Lp().settings.get_value("dark-ui"):
+        if not App().gtk_application_prefer_dark_theme and\
+                not App().settings.get_value("dark-ui"):
             settings = Gtk.Settings.get_default()
             settings.set_property("gtk-application-prefer-dark-theme", active)
-        Lp().player.set_party(active)
-        self.on_next_changed(Lp().player)
+        App().player.set_party(active)
+        self.on_next_changed(App().player)
 
     def _on_party_press_event(self, eventbox, event):
         """
@@ -256,30 +268,9 @@ class ToolbarEnd(Gtk.Bin):
         """
         if self.__list_popover is not None:
             return
-        self.__next_was_inhibited = self.__next_popover.inhibited
         self.__next_popover.hide()
         self.__next_popover.inhibit(True)
-        if Lp().player.current_track.id == Type.EXTERNALS:
-            from lollypop.pop_externals import ExternalsPopover
-            self.__list_popover = ExternalsPopover()
-            self.__list_popover.set_relative_to(self.__list_button)
-            self.__list_popover.populate()
-            self.__list_popover.show()
-        elif Lp().player.queue:
-            from lollypop.pop_queue import QueuePopover
-            self.__list_popover = QueuePopover()
-            self.__list_popover.set_relative_to(self.__list_button)
-            self.__list_popover.show()
-        elif Lp().player.get_user_playlist_ids():
-            from lollypop.pop_playlists import PlaylistsPopover
-            self.__list_popover = PlaylistsPopover()
-            self.__list_popover.set_relative_to(self.__list_button)
-            self.__list_popover.show()
-        else:
-            from lollypop.pop_albums import AlbumsPopover
-            self.__list_popover = AlbumsPopover()
-            self.__list_popover.set_relative_to(self.__list_button)
-            self.__list_popover.show()
+        self.__list_popover = self.show_list_popover(widget)
         self.__list_popover.connect("closed", self.__on_list_popover_closed)
         return True
 
@@ -288,18 +279,17 @@ class ToolbarEnd(Gtk.Bin):
 #######################
     def __on_search_long(self, args):
         """
-            Show current track menu
+            Filter view
             @param args as []
         """
-        if Lp().window.view is not None:
-            Lp().window.view.set_search_mode()
+        if App().window.container.view is not None:
+            App().window.container.view.enable_filter()
 
     def __on_search_short(self, args):
         """
             Show search popover
             @param args as []
         """
-        self.__next_was_inhibited = self.__next_popover.inhibited
         self.__next_popover.hide()
         self.__next_popover.inhibit(True)
         if self.__search is None:
@@ -313,8 +303,8 @@ class ToolbarEnd(Gtk.Bin):
         """
             Set shuffle icon
         """
-        shuffle = Lp().settings.get_enum("shuffle")
-        repeat = Lp().settings.get_enum("playback")
+        shuffle = App().settings.get_enum("shuffle")
+        repeat = App().settings.get_enum("playback")
         if repeat == NextContext.REPEAT_TRACK:
             self.__shuffle_image.get_style_context().remove_class("selected")
             self.__shuffle_image.set_from_icon_name(
@@ -347,7 +337,7 @@ class ToolbarEnd(Gtk.Bin):
             @param param as GLib.Variant
         """
         self.__party_button.set_active(not self.__party_button.get_active())
-        Lp().window.responsive_design()
+        App().window.responsive_design()
 
     def __activate_shuffle_button(self, action=None, param=None):
         """
@@ -397,10 +387,8 @@ class ToolbarEnd(Gtk.Bin):
             Restore next popover if needed
             @param popover as Gtk.Popover
         """
-        if not self.__next_was_inhibited:
-            self.__next_popover.inhibit(False)
+        self.__next_popover.inhibit(False)
         if self.__next_popover.should_be_shown():
-            self.__next_popover.set_relative_to(self.__grid_next)
             self.__next_popover.show()
 
     def __on_show(self, widget):
@@ -426,11 +414,11 @@ class ToolbarEnd(Gtk.Bin):
             @param keyboard as bool
             @param tooltip as Gtk.Tooltip
         """
-        if Lp().player.current_track.id == Type.EXTERNALS:
+        if App().player.current_track.id == Type.EXTERNALS:
             widget.set_tooltip_text(_("External tracks"))
-        elif Lp().player.queue:
+        elif App().player.queue:
             widget.set_tooltip_text(_("Queue"))
-        elif Lp().player.get_user_playlist_ids():
+        elif App().player.get_playlist_ids():
             widget.set_tooltip_text(_("Playing playlists"))
         else:
             widget.set_tooltip_text(_("Playing albums"))
