@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,9 +15,42 @@ from gi.repository import Gio, GLib
 from gettext import gettext as _
 import unicodedata
 
-from lollypop.helper_task import TaskHelper
-from lollypop.define import Lp, Type
-from lollypop.objects import Track
+from lollypop.define import App
+
+
+def set_proxy_from_gnome():
+    """
+        Set proxy settings from GNOME
+    """
+    try:
+        proxy = Gio.Settings.new("org.gnome.system.proxy")
+        mode = proxy.get_value("mode").get_string()
+        if mode == "manual":
+            no_http_proxy = True
+            http = Gio.Settings.new("org.gnome.system.proxy.http")
+            https = Gio.Settings.new("org.gnome.system.proxy.https")
+            h = http.get_value("host").get_string()
+            p = http.get_value("port").get_int32()
+            hs = https.get_value("host").get_string()
+            ps = https.get_value("port").get_int32()
+            if h != "" and p != 0:
+                no_http_proxy = False
+                GLib.setenv("http_proxy", "http://%s:%s" % (h, p), True)
+            if hs != "" and ps != 0:
+                no_http_proxy = False
+                GLib.setenv("https_proxy", "http://%s:%s" % (hs, ps), True)
+            if no_http_proxy:
+                socks = Gio.Settings.new("org.gnome.system.proxy.socks")
+                h = socks.get_value("host").get_string()
+                p = socks.get_value("port").get_int32()
+                # Set socks proxy
+                if h != "" and p != 0:
+                    import socket
+                    import socks
+                    socks.set_default_proxy(socks.SOCKS4, h, p)
+                    socket.socket = socks.socksocket
+    except Exception as e:
+        print("set_proxy_from_gnome():", e)
 
 
 def debug(str):
@@ -25,7 +58,7 @@ def debug(str):
         Print debug
         @param debug as str
     """
-    if Lp().debug is True:
+    if App().debug is True:
         print(str)
 
 
@@ -35,7 +68,7 @@ def get_network_available():
         @return bool
     """
     return Gio.NetworkMonitor.get_default().get_network_available() and\
-        Lp().settings.get_value("network-access")
+        App().settings.get_value("network-access")
 
 
 def noaccents(string):
@@ -125,7 +158,7 @@ def format_artist_name(name):
         Return formated artist name
         @param str
     """
-    if not Lp().settings.get_value("smart-artist-sort"):
+    if not App().settings.get_value("smart-artist-sort"):
         return name
     # Handle language ordering
     # Translators: Add here words that shoud be ignored for artist sort order
@@ -171,57 +204,9 @@ def is_readonly(uri):
     return not info.get_attribute_boolean("access::can-write")
 
 
-def is_loved(track_id):
-    """
-        Check if object is in loved playlist
-        @return bool
-    """
-    return Lp().playlists.exists_track(Type.LOVED,
-                                       track_id)
-
-
 def remove_static_genres(genre_ids):
     """
         Remove static genre ids
         @param genre ids as [int]
     """
     return [item for item in genre_ids if item >= 0]
-
-
-def set_loved(track_id, loved):
-    """
-        Add or remove track from loved playlist
-        @param track_id
-        @param loved Add to loved playlist if `True`; remove if `False`
-    """
-    if not is_loved(track_id):
-        if loved:
-            Lp().playlists.add_tracks(Type.LOVED,
-                                      [Track(track_id)])
-            if Lp().lastfm is not None:
-                helper = TaskHelper()
-                helper.run(_set_loved_on_lastfm, track_id, True)
-    else:
-        if not loved:
-            Lp().playlists.remove_tracks(Type.LOVED,
-                                         [Track(track_id)])
-            if Lp().lastfm is not None:
-                helper = TaskHelper()
-                helper.run(_set_loved_on_lastfm, track_id, False)
-
-
-def _set_loved_on_lastfm(track_id, loved):
-    """
-        Add or remove track from loved playlist on Last.fm
-        @param track_id
-        @param loved Add to loved playlist if `True`; remove if `False`
-    """
-    # Love the track on lastfm
-    if Gio.NetworkMonitor.get_default().get_network_available() and\
-            Lp().lastfm.is_auth:
-        title = Lp().tracks.get_name(track_id)
-        artists = ", ".join(Lp().tracks.get_artists(track_id))
-        if loved:
-            Lp().lastfm.love(artists, title)
-        else:
-            Lp().lastfm.unlove(artists, title)
