@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,7 +16,7 @@ from gettext import gettext as _
 from math import pi
 from random import choice
 
-from lollypop.define import Lp, ArtSize, Shuffle, Type
+from lollypop.define import App, ArtSize, Shuffle, Type
 from lollypop.utils import get_network_available
 from lollypop.objects import Album
 from lollypop.pop_info import InfoPopover
@@ -52,7 +52,7 @@ class ArtistView(ArtistAlbumsView):
         self.__add_button = builder.get_object("add-button")
         self.__play_button = builder.get_object("play-button")
         self.__grid = builder.get_object("header")
-        if Lp().lastfm is None:
+        if App().lastfm is None:
             builder.get_object("lastfm-button").hide()
         elif not get_network_available():
             builder.get_object("lastfm-button").set_sensitive(False)
@@ -66,13 +66,13 @@ class ArtistView(ArtistAlbumsView):
 
         self.__scale_factor = self.__artwork.get_scale_factor()
         self.__set_artwork()
-        self.__set_add_icon()
-        self.__on_lock_changed(Lp().player)
+        self.__on_album_changed(App().player)
+        self.__on_lock_changed(App().player)
 
         artists = []
         for artist_id in artist_ids:
-            artists.append(Lp().artists.get_name(artist_id))
-        if Lp().settings.get_value("artist-artwork"):
+            artists.append(App().artists.get_name(artist_id))
+        if App().settings.get_value("artist-artwork"):
             self.__label.set_markup(
                                 '<span size="x-large" weight="bold">' +
                                 GLib.markup_escape_text(", ".join(artists)) +
@@ -89,7 +89,7 @@ class ArtistView(ArtistAlbumsView):
         """
         widget = None
         for child in self._albumbox.get_children():
-            if child.id == Lp().player.current_track.album.id:
+            if child.id == App().player.current_track.album.id:
                 widget = child
                 break
         if widget is not None:
@@ -129,7 +129,7 @@ class ArtistView(ArtistAlbumsView):
         if InfoPopover.should_be_shown() and self._artist_ids:
             eventbox.get_window().set_cursor(Gdk.Cursor(Gdk.CursorType.HAND2))
 
-    def _on_artwork_realize(self, eventbox):
+    def _on_artwork_box_realize(self, eventbox):
         """
             Change cursor on image
             @param eventbox as Gtk.EventBox
@@ -143,7 +143,8 @@ class ArtistView(ArtistAlbumsView):
             @param event as Gdk.Event
         """
         if InfoPopover.should_be_shown() and self._artist_ids:
-            pop = InfoPopover(self._artist_ids, Type.NONE)
+            pop = InfoPopover(self._artist_ids)
+            pop.set_view_type(Type.NONE)
             pop.set_relative_to(eventbox)
             pop.show()
 
@@ -163,53 +164,49 @@ class ArtistView(ArtistAlbumsView):
             Play artist albums
         """
         try:
-            if Lp().player.is_party:
-                Lp().player.set_party(False)
-            album_ids = Lp().albums.get_ids(self._artist_ids,
-                                            self._genre_ids)
+            if App().player.is_party:
+                App().player.set_party(False)
+            album_ids = App().albums.get_ids(self._artist_ids,
+                                             self._genre_ids)
             if album_ids:
                 track = None
-                if Lp().settings.get_enum("shuffle") == Shuffle.TRACKS:
+                if App().settings.get_enum("shuffle") == Shuffle.TRACKS:
                     album_id = choice(album_ids)
                     album_tracks = Album(album_id).tracks
                     if album_tracks:
                         track = choice(album_tracks)
                 else:
-                    album_id = choice(album_ids)
+                    if App().settings.get_enum("shuffle") == Shuffle.ALBUMS:
+                        album_id = choice(album_ids)
+                    else:
+                        album_id = album_ids[0]
                     album_tracks = Album(album_id).tracks
                     if album_tracks:
                         track = album_tracks[0]
-                if track is not None:
-                    Lp().player.load(track)
-                    Lp().player.set_albums(track.id, self._artist_ids,
-                                           self._genre_ids)
-                    self.__set_add_icon()
-        except:
-            pass  # Artist not available anymore for this context
+                if track.id is not None:
+                    App().player.play_albums(track,
+                                             self._genre_ids,
+                                             self._artist_ids)
+            self.__update_icon(False)
+        except Exception as e:
+            print("ArtistView::_on_play_clicked:", e)
 
     def _on_add_clicked(self, widget):
         """
             Add artist albums
         """
         try:
-            albums = Lp().albums.get_ids(self._artist_ids, self._genre_ids)
-            if self.__add_button.get_image().get_icon_name(
-                                                   )[0] == "list-add-symbolic":
-                for album_id in albums:
-                    album = Album(album_id)
-                    # If playing and no albums, play it
-                    if not Lp().player.has_album(album):
-                        if Lp().player.is_playing and\
-                                not Lp().player.get_albums():
-                            Lp().player.play_album(album)
-                        else:
-                            Lp().player.add_album(album)
-            else:
-                for album_id in albums:
-                    album = Album(album_id)
-                    if Lp().player.has_album(album):
-                        Lp().player.remove_album(album)
-            self.__set_add_icon()
+            album_ids = App().albums.get_ids(self._artist_ids, self._genre_ids)
+            player_ids = [album.id for album in App().player.albums]
+            icon_name = self.__add_button.get_image().get_icon_name()[0]
+            add = icon_name == "list-add-symbolic"
+            for album_id in album_ids:
+                in_player = album_id in player_ids
+                if add and not in_player:
+                    App().player.add_album(Album(album_id))
+                elif not add and in_player:
+                    App().player.remove_album(Album(album_id))
+            self.__update_icon(not add)
         except:
             pass  # Artist not available anymore for this context
 
@@ -235,9 +232,13 @@ class ArtistView(ArtistAlbumsView):
             @param image as Gtk.Image
             @param ctx as cairo.Context
         """
+        internal_scale_factor = 1
+        scale_factor = image.get_scale_factor()
+        if not App().window.container.is_paned_stack:
+            internal_scale_factor = 2
+        scale_factor = scale_factor * internal_scale_factor
         # Update image if scale factor changed
-        if self.__scale_factor != image.get_scale_factor():
-            self.__scale_factor = image.get_scale_factor()
+        if self.__scale_factor != scale_factor:
             self.__set_artwork()
         if not image.is_drawable():
             return
@@ -247,14 +248,14 @@ class ArtistView(ArtistAlbumsView):
             if pixbuf is None:
                 return
             surface = Gdk.cairo_surface_create_from_pixbuf(
-                                                         pixbuf,
-                                                         self.__scale_factor,
-                                                         None)
+                                             pixbuf,
+                                             self.__artwork.get_scale_factor(),
+                                             None)
         else:
             surface = image.props.surface
 
         ctx.translate(2, 2)
-        size = ArtSize.ARTIST_SMALL * 2 - 4
+        size = ArtSize.ARTIST_SMALL * internal_scale_factor - 4
         ctx.new_sub_path()
         radius = size / 2
         ctx.arc(size/2, size/2, radius, 0, 2 * pi)
@@ -294,11 +295,16 @@ class ArtistView(ArtistAlbumsView):
             Set artist artwork
         """
         artwork_height = 0
-        if Lp().settings.get_value("artist-artwork"):
+        internal_scale_factor = 1
+        self.__scale_factor = self.__artwork.get_scale_factor()
+        if not App().window.container.is_paned_stack:
+            internal_scale_factor = 2
+        self.__scale_factor *= internal_scale_factor
+        if App().settings.get_value("artist-artwork"):
             if len(self._artist_ids) == 1 and\
-                    Lp().settings.get_value("artist-artwork"):
-                artist = Lp().artists.get_name(self._artist_ids[0])
-                size = ArtSize.ARTIST_SMALL * 2 * self.__scale_factor
+                    App().settings.get_value("artist-artwork"):
+                artist = App().artists.get_name(self._artist_ids[0])
+                size = ArtSize.ARTIST_SMALL * self.__scale_factor
                 for suffix in ["lastfm", "spotify", "wikipedia"]:
                     uri = InfoCache.get_artwork(artist, suffix, size)
                     if uri is not None:
@@ -317,9 +323,12 @@ class ArtistView(ArtistAlbumsView):
                                                                        None)
                         stream.close()
                         surface = Gdk.cairo_surface_create_from_pixbuf(
-                                            pixbuf, self.__scale_factor, None)
+                                            pixbuf,
+                                            self.__artwork.get_scale_factor(),
+                                            None)
                         self.__artwork.set_from_surface(surface)
-                        artwork_height = ArtSize.ARTIST_SMALL * 2
+                        artwork_height = ArtSize.ARTIST_SMALL *\
+                            internal_scale_factor
                         self.__artwork.get_style_context().remove_class(
                                                                 "artwork-icon")
                         self.__artwork.show()
@@ -330,7 +339,7 @@ class ArtistView(ArtistAlbumsView):
                 self.__artwork.set_from_icon_name(
                                             "avatar-default-symbolic",
                                             Gtk.IconSize.DND)
-                artwork_height = 32
+                artwork_height = 16 * internal_scale_factor
                 self.__artwork.get_style_context().add_class("artwork-icon")
                 self.__artwork.show()
                 self.__artwork_box.show()
@@ -353,7 +362,7 @@ class ArtistView(ArtistAlbumsView):
         """
         found = False
         for child in self._get_children():
-            if child.id == Lp().player.current_track.album.id:
+            if child.id == App().player.current_track.album.id:
                 found = True
                 break
         if found:
@@ -361,41 +370,42 @@ class ArtistView(ArtistAlbumsView):
         else:
             self.__jump_button.set_sensitive(False)
 
-    def __set_add_icon(self):
+    def __update_icon(self, add):
         """
-            Set add icon based on player albums
+            Set icon for Artist +/-
+            @param add as bool
         """
-        albums = Lp().albums.get_ids(self._artist_ids, self._genre_ids)
-        player_albums = Lp().player.get_albums()
-        if len(set(albums) & set(player_albums)) == len(albums):
+        if add:
             # Translators: artist context
-            self.__add_button.set_tooltip_text(_("Remove"))
-            self.__add_button.get_image().set_from_icon_name(
-                                                        "list-remove-symbolic",
-                                                        Gtk.IconSize.MENU)
-        else:
-            # Translators: artist context
-            self.__add_button.set_tooltip_text(_("Add"))
+            self.__add_button.set_tooltip_text(_("Add to current playlist"))
             self.__add_button.get_image().set_from_icon_name(
                                                            "list-add-symbolic",
                                                            Gtk.IconSize.MENU)
+        else:
+            # Translators: artist context
+            self.__add_button.set_tooltip_text(
+                                          _("Remove from current playlist"))
+            self.__add_button.get_image().set_from_icon_name(
+                                                        "list-remove-symbolic",
+                                                        Gtk.IconSize.MENU)
 
     def __on_realize(self, widget):
         """
             Connect signal
             @param widget as Gtk.Widget
         """
-        self.__art_signal_id = Lp().art.connect(
-                                              "artist-artwork-changed",
-                                              self.__on_artist_artwork_changed)
-        self.__party_signal_id = Lp().player.connect("party-changed",
-                                                     self.__on_album_changed)
-        self.__added_signal_id = Lp().player.connect("album-added",
-                                                     self.__on_album_changed)
-        self.__removed_signal_id = Lp().player.connect("album-removed",
-                                                       self.__on_album_changed)
-        self.__lock_signal_id = Lp().player.connect("lock-changed",
-                                                    self.__on_lock_changed)
+        player = App().player
+        art = App().art
+        self.__art_signal_id = art.connect("artist-artwork-changed",
+                                           self.__on_artist_artwork_changed)
+        self.__party_signal_id = player.connect("party-changed",
+                                                self.__on_album_changed)
+        self.__added_signal_id = player.connect("album-added",
+                                                self.__on_album_changed)
+        self.__removed_signal_id = player.connect("album-removed",
+                                                  self.__on_album_changed)
+        self.__lock_signal_id = player.connect("lock-changed",
+                                               self.__on_lock_changed)
 
     def __on_unrealize(self, widget):
         """
@@ -403,28 +413,30 @@ class ArtistView(ArtistAlbumsView):
             @param widget as Gtk.Widget
         """
         if self.__art_signal_id is not None:
-            Lp().art.disconnect(self.__art_signal_id)
+            App().art.disconnect(self.__art_signal_id)
             self.__art_signal_id = None
         if self.__party_signal_id is not None:
-            Lp().player.disconnect(self.__party_signal_id)
+            App().player.disconnect(self.__party_signal_id)
             self.__party_signal_id = None
         if self.__added_signal_id is not None:
-            Lp().player.disconnect(self.__added_signal_id)
+            App().player.disconnect(self.__added_signal_id)
             self.__added_signal_id = None
         if self.__removed_signal_id is not None:
-            Lp().player.disconnect(self.__removed_signal_id)
+            App().player.disconnect(self.__removed_signal_id)
             self.__removed_signal_id = None
         if self.__lock_signal_id is not None:
-            Lp().player.disconnect(self.__lock_signal_id)
+            App().player.disconnect(self.__lock_signal_id)
             self.__lock_signal_id = None
 
-    def __on_album_changed(self, player, unused):
+    def __on_album_changed(self, player, album_id=None):
         """
-            Update add icon
+            Update icon
             @param player as Player
-            @param unused
+            @param album_id as int
         """
-        self.__set_add_icon()
+        albums = App().albums.get_ids(self._artist_ids, self._genre_ids)
+        album_ids = [album.id for album in App().player.albums]
+        self.__update_icon(len(set(albums) & set(album_ids)) != len(albums))
 
     def __on_lock_changed(self, player):
         """
@@ -440,7 +452,7 @@ class ArtistView(ArtistAlbumsView):
             @param art as Art
             @param prefix as str
         """
-        artist = Lp().artists.get_name(self._artist_ids[0])
+        artist = App().artists.get_name(self._artist_ids[0])
         if prefix == artist:
             self.__artwork.clear()
             self.__set_artwork()
