@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,7 +16,7 @@ from gettext import gettext as _
 
 from lollypop.view import LazyLoadingView, View
 from lollypop.view_container import ViewContainer
-from lollypop.define import Lp, Type, ArtSize
+from lollypop.define import App, Type, ArtSize
 from lollypop.objects import Track
 from lollypop.widgets_album_detailed import AlbumDetailedWidget
 
@@ -84,13 +84,32 @@ class ArtistAlbumsView(LazyLoadingView):
             self.set_sensitive(False)
             self._albumbox.add(label)
 
+    def lazy_loading(self, widgets=[], scroll_value=0):
+        """
+            Load the view in a lazy way
+            @param widgets as [AlbumSimpleWidgets]
+            @param scroll_value as float
+        """
+        widget = None
+        if self._stop or self._scroll_value != scroll_value:
+            return
+        if widgets:
+            widget = widgets.pop(0)
+            self._lazy_queue.remove(widget)
+        elif self._lazy_queue:
+            widget = self._lazy_queue.pop(0)
+        if widget is not None:
+            widget.connect("populated", self._on_populated,
+                           widgets, scroll_value)
+            widget.populate()
+
     def jump_to_current(self):
         """
             Jump to current album
         """
         widget = None
         for child in self._albumbox.get_children():
-            if child.id == Lp().player.current_track.album.id:
+            if child.id == App().player.current_track.album.id:
                 widget = child
                 break
         if widget is not None:
@@ -146,27 +165,6 @@ class ArtistAlbumsView(LazyLoadingView):
 #######################
 # PRIVATE             #
 #######################
-    def lazy_loading(self, widgets=[], scroll_value=0):
-        """
-            Load the view in a lazy way:
-                - widgets first
-                - _waiting_init then
-            @param widgets as [AlbumSimpleWidgets]
-            @param scroll_value as float
-        """
-        widget = None
-        if self._stop or self._scroll_value != scroll_value:
-            return
-        if widgets:
-            widget = widgets.pop(0)
-            self._lazy_queue.remove(widget)
-        elif self._lazy_queue:
-            widget = self._lazy_queue.pop(0)
-        if widget is not None:
-            widget.connect("populated", self._on_populated,
-                           widgets, scroll_value)
-            widget.populate()
-
     def __add_albums(self, albums):
         """
             Pop an album and add it to the view,
@@ -289,10 +287,10 @@ class CurrentArtistAlbumsView(ViewContainer):
         albums = []
         if self.__track.album.artist_ids[0] == Type.COMPILATIONS:
             albums += [self.__track.album.id]
-        albums += Lp().artists.get_albums(self.__get_artist_ids())
+        albums += App().artists.get_albums(self.__get_artist_ids())
         # Charts album playing
-        if Lp().player.current_track.album.id not in albums:
-            albums.append(Lp().player.current_track.album.id)
+        if App().player.current_track.album.id not in albums:
+            albums.append(App().player.current_track.album.id)
         return albums
 
     def __on_populated(self, view, spinner):
