@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -13,6 +13,7 @@
 from gi.repository import Gtk, GLib
 
 from lollypop.view_device import DeviceView
+from lollypop.view import View
 from lollypop.utils import debug
 
 
@@ -38,7 +39,9 @@ class ViewContainer(Gtk.Stack):
             @param view as new View
         """
         for child in self.get_children():
-            if child != view and not isinstance(child, DeviceView):
+            if child != view\
+                    and not isinstance(child, DeviceView)\
+                    and isinstance(child, View):
                 # Delayed destroy as we may have an animation running
                 # Gtk.StackTransitionType.CROSSFADE
                 GLib.timeout_add(self.__duration*5,
