@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,7 +16,7 @@ from gettext import gettext as _
 import re
 
 from lollypop.view import View
-from lollypop.define import Lp
+from lollypop.define import App
 from lollypop.widgets_device import DeviceManagerWidget
 
 
@@ -122,6 +122,14 @@ class DeviceView(View):
         """
         return self.__device_widget.is_syncing()
 
+    @property
+    def device(self):
+        """
+            Get device for view
+            @return Device
+        """
+        return self.__device
+
 #######################
 # PROTECTED           #
 #######################
@@ -142,7 +150,7 @@ class DeviceView(View):
         """
         if self.__device_widget.is_syncing():
             self.__device_widget.cancel_sync()
-        elif not Lp().window.progress.is_visible():
+        elif not App().window.container.progress.is_visible():
             self.__memory_combo.hide()
             self.__syncing_btn.set_label(_("Cancel synchronization"))
             self.__device_widget.sync()
