@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -10,14 +10,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GLib
+from gi.repository import Gtk, GLib, Gdk
 
 from gettext import gettext as _
 
 from lollypop.view import View
 from lollypop.widgets_playlist import PlaylistsWidget, PlaylistEditWidget
 from lollypop.widgets_playlist import PlaylistsManagerWidget
-from lollypop.define import Lp, Type
+from lollypop.define import App, Type, ArtSize
 from lollypop.objects import Track
 
 
@@ -33,24 +33,26 @@ class PlaylistsView(View):
             @param editable as bool
         """
         View.__init__(self, True)
+        self.__autoscroll_timeout_id = None
+        self.__prev_animated_rows = []
         self.__tracks = []
         self.__playlist_ids = playlist_ids
-        self.__signal_id1 = Lp().playlists.connect("playlist-add",
-                                                   self.__on_playlist_add)
-        self.__signal_id2 = Lp().playlists.connect("playlist-del",
-                                                   self.__on_playlist_del)
+        self.__signal_id1 = App().playlists.connect("playlist-add",
+                                                    self.__on_playlist_add)
+        self.__signal_id2 = App().playlists.connect("playlist-del",
+                                                    self.__on_playlist_del)
 
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Lollypop/PlaylistView.ui")
         self.__duration_label = builder.get_object("duration")
         builder.get_object("title").set_label(
-                             ", ".join(Lp().playlists.get_names(playlist_ids)))
+                            ", ".join(App().playlists.get_names(playlist_ids)))
 
         self.__edit_button = builder.get_object("edit-button")
         self.__jump_button = builder.get_object("jump-button")
         split_button = builder.get_object("split-button")
         if editable:
-            split_button.set_active(not Lp().settings.get_value("split-view"))
+            split_button.set_active(not App().settings.get_value("split-view"))
         else:
             split_button.hide()
 
@@ -70,6 +72,12 @@ class PlaylistsView(View):
         # Connect signals after ui init
         # "split-button" will emit a signal otherwise
         builder.connect_signals(self)
+
+        self.drag_dest_set(Gtk.DestDefaults.DROP | Gtk.DestDefaults.MOTION,
+                           [], Gdk.DragAction.MOVE)
+        self.drag_dest_add_text_targets()
+        self.connect("drag-motion", self.__on_drag_motion)
+
         # No duration for non user playlists
         # FIXME
         if playlist_ids[0] > 0:
@@ -112,6 +120,15 @@ class PlaylistsView(View):
         self.__playlists_widget.populate_list_right(tracks[mid_tracks:],
                                                     mid_tracks + 1)
 
+    def clear_animation(self):
+        """
+            Clear any animation
+        """
+        for row in self.__prev_animated_rows:
+            ctx = row.get_style_context()
+            ctx.remove_class("drag-up")
+            ctx.remove_class("drag-down")
+
     def get_ids(self):
         """
             Return playlist ids
@@ -150,18 +167,18 @@ class PlaylistsView(View):
         """
         View._on_destroy(self, widget)
         if self.__signal_id1:
-            Lp().playlists.disconnect(self.__signal_id1)
+            App().playlists.disconnect(self.__signal_id1)
             self.__signal_id1 = None
         if self.__signal_id2:
-            Lp().playlists.disconnect(self.__signal_id2)
+            App().playlists.disconnect(self.__signal_id2)
             self.__signal_id2 = None
 
     def _on_split_button_toggled(self, button):
         """
             Split/Unsplit view
         """
-        Lp().settings.set_value("split-view",
-                                GLib.Variant("b", not button.get_active()))
+        App().settings.set_value("split-view",
+                                 GLib.Variant("b", not button.get_active()))
         self.__playlists_widget.update_allocation()
 
     def _on_jump_button_clicked(self, button):
@@ -178,7 +195,7 @@ class PlaylistsView(View):
             Edit playlist
             @param button as Gtk.Button
         """
-        Lp().window.show_playlist_editor(self.__playlist_ids[0])
+        App().window.container.show_playlist_editor(self.__playlist_ids[0])
 
     def _on_current_changed(self, player):
         """
@@ -191,13 +208,44 @@ class PlaylistsView(View):
 #######################
 # PRIVATE             #
 #######################
+    def __auto_scroll(self, up):
+        """
+            Auto scroll up/down
+            @param up as bool
+        """
+        adj = self._scrolled.get_vadjustment()
+        value = adj.get_value()
+        if up:
+            adj_value = value - ArtSize.SMALL
+            adj.set_value(adj_value)
+            if adj.get_value() == 0:
+                self.__autoscroll_timeout_id = None
+                self.get_style_context().remove_class("drag-down")
+                self.get_style_context().remove_class("drag-up")
+                return False
+            else:
+                self.get_style_context().add_class("drag-up")
+                self.get_style_context().remove_class("drag-down")
+        else:
+            adj_value = value + ArtSize.SMALL
+            adj.set_value(adj_value)
+            if adj.get_value() < adj_value:
+                self.__autoscroll_timeout_id = None
+                self.get_style_context().remove_class("drag-down")
+                self.get_style_context().remove_class("drag-up")
+                return False
+            else:
+                self.get_style_context().add_class("drag-down")
+                self.get_style_context().remove_class("drag-up")
+        return True
+
     def __set_duration(self):
         """
             Set playlist duration
         """
         duration = 0
         for playlist_id in self.__playlist_ids:
-            duration += Lp().playlists.get_duration(playlist_id)
+            duration += App().playlists.get_duration(playlist_id)
 
         hours = int(duration / 3600)
         mins = int(duration / 60)
@@ -214,17 +262,49 @@ class PlaylistsView(View):
         """
             Update jump button status
         """
-        if Lp().player.current_track.id in self.__tracks:
+        if App().player.current_track.id in self.__tracks:
             self.__jump_button.set_sensitive(True)
-            artists = ", ".join(Lp().player.current_track.artists)
+            artists = ", ".join(App().player.current_track.artists)
             self.__jump_button.set_tooltip_markup(
              "<b>%s</b>\n%s" % (GLib.markup_escape_text(artists),
                                 GLib.markup_escape_text(
-                                              Lp().player.current_track.name)))
+                                             App().player.current_track.name)))
         else:
             self.__jump_button.set_sensitive(False)
             self.__jump_button.set_tooltip_text("")
 
+    def __on_drag_motion(self, widget, context, x, y, time):
+        """
+            Add style
+            @param widget as Gtk.Widget
+            @param context as Gdk.DragContext
+            @param x as int
+            @param y as int
+            @param time as int
+        """
+        auto_scroll = False
+        up = y <= ArtSize.MEDIUM
+        if up:
+            auto_scroll = True
+        elif y >= self._scrolled.get_allocated_height() - ArtSize.MEDIUM:
+            auto_scroll = True
+        else:
+            self.get_style_context().remove_class("drag-down")
+            self.get_style_context().remove_class("drag-up")
+            if self.__autoscroll_timeout_id is not None:
+                GLib.source_remove(self.__autoscroll_timeout_id)
+                self.__autoscroll_timeout_id = None
+            self.clear_animation()
+            row = self.__playlists_widget.rows_animation(x, y, self)
+            if row is not None:
+                self.__prev_animated_rows.append(row)
+            return
+        if self.__autoscroll_timeout_id is None and auto_scroll:
+            self.clear_animation()
+            self.__autoscroll_timeout_id = GLib.timeout_add(100,
+                                                            self.__auto_scroll,
+                                                            up)
+
     def __on_playlist_add(self, manager, playlist_id, track_id, pos):
         """
             Update tracks widgets
@@ -251,26 +331,19 @@ class PlaylistsManageView(View):
         Playlist view used to manage playlists
     """
 
-    def __init__(self, object_id, genre_ids, artist_ids, is_album):
+    def __init__(self, object):
         """
             Init View
-            @param object id as int
-            @param genre ids as [int]
-            @param artist ids as [int]
-            @param is album as bool
-            @param width as int
+            @param object as Track/Album/Disc
         """
         View.__init__(self)
         builder = Gtk.Builder()
         builder.add_from_resource(
             "/org/gnome/Lollypop/PlaylistsManagerView.ui")
-        if object_id != Type.NONE:
+        if object is not None:
             builder.get_object("back_btn").show()
         builder.connect_signals(self)
-        self.__manage_widget = PlaylistsManagerWidget(object_id,
-                                                      genre_ids,
-                                                      artist_ids,
-                                                      is_album)
+        self.__manage_widget = PlaylistsManagerWidget(object)
         self.__manage_widget.show()
         self._viewport.add(self.__manage_widget)
         self.add(builder.get_object("widget"))
@@ -297,7 +370,7 @@ class PlaylistsManageView(View):
             Restore previous view
             @param button as Gtk.Button
         """
-        Lp().window.destroy_current_view()
+        App().window.container.destroy_current_view()
 
 
 class PlaylistEditView(View):
@@ -316,7 +389,7 @@ class PlaylistEditView(View):
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/Lollypop/PlaylistEditView.ui")
         builder.get_object("title").set_label(
-                                          Lp().playlists.get_name(playlist_id))
+                                         App().playlists.get_name(playlist_id))
         builder.connect_signals(self)
         grid = builder.get_object("widget")
         self.add(grid)
@@ -339,4 +412,4 @@ class PlaylistEditView(View):
             Restore previous view
             @param button as Gtk.Button
         """
-        Lp().window.reload_view()
+        App().window.container.reload_view()
