@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -14,7 +14,7 @@ from gi.repository import Gtk, GLib, Gdk
 
 from gettext import gettext as _
 
-from lollypop.define import Lp, ArtSize
+from lollypop.define import App, ArtSize
 from lollypop.define import Shuffle, Loading
 from lollypop.objects import Album
 from lollypop.pop_artwork import CoversPopover
@@ -29,7 +29,6 @@ class BaseWidget:
         """
             Init widget
         """
-        self._selected = None
         self._loading = Loading.NONE
         self._cover = None
         self._widget = None
@@ -112,7 +111,7 @@ class BaseWidget:
         """
             Set play all image based on current shuffle status
         """
-        if Lp().settings.get_enum("shuffle") == Shuffle.NONE:
+        if App().settings.get_enum("shuffle") == Shuffle.NONE:
             self._play_all_button.set_from_icon_name(
                                         "media-playlist-consecutive-symbolic",
                                         Gtk.IconSize.BUTTON)
@@ -132,7 +131,7 @@ class BaseWidget:
         self._show_overlay = set
         self.emit("overlayed", set)
         if set:
-            if Lp().player.locked:
+            if App().player.locked:
                 opacity = 0.2
             else:
                 opacity = 1
@@ -153,7 +152,7 @@ class BaseWidget:
                                                            self._squared_class)
                 self._artwork_button.show()
             if self._action_button is not None:
-                self._show_append(not Lp().player.has_album(self._album))
+                self._show_append(self._album not in App().player.albums)
                 self._action_button.set_opacity(opacity)
                 self._action_button.get_style_context().add_class(
                                                        self._squared_class)
@@ -225,9 +224,9 @@ class BaseWidget:
             @param: widget as Gtk.EventBox
             @param: event as Gdk.Event
         """
-        if Lp().player.locked:
+        if App().player.locked:
             return True
-        Lp().player.play_album(self._album)
+        App().player.play_album(self._album)
         self._show_append(False)
         return True
 
@@ -251,27 +250,27 @@ class BaseWidget:
             @param: widget as Gtk.EventBox
             @param: event as Gdk.Event
         """
-        if Lp().player.locked:
+        if App().player.locked:
             return True
-        if Lp().player.has_album(self._album):
-            if Lp().player.current_track.album.id == self._album.id:
+        if self._album in App().player.albums:
+            if App().player.current_track.album.id == self._album.id:
                 # If not last album, skip it
-                if len(Lp().player.get_albums()) > 1:
-                    Lp().player.skip_album()
-                    Lp().player.remove_album(self._album)
+                if len(App().player.albums) > 1:
+                    App().player.skip_album()
+                    App().player.remove_album(self._album)
                 # remove it and stop playback by going to next track
                 else:
-                    Lp().player.remove_album(self._album)
-                    Lp().player.set_next()
-                    Lp().player.next()
+                    App().player.remove_album(self._album)
+                    App().player.set_next()
+                    App().player.next()
             else:
-                Lp().player.remove_album(self._album)
+                App().player.remove_album(self._album)
             self._show_append(True)
         else:
-            if Lp().player.is_playing and not Lp().player.get_albums():
-                Lp().player.play_album(self._album)
+            if App().player.is_playing and not App().player.albums:
+                App().player.play_album(self._album)
             else:
-                Lp().player.add_album(self._album)
+                App().player.add_album(self._album)
             self._show_append(False)
         return True
 
@@ -290,11 +289,12 @@ class BaseWidget:
         if append:
             self._action_button.set_from_icon_name("list-add-symbolic",
                                                    Gtk.IconSize.BUTTON)
-            self._action_event.set_tooltip_text(_("Append"))
+            self._action_event.set_tooltip_text(_("Add to current playlist"))
         else:
             self._action_button.set_from_icon_name("list-remove-symbolic",
                                                    Gtk.IconSize.BUTTON)
-            self._action_event.set_tooltip_text(_("Remove"))
+            self._action_event.set_tooltip_text(
+                                           _("Remove from current playlist"))
 
 #######################
 # PRIVATE             #
@@ -319,11 +319,10 @@ class AlbumWidget(BaseWidget):
         """
         BaseWidget.__init__(self)
         self._album = Album(album_id, genre_ids, artist_ids)
-        self._artist_ids = artist_ids
         self._art_size = art_size
         self.connect("destroy", self.__on_destroy)
-        self._scan_signal = Lp().scanner.connect("album-updated",
-                                                 self._on_album_updated)
+        self._scan_signal = App().scanner.connect("album-updated",
+                                                  self._on_album_updated)
 
     @property
     def album(self):
@@ -359,7 +358,7 @@ class AlbumWidget(BaseWidget):
         """
         if self._cover is None:
             return
-        surface = Lp().art.get_album_artwork(
+        surface = App().art.get_album_artwork(
                             self._album,
                             self._art_size,
                             self._cover.get_scale_factor())
@@ -375,7 +374,7 @@ class AlbumWidget(BaseWidget):
         """
         if self._cover is None:
             return
-        surface = Lp().art.get_album_artwork(
+        surface = App().art.get_album_artwork(
                             self._album,
                             self._art_size,
                             self._cover.get_scale_factor())
@@ -391,14 +390,12 @@ class AlbumWidget(BaseWidget):
         """
         if self._cover is None or self._art_size != ArtSize.BIG:
             return
-        selected = self._album.id == Lp().player.current_track.album.id
-        if selected != self._selected:
-            if selected:
-                self._cover.get_style_context().add_class(
-                                                    "cover-frame-selected")
-            else:
-                self._cover.get_style_context().remove_class(
-                                                    "cover-frame-selected")
+        selected = self._album.id == App().player.current_track.album.id
+        style_context = self._cover.get_style_context()
+        if selected:
+            style_context.add_class("cover-frame-selected")
+        else:
+            style_context.remove_class("cover-frame-selected")
 
 #######################
 # PROTECTED           #
@@ -415,4 +412,4 @@ class AlbumWidget(BaseWidget):
             @param widget as Gtk.Widget
         """
         if self._scan_signal is not None:
-            Lp().scanner.disconnect(self._scan_signal)
+            App().scanner.disconnect(self._scan_signal)
