@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -18,7 +18,7 @@ try:
     from lollypop.wikipedia import Wikipedia
 except:
     pass
-from lollypop.define import Lp
+from lollypop.define import App
 from lollypop.utils import get_network_available
 from lollypop.cache import InfoCache
 from lollypop.helper_task import TaskHelper
@@ -104,12 +104,7 @@ class InfoContent(Gtk.Stack):
         """
         (content, data) = InfoCache.get(prefix, suffix)
         if content is not None:
-            stream = None
-            if data is not None:
-                bytes = GLib.Bytes(data)
-                stream = Gio.MemoryInputStream.new_from_bytes(bytes)
-                bytes.unref()
-            GLib.idle_add(self.__set_content, content, stream)
+            GLib.idle_add(self.__set_content, content, data)
             return True
         return False
 
@@ -130,27 +125,30 @@ class InfoContent(Gtk.Stack):
 #######################
 # PRIVATE             #
 #######################
-    def __set_content(self, content, stream):
+    def __set_content(self, content, data):
         """
             Set content
             @param content as string
-            @param data as Gio.MemoryInputStream
+            @param data as bytes
         """
         if content is not None:
             self.__content.set_markup(
                               GLib.markup_escape_text(content.decode("utf-8")))
-            if stream is not None:
+            if data is not None:
+                gbytes = GLib.Bytes(data)
+                stream = Gio.MemoryInputStream.new_from_bytes(gbytes)
                 scale = self.__image.get_scale_factor()
                 # Will happen if cache is broken or when reading empty files
                 try:
                     pixbuf = GdkPixbuf.Pixbuf.new_from_stream_at_scale(
                                stream,
-                               Lp().settings.get_value(
+                               App().settings.get_value(
                                         "cover-size").get_int32() + 50 * scale,
                                -1,
                                True,
                                None)
                     stream.close()
+                    gbytes.unref()
                     surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf,
                                                                    scale,
                                                                    None)
@@ -337,7 +335,7 @@ class LastfmContent(InfoContent):
             @param artist as str
         """
         try:
-            (url, content) = Lp().lastfm.get_artist_info(artist)
+            (url, content) = App().lastfm.get_artist_info(artist)
         except Exception as e:
             print("LastfmContent::__load_page_content():", e)
             url = content = None
