@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -14,10 +14,10 @@ from gi.repository import Gtk, GLib, Gdk, Pango, GObject
 
 from gettext import gettext as _
 
-from lollypop.define import Lp, Type, WindowSize, Loading
+from lollypop.define import App, Type, WindowSize, Loading
 from lollypop.cellrenderer import CellRendererAlbum
 from lollypop.widgets_track import TracksWidget, PlaylistRow
-from lollypop.objects import Track
+from lollypop.objects import Track, Album, Disc
 from lollypop.helper_task import TaskHelper
 
 
@@ -38,14 +38,18 @@ class PlaylistsWidget(Gtk.Grid):
         self.set_row_spacing(5)
         self.set_orientation(Gtk.Orientation.VERTICAL)
         self.__playlist_ids = playlist_ids
-        self.__tracks_left = []
-        self.__tracks_right = []
+        self.__row_tracks_left = []
+        self.__row_tracks_right = []
         self.__width = None
         self.__orientation = None
         self.__loading = Loading.NONE
         # Used to block widget2 populate while showing one column
         self.__locked_widget_right = True
 
+        self.set_margin_start(5)
+        # 15 for scrollbar overlay
+        self.set_margin_end(15)
+
         self.__grid = Gtk.Grid()
         self.__grid.set_vexpand(True)
         self.__grid.set_column_homogeneous(True)
@@ -64,7 +68,7 @@ class PlaylistsWidget(Gtk.Grid):
         self.__tracks_widget_left.show()
         self.__tracks_widget_right.show()
 
-        self.drag_dest_set(Gtk.DestDefaults.DROP | Gtk.DestDefaults.MOTION,
+        self.drag_dest_set(Gtk.DestDefaults.DROP,
                            [], Gdk.DragAction.MOVE)
         self.drag_dest_add_text_targets()
         self.connect("drag-data-received", self.__on_drag_data_received)
@@ -126,20 +130,19 @@ class PlaylistsWidget(Gtk.Grid):
         ordinate = None
         for child in self.__tracks_widget_left.get_children() + \
                 self.__tracks_widget_right.get_children():
-            if child.id == Lp().player.current_track.id:
+            if child.id == App().player.current_track.id:
                 ordinate = child.translate_coordinates(self.__grid, 0, 0)[1]
         return ordinate
 
     def populate_list_left(self, tracks, pos):
         """
             Populate left list
-            @param track"s ids as array of int (not null)
+            @param track's ids as array of int (not null)
             @param track position as int
             @thread safe
         """
         # We reset width here to allow size allocation code to run
         self.__width = None
-        self.__tracks_left = list(tracks)
         GLib.idle_add(self.__add_tracks,
                       tracks,
                       self.__tracks_widget_left,
@@ -152,7 +155,6 @@ class PlaylistsWidget(Gtk.Grid):
             @param track position as int
             @thread safe
         """
-        self.__tracks_right = list(tracks)
         # If we are showing only one column, wait for widget1
         if self.__orientation == Gtk.Orientation.VERTICAL and\
            self.__locked_widget_right:
@@ -169,9 +171,11 @@ class PlaylistsWidget(Gtk.Grid):
         """
             Update playing indicator
         """
-        self.__tracks_widget_left.update_playing(Lp().player.current_track.id)
-        self.__tracks_widget_right.update_playing(Lp().player.current_track.id)
+        track_id = App().player.current_track.id
+        self.__tracks_widget_left.update_playing(track_id)
+        self.__tracks_widget_right.update_playing(track_id)
 
+    # FIXME track_id needed? See update_playing_indicator()
     def update_duration(self, track_id):
         """
             Update duration for current track
@@ -197,14 +201,16 @@ class PlaylistsWidget(Gtk.Grid):
         if pos > children_len / 2:
             widget = self.__tracks_widget_right
             pos -= len(self.__tracks_widget_left.get_children())
+        elif pos == -1:
+            widget = self.__tracks_widget_right
         else:
             widget = self.__tracks_widget_left
         self.__add_tracks([track_id], widget, pos)
-        self.__update_tracks()
         self.__update_position()
         self.__update_headers()
         self.__tracks_widget_left.update_indexes(1)
-        self.__tracks_widget_right.update_indexes(len(self.__tracks_left) + 1)
+        track_left_children = self.__tracks_widget_left.get_children()
+        self.__tracks_widget_right.update_indexes(len(track_left_children) + 1)
 
     def remove(self, track_id):
         """
@@ -217,18 +223,47 @@ class PlaylistsWidget(Gtk.Grid):
         if track_id is None:
             for child in children:
                 child.destroy()
-            self.__update_tracks()
         else:
             for child in children:
-                if child.id == track_id:
+                if child.track.id == track_id:
                     child.destroy()
                     break
-            self.__update_tracks()
             self.__update_position()
             self.__update_headers()
             self.__tracks_widget_left.update_indexes(1)
+            track_left_children = self.__tracks_widget_left.get_children()
             self.__tracks_widget_right.update_indexes(
-                                                   len(self.__tracks_left) + 1)
+                                                  len(track_left_children) + 1)
+
+    def rows_animation(self, x, y, widget):
+        """
+            Show animation to help user dnd
+            @param x as int
+            @param y as int
+            @param widget as Gtk.Widget (related widget)
+            @return child as PlaylistRow/None
+        """
+        rows = self.__tracks_widget_left.get_children() +\
+            self.__tracks_widget_right.get_children()
+        for row in rows:
+            coordinates = row.translate_coordinates(widget, 0, 0)
+            if coordinates is None:
+                continue
+            (row_x, row_y) = coordinates
+            row_width = row.get_allocated_width()
+            row_height = row.get_allocated_height()
+            if x < row_x or\
+                    x > row_x + row_width or\
+                    y < row_y or\
+                    y > row_y + row_height:
+                continue
+            if y <= row_y + row_height / 2:
+                row.get_style_context().add_class("drag-up")
+                return row
+            elif y >= row_y + row_height / 2:
+                row.get_style_context().add_class("drag-down")
+                return row
+        return None
 
 #######################
 # PRIVATE             #
@@ -256,49 +291,39 @@ class PlaylistsWidget(Gtk.Grid):
             return
 
         track = Track(tracks.pop(0))
-        row = PlaylistRow(track.id, pos,
-                          track.album.id != previous_album_id)
+        track.set_number(pos)
+        row = PlaylistRow(track, track.album.id != previous_album_id)
         row.connect("track-moved", self.__on_track_moved)
         row.show()
         widget.insert(row, pos)
         GLib.idle_add(self.__add_tracks, tracks, widget,
                       pos + 1, track.album.id)
 
-    def __update_tracks(self):
-        """
-            Update tracks based on current widget
-        """
-        # Recalculate tracks
-        self.__tracks_left = []
-        self.__tracks_right = []
-        for child in self.__tracks_widget_left.get_children():
-            self.__tracks_left.append(child.id)
-        for child in self.__tracks_widget_right.get_children():
-            self.__tracks_right.append(child.id)
-
     def __update_position(self):
         """
             Update widget position
+
         """
-        len_tracks1 = len(self.__tracks_left)
-        len_tracks2 = len(self.__tracks_right)
+        row_tracks_left = self.__get_row_tracks(Loading.LEFT)
+        row_tracks_right = self.__get_row_tracks(Loading.RIGHT)
+        len_row_tracks_left = len(row_tracks_left)
+        len_row_tracks_right = len(row_tracks_right)
         # Take first track from tracks2 and put it at the end of tracks1
-        if len_tracks2 > len_tracks1:
-            src = self.__tracks_right[0]
-            if self.__tracks_left:
-                dst = self.__tracks_left[-1]
+        if len_row_tracks_right > len_row_tracks_left:
+            src_track = row_tracks_right[0]
+            if row_tracks_left:
+                dst_track = row_tracks_left[-1]
             else:
-                dst = -1
-            self.__move_track(dst, src, False)
+                dst_track = None
+            self.__move_track(dst_track, src_track, False)
         # Take last track of tracks1 and put it at the bottom of tracks2
-        elif len_tracks1 - 1 > len_tracks2:
-            src = self.__tracks_left[-1]
-            if self.__tracks_right:
-                dst = self.__tracks_right[0]
+        elif len_row_tracks_left - 1 > len_row_tracks_right:
+            src_track = row_tracks_left[-1]
+            if row_tracks_right:
+                dst_track = row_tracks_right[0]
             else:
-                dst = -1
-            self.__move_track(dst, src, True)
-        self.__update_tracks()
+                dst_track = None
+            self.__move_track(dst_track, src_track, True)
 
     def __update_headers(self):
         """
@@ -307,57 +332,69 @@ class PlaylistsWidget(Gtk.Grid):
         self.__tracks_widget_left.update_headers()
         prev_album_id = None
         if self.__orientation == Gtk.Orientation.VERTICAL:
-            if self.__tracks_left:
-                prev_album_id = Track(self.__tracks_left[-1]).album.id
+            row_tracks_left = self.__tracks_widget_left.get_children()
+            if row_tracks_left:
+                prev_album_id = row_tracks_left[-1].track.album.id
         self.__tracks_widget_right.update_headers(prev_album_id)
 
-    def __move_track(self, dst, src, up):
+    def __get_row_tracks(self, loading):
+        """
+            Get tracks for loading
+            @param loading as Loading
+        """
+        if loading == Loading.LEFT:
+            widget = self.__tracks_widget_left
+        else:
+            widget = self.__tracks_widget_right
+        tracks = []
+        for row in widget.get_children():
+            tracks.append(row.track)
+        return tracks
+
+    def __move_track(self, dst_track, src_track, down):
         """
             Move track from src to row
-            @param dst as int
-            @param src as int
-            @param up as bool
-            @return (dst_widget as TracksWidget,
-                     src index as int, dst index as int)
+            @param dst_track as Track
+            @param src_track as Track
+            @param down as bool
         """
-        tracks1_len = len(self.__tracks_left)
-        tracks2_len = len(self.__tracks_right)
-        if src in self.__tracks_left:
+        row_tracks_left = self.__get_row_tracks(Loading.LEFT)
+        row_tracks_right = self.__get_row_tracks(Loading.RIGHT)
+        tracks1_len = len(row_tracks_left)
+        tracks2_len = len(row_tracks_right)
+        if src_track in row_tracks_left:
             src_widget = self.__tracks_widget_left
-            src_index = self.__tracks_left.index(src) - 1
         else:
             src_widget = self.__tracks_widget_right
-            src_index = self.__tracks_right.index(src) - 1
-        if tracks1_len == 0 or dst in self.__tracks_left:
+        if tracks1_len == 0 or dst_track in row_tracks_left:
             dst_widget = self.__tracks_widget_left
-            dst_tracks = self.__tracks_left
-        elif tracks2_len == 0 or dst in self.__tracks_right:
+            dst_tracks = row_tracks_left
+        elif tracks2_len == 0 or dst_track in row_tracks_right:
             dst_widget = self.__tracks_widget_right
-            dst_tracks = self.__tracks_right
+            dst_tracks = row_tracks_right
         else:
             return
         # Remove src from src_widget
         for child in src_widget.get_children():
-            if child.id == src:
+            if child.track == src_track:
                 child.destroy()
                 break
-        src_track = Track(src)
         prev_track = Track()
         name = GLib.markup_escape_text(src_track.name)
         index = 0
         # Get previous track
-        if dst != -1:
+        if dst_track is not None:
             for child in dst_widget.get_children():
-                if child.id == dst:
+                if child.track == dst_track:
                     break
                 index += 1
-            if not up:
+            if down:
                 index += 1
             # Get previous track (in dst context)
-            prev_index = dst_tracks.index(dst)
-            if up:
+            prev_index = dst_tracks.index(dst_track)
+            if not down:
                 prev_index -= 1
-            prev_track = Track(dst_tracks[prev_index])
+            prev_track = dst_tracks[prev_index]
             # If we are listening to a compilation, prepend artist name
             if (src_track.album.artist_id == Type.COMPILATIONS or
                     len(src_track.artist_ids) > 1 or
@@ -365,15 +402,12 @@ class PlaylistsWidget(Gtk.Grid):
                 name = "<b>%s</b>\n%s" % (
                          GLib.markup_escape_text(", ".join(src_track.artists)),
                          name)
-            self.__tracks_left.insert(index, src_track.id)
-        row = PlaylistRow(src_track.id,
-                          index,
-                          index == 0 or
+        src_track.set_number(index)
+        row = PlaylistRow(src_track, index == 0 or
                           src_track.album.id != prev_track.album.id)
         row.connect("track-moved", self.__on_track_moved)
         row.show()
         dst_widget.insert(row, index)
-        return (src_widget, dst_widget, src_index, index)
 
     def __on_drag_data_received(self, widget, context, x, y, data, info, time):
         """
@@ -387,49 +421,59 @@ class PlaylistsWidget(Gtk.Grid):
             @param info as int
             @param time as int
         """
+        from lollypop.view import View
+        view = widget.get_ancestor(View)
+        if view is not None:
+            view.clear_animation()
         try:
             value = int(data.get_text())
             try:
                 child = self.__tracks_widget_right.get_children()[-1]
             except:
                 child = self.__tracks_widget_left.get_children()[-1]
-            self.__on_track_moved(widget, child.id, value, False)
+            self.__on_track_moved(widget, child.track.id, value, False)
         except:
             if len(self.__playlist_ids) == 1:
-                Lp().playlists.import_uri(self.__playlist_ids[0],
-                                          data.get_text())
+                App().playlists.import_uri(self.__playlist_ids[0],
+                                           data.get_text())
 
     def __on_track_moved(self, widget, dst, src, up):
         """
             Move track from src to row
             Recalculate track position
             @param widget as TracksWidget
-            @param dst as int
-            @param src as int
+            @param dst as str
+            @param src as str
             @param up as bool
         """
         def update_playlist():
             # Save playlist in db only if one playlist visible
             if len(self.__playlist_ids) == 1 and self.__playlist_ids[0] >= 0:
-                Lp().playlists.clear(self.__playlist_ids[0], False)
-                tracks = []
-                for track_id in self.__tracks_left + self.__tracks_right:
-                    tracks.append(Track(track_id))
-                Lp().playlists.add_tracks(self.__playlist_ids[0],
-                                          tracks,
-                                          False)
+                row_tracks_left = self.__get_row_tracks(Loading.LEFT)
+                row_tracks_right = self.__get_row_tracks(Loading.RIGHT)
+                tracks = row_tracks_left + row_tracks_right
+                App().playlists.clear(self.__playlist_ids[0], False)
+                App().playlists.add_tracks(self.__playlist_ids[0],
+                                           tracks,
+                                           False)
             if not (set(self.__playlist_ids) -
-               set(Lp().player.get_user_playlist_ids())):
-                Lp().player.update_user_playlist(self.__tracks_left +
-                                                 self.__tracks_right)
-
-        (src_widget, dst_widget, src_index, dst_index) = \
-            self.__move_track(dst, src, up)
-        self.__update_tracks()
+               set(App().player.get_playlist_ids())):
+                App().player.update_playlist(tracks)
+        src_track = dst_track = None
+        row_tracks_left = self.__get_row_tracks(Loading.LEFT)
+        row_tracks_right = self.__get_row_tracks(Loading.RIGHT)
+        for track in row_tracks_left + row_tracks_right:
+            if track.id == int(dst):
+                dst_track = track
+            elif track.id == int(src):
+                src_track = track
+            elif src_track is not None and dst_track is not None:
+                break
+        self.__move_track(dst_track, src_track, up)
         self.__update_position()
         self.__update_headers()
         self.__tracks_widget_left.update_indexes(1)
-        self.__tracks_widget_right.update_indexes(len(self.__tracks_left) + 1)
+        self.__tracks_widget_right.update_indexes(len(row_tracks_left) + 1)
         helper = TaskHelper()
         helper.run(update_playlist)
 
@@ -444,11 +488,13 @@ class PlaylistsWidget(Gtk.Grid):
         self.__width = allocation.width
         redraw = False
         if allocation.width < WindowSize.MONSTER or\
-                not Lp().settings.get_value("split-view"):
+                not App().settings.get_value("split-view"):
             self.__grid.set_property("valign", Gtk.Align.START)
+            self.__grid.set_column_spacing(0)
             orientation = Gtk.Orientation.VERTICAL
         else:
             self.__grid.set_property("valign", Gtk.Align.FILL)
+            self.__grid.set_column_spacing(5)
             orientation = Gtk.Orientation.HORIZONTAL
         if orientation != self.__orientation:
             self.__orientation = orientation
@@ -461,25 +507,27 @@ class PlaylistsWidget(Gtk.Grid):
             GLib.idle_add(self.__grid.add, self.__tracks_widget_right)
         self.__update_headers()
 
-    def __on_activated(self, widget, track_id):
+    def __on_activated(self, widget, track):
         """
             On track activation, play track
             @param widget as TracksWidget
             @param track as Track
         """
         # Add to queue by default
-        if Lp().player.locked:
-            if track_id in Lp().player.queue:
-                Lp().player.del_from_queue(track_id)
+        if App().player.locked:
+            if track.id in App().player.queue:
+                App().player.del_from_queue(track.id)
             else:
-                Lp().player.append_to_queue(track_id)
+                App().player.append_to_queue(track.id)
         else:
-            Lp().player.load(Track(track_id))
-            if not Lp().player.is_party:
-                Lp().player.populate_user_playlist_by_tracks(
-                                                           self.__tracks_left +
-                                                           self.__tracks_right,
-                                                           self.__playlist_ids)
+            App().player.load(track)
+            if not App().player.is_party:
+                row_tracks_left = self.__get_row_tracks(Loading.LEFT)
+                row_tracks_right = self.__get_row_tracks(Loading.RIGHT)
+                App().player.populate_playlist_by_tracks(
+                                                       row_tracks_left +
+                                                       row_tracks_right,
+                                                       self.__playlist_ids)
 
 
 class PlaylistsManagerWidget(Gtk.Bin):
@@ -487,19 +535,13 @@ class PlaylistsManagerWidget(Gtk.Bin):
         Widget for playlists management
     """
 
-    def __init__(self, object_id, genre_ids, artist_ids, is_album):
+    def __init__(self, object):
         """
             Init widget
-            @param object id as int
-            @param genre ids as [int]
-            @param artist ids as [int]
-            @param is album as bool
+            @param object as Track/Album/Disc
         """
         Gtk.Bin.__init__(self)
-        self.__genre_ids = genre_ids
-        self.__artist_ids = artist_ids
-        self.__object_id = object_id
-        self.__is_album = is_album
+        self.__object = object
         self.__deleted_path = None
 
         builder = Gtk.Builder()
@@ -524,7 +566,7 @@ class PlaylistsManagerWidget(Gtk.Bin):
 
         self.add(builder.get_object("widget"))
 
-        if self.__object_id != Type.NONE:
+        if object is not None:
             renderer0 = Gtk.CellRendererToggle()
             renderer0.set_property("activatable", True)
             renderer0.connect("toggled", self.__on_playlist_toggled)
@@ -552,7 +594,7 @@ class PlaylistsManagerWidget(Gtk.Bin):
         column2.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
         column2.set_property("fixed_width", 50)
 
-        if self.__object_id != Type.NONE:
+        if self.__object is not None:
             self.__view.append_column(column0)
         self.__view.append_column(column1)
         self.__view.append_column(column2)
@@ -562,7 +604,7 @@ class PlaylistsManagerWidget(Gtk.Bin):
             Populate playlists
             @thread safe
         """
-        playlists = Lp().playlists.get()
+        playlists = App().playlists.get()
         self.__append_playlists(playlists)
 
     def add_new_playlist(self):
@@ -579,8 +621,8 @@ class PlaylistsManagerWidget(Gtk.Bin):
         while name in existing_playlists:
             count += 1
             name = _("New playlist ") + str(count)
-        Lp().playlists.add(name)
-        playlist_id = Lp().playlists.get_id(name)
+        App().playlists.add(name)
+        playlist_id = App().playlists.get_id(name)
         iterator = self.__model.append([True,
                                         name,
                                         "user-trash-symbolic",
@@ -619,7 +661,7 @@ class PlaylistsManagerWidget(Gtk.Bin):
         """
         if self.__deleted_path:
             iterator = self.__model.get_iter(self.__deleted_path)
-            Lp().playlists.delete(self.__model.get_value(iterator, 1))
+            App().playlists.delete(self.__model.get_value(iterator, 1))
             self.__model.remove(iterator)
             self.__deleted_path = None
             self.__infobar.hide()
@@ -660,18 +702,17 @@ class PlaylistsManagerWidget(Gtk.Bin):
             @param playlist selected as bool
         """
         for playlist in playlists:
-            if self.__object_id != Type.NONE:
-                if self.__is_album:
-                    selected = Lp().playlists.exists_album(
+            if self.__object is not None:
+                if isinstance(self.__object, Album) or\
+                        isinstance(self.__object, Disc):
+                    selected = App().playlists.exists_album(
                                                        playlist[0],
-                                                       self.__object_id,
-                                                       self.__genre_ids,
-                                                       self.__artist_ids)
+                                                       self.__object)
                 else:
 
-                    selected = Lp().playlists.exists_track(
+                    selected = App().playlists.exists_track(
                                                        playlist[0],
-                                                       self.__object_id)
+                                                       self.__object)
             else:
                 selected = False
             self.__model.append([selected, playlist[1],
@@ -705,7 +746,7 @@ class PlaylistsManagerWidget(Gtk.Bin):
             iterator = self.__model.get_iter(path)
             if iterator is not None:
                 playlist_id = self.__model.get_value(iterator, 3)
-                Lp().playlists.import_uri(playlist_id, data.get_text())
+                App().playlists.import_uri(playlist_id, data.get_text())
         except:
             pass
 
@@ -728,19 +769,16 @@ class PlaylistsManagerWidget(Gtk.Bin):
             @param add as bool
         """
         def set(playlist_id, add):
-            tracks = []
-            if self.__is_album:
-                track_ids = Lp().albums.get_track_ids(self.__object_id,
-                                                      self.__genre_ids,
-                                                      self.__artist_ids)
-                for track_id in track_ids:
-                    tracks.append(Track(track_id))
+            if isinstance(self.__object, Disc):
+                tracks = self.__object.tracks
+            elif isinstance(self.__object, Album):
+                tracks = self.__object.tracks
             else:
-                tracks = [Track(self.__object_id)]
+                tracks = [self.__object]
             if add:
-                Lp().playlists.add_tracks(playlist_id, tracks)
+                App().playlists.add_tracks(playlist_id, tracks)
             else:
-                Lp().playlists.remove_tracks(playlist_id, tracks)
+                App().playlists.remove_tracks(playlist_id, tracks)
         helper = TaskHelper()
         helper.run(set, playlist_id, add)
 
@@ -757,11 +795,11 @@ class PlaylistsManagerWidget(Gtk.Bin):
         if name.find("/") != -1 or\
            old_name == name or\
            not name or\
-           Lp().playlists.get_id(name) != Type.NONE:
+           App().playlists.get_id(name) != Type.NONE:
             return
         self.__model.remove(iterator)
         self.__model.append([True, name, "user-trash-symbolic", playlist_id])
-        Lp().playlists.rename(name, old_name)
+        App().playlists.rename(name, old_name)
 
     def __on_playlist_editing_start(self, widget, editable, path):
         """
@@ -771,7 +809,7 @@ class PlaylistsManagerWidget(Gtk.Bin):
             @param path as str representation of Gtk.TreePath
         """
         # FIXME Not needed with GTK >= 3.18
-        Lp().window.enable_global_shortcuts(False)
+        App().window.enable_global_shortcuts(False)
 
     def __on_playlist_editing_cancel(self, widget):
         """
@@ -779,7 +817,7 @@ class PlaylistsManagerWidget(Gtk.Bin):
             @param widget as cell renderer
         """
         # FIXME Not needed with GTK >= 3.18
-        Lp().window.enable_global_shortcuts(True)
+        App().window.enable_global_shortcuts(True)
 
     def __on_column0_clicked(self, column):
         """
@@ -867,27 +905,20 @@ class PlaylistEditWidget(Gtk.Bin):
             self.__view.grab_focus()
             self.__view.get_selection().unselect_all()
 
-    def _on_row_activated(self, view, path, column):
-        """
-            Delete playlist
-            @param TreeView, TreePath, TreeViewColumn
-        """
-        iterator = self.__model.get_iter(path)
-        if iterator:
-            if column.get_title() == "delete":
-                self.__show_infobar(path)
-            else:
-                self.__infobar.hide()
-
     def _on_selection_changed(self, selection):
         """
             On selection changed, show infobar
             @param selection as Gtk.TreeSelection
         """
+        (model, paths) = selection.get_selected_rows()
+        for item in self.__model:
+            if item.path in paths:
+                item[2] = "checkbox-checked-symbolic"
+            else:
+                item[2] = "checkbox-symbolic"
         count = selection.count_selected_rows()
         if count == 1:
-            (model, path) = selection.get_selected_rows()
-            iterator = model.get_iter(path)
+            iterator = model.get_iter(paths[0])
             self.__infobar_label.set_markup(_("Remove \"%s\"?") %
                                             model.get_value(iterator,
                                                             1).replace("\n",
@@ -919,15 +950,15 @@ class PlaylistEditWidget(Gtk.Bin):
             iterator = self.__model.get_iter(row.get_path())
             track = Track(self.__model.get_value(iterator, 3))
             tracks.append(track)
-            if self.__playlist_id == Type.LOVED and Lp().lastfm is not None:
+            if self.__playlist_id == Type.LOVED and App().lastfm is not None:
                 if track.album.artist_id == Type.COMPILATIONS:
                     artist_name = ", ".join(track.artists)
                 else:
                     artist_name = ", ".join(track.album.artists)
                 helper = TaskHelper()
-                helper.run(Lp().lastfm.unlove, artist_name, track.name)
+                helper.run(App().lastfm.unlove, artist_name, track.name)
             self.__model.remove(iterator)
-        Lp().playlists.remove_tracks(self.__playlist_id, tracks)
+        App().playlists.remove_tracks(self.__playlist_id, tracks)
         self.__infobar.hide()
         self.__unselectall()
 
@@ -945,7 +976,7 @@ class PlaylistEditWidget(Gtk.Bin):
         """
             Append tracks
         """
-        return Lp().playlists.get_track_ids(self.__playlist_id)
+        return App().playlists.get_track_ids(self.__playlist_id)
 
     def __append_track(self, track_ids):
         """
@@ -962,7 +993,7 @@ class PlaylistEditWidget(Gtk.Bin):
                                 "<b>%s</b>\n%s" % (
                                    GLib.markup_escape_text(artists),
                                    GLib.markup_escape_text(track.name)),
-                                 "user-trash-symbolic", track.id])
+                                 "checkbox-symbolic", track.id])
             GLib.idle_add(self.__append_track, track_ids)
         else:
             self.__in_thread = False
