@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -14,7 +14,7 @@ from gi.repository import Gtk, Pango, GObject
 
 from gettext import gettext as _
 
-from lollypop.define import Lp, ArtSize, Type
+from lollypop.define import App, ArtSize, Type
 from lollypop.objects import Track
 from lollypop.widgets_album import BaseWidget
 from lollypop.pop_radio import RadioPopover
@@ -45,6 +45,8 @@ class RadioWidget(Gtk.FlowBoxChild, BaseWidget):
         """
             Init widget content
         """
+        grid = Gtk.Grid()
+        grid.set_orientation(Gtk.Orientation.VERTICAL)
         self.get_style_context().remove_class("loading")
         self._widget = Gtk.EventBox()
         self.__helper = TouchHelper(self._widget, None, None)
@@ -55,21 +57,21 @@ class RadioWidget(Gtk.FlowBoxChild, BaseWidget):
         self._cover = Gtk.Image()
         self._cover.set_property("halign", Gtk.Align.CENTER)
         self._cover.set_size_request(ArtSize.BIG, ArtSize.BIG)
+        self._cover.get_style_context().add_class("cover-frame")
         self.__title_label = Gtk.Label()
         self.__title_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.__title_label.set_property("halign", Gtk.Align.CENTER)
         self.__title_label.set_text(self.__name)
         self.__title_label.set_property("has-tooltip", True)
         self.__title_label.connect("query-tooltip", self._on_query_tooltip)
-        self._overlay = Gtk.Overlay()
-        frame = Gtk.Frame()
-        frame.get_style_context().add_class("cover-frame")
-        frame.add(self._cover)
-        self._overlay.add(frame)
-        grid = Gtk.Grid()
-        grid.set_orientation(Gtk.Orientation.VERTICAL)
-        self._overlay.get_style_context().add_class("white")
-        grid.add(self._overlay)
+        self.__overlay = Gtk.Overlay()
+        self.__overlay.add(self._cover)
+        white = Gtk.Grid()
+        white.set_property("halign", Gtk.Align.CENTER)
+        white.set_property("valign", Gtk.Align.CENTER)
+        white.get_style_context().add_class("white")
+        white.add(self.__overlay)
+        grid.add(white)
         grid.add(self.__title_label)
         self._widget.add(grid)
         self.set_property("halign", Gtk.Align.CENTER)
@@ -86,27 +88,6 @@ class RadioWidget(Gtk.FlowBoxChild, BaseWidget):
         """
         pass
 
-    @property
-    def id(self):
-        """
-            Return widget id (same value for all radio widgets)
-        """
-        return Type.RADIOS
-
-    @property
-    def filter(self):
-        """
-            @Return filter as str
-        """
-        return self.__name
-
-    @property
-    def title(self):
-        """
-            @Return title as str
-        """
-        return self.__name
-
     def do_get_preferred_width(self):
         """
             Return preferred width
@@ -131,7 +112,7 @@ class RadioWidget(Gtk.FlowBoxChild, BaseWidget):
         """
         if self._cover is None:
             return
-        surface = Lp().art.get_radio_artwork(
+        surface = App().art.get_radio_artwork(
                     self.__name,
                     ArtSize.BIG,
                     self._cover.get_scale_factor())
@@ -143,7 +124,7 @@ class RadioWidget(Gtk.FlowBoxChild, BaseWidget):
         """
         if self._cover is None:
             return
-        surface = Lp().art.get_radio_artwork(
+        surface = App().art.get_radio_artwork(
                     self.__name,
                     ArtSize.BIG,
                     self._cover.get_scale_factor())
@@ -155,14 +136,34 @@ class RadioWidget(Gtk.FlowBoxChild, BaseWidget):
         """
         if self._cover is None:
             return
-        selected = Lp().player.current_track.id == Type.RADIOS and\
-            self.__name == Lp().player.current_track.album_artists[0]
+        selected = App().player.current_track.id == Type.RADIOS and\
+            self.__name == App().player.current_track.album_artists[0]
+        style_context = self.get_style_context()
         if selected:
-            self._overlay.get_style_context().add_class(
-                                                    "cover-frame-selected")
+            style_context.add_class("cover-frame-selected")
         else:
-            self._overlay.get_style_context().remove_class(
-                                                    "cover-frame-selected")
+            style_context.remove_class("cover-frame-selected")
+
+    @property
+    def id(self):
+        """
+            Return widget id (same value for all radio widgets)
+        """
+        return Type.RADIOS
+
+    @property
+    def filter(self):
+        """
+            @Return filter as str
+        """
+        return self.__name
+
+    @property
+    def title(self):
+        """
+            @Return title as str
+        """
+        return self.__name
 
 #######################
 # PROTECTED           #
@@ -208,9 +209,9 @@ class RadioWidget(Gtk.FlowBoxChild, BaseWidget):
             self._artwork_button.set_opacity(0)
             self._play_event.add(self._play_button)
             self._artwork_event.add(self._artwork_button)
-            self._overlay.add_overlay(self._play_event)
-            self._overlay.add_overlay(self._artwork_event)
-            self._overlay.show_all()
+            self.__overlay.add_overlay(self._play_event)
+            self.__overlay.add_overlay(self._artwork_event)
+            self.__overlay.show_all()
             BaseWidget._show_overlay_func(self, True)
         else:
             BaseWidget._show_overlay_func(self, False)
@@ -261,13 +262,13 @@ class RadioWidget(Gtk.FlowBoxChild, BaseWidget):
             @param: widget as Gtk.EventBox
             @param: event as Gdk.Event
         """
-        if Lp().player.locked:
+        if App().player.locked:
             return True
         url = self.__radios_manager.get_url(self.__name)
         if url:
             track = Track()
             track.set_radio(self.__name, url)
-            Lp().player.load(track)
+            App().player.load(track)
 
     def _on_edit_press_event(self, widget, event):
         """
