@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -12,42 +12,46 @@
 
 from gi.repository import GObject, Gtk, Gdk, Pango, GLib, Gst
 
-from lollypop.define import Lp, ArtSize, Type
+from lollypop.define import App, ArtSize, Type
 from lollypop.pop_menu import TrackMenuPopover, TrackMenu
 from lollypop.widgets_indicator import IndicatorWidget
 from lollypop.widgets_context import ContextWidget
 from lollypop.utils import seconds_to_string
 from lollypop.objects import Track
-from lollypop import utils
 
 
 class Row(Gtk.ListBoxRow):
     """
         A row
     """
-    def __init__(self, rowid, num, artist_ids=[]):
+    __gsignals__ = {
+        "album-moved": (GObject.SignalFlags.RUN_FIRST, None, (str, bool)),
+        "track-moved": (GObject.SignalFlags.RUN_FIRST, None, (str, str, bool))
+    }
+
+    def __init__(self, track, dnd):
         """
             Init row widgets
-            @param rowid as int
-            @param num as int
-            @param artist_ids as [int]: Allow to tell Row that artist_ids
-                   should not be displayed
+            @param track as Track
+            @param dnd as bool
         """
         # We do not use Gtk.Builder for speed reasons
         Gtk.ListBoxRow.__init__(self)
         self._artists_label = None
-        self._track = Track(rowid)
-        self.__number = num
+        self._track = track
         self.__preview_timeout_id = None
         self.__context_timeout_id = None
         self.__context = None
-        self._indicator = IndicatorWidget(self._track.id)
-        self.set_indicator(Lp().player.current_track.id == self._track.id,
-                           utils.is_loved(self._track.id))
+        self._indicator = IndicatorWidget(self._track, self if dnd else None)
+        self.set_indicator(App().player.current_track.id == self._track.id,
+                           self._track.loved)
         self._row_widget = Gtk.EventBox()
-        self._row_widget.connect("button-press-event", self.__on_button_press)
-        self._row_widget.connect("enter-notify-event", self.__on_enter_notify)
-        self._row_widget.connect("leave-notify-event", self.__on_leave_notify)
+        self._row_widget.connect("button-release-event",
+                                 self.__on_button_release_event)
+        self._row_widget.connect("enter-notify-event",
+                                 self.__on_enter_notify_event)
+        self._row_widget.connect("leave-notify-event",
+                                 self.__on_leave_notify_event)
         self._grid = Gtk.Grid()
         self._grid.set_column_spacing(5)
         self._row_widget.add(self._grid)
@@ -58,11 +62,11 @@ class Row(Gtk.ListBoxRow):
         self._title_label.set_property("hexpand", True)
         self._title_label.set_property("halign", Gtk.Align.START)
         self._title_label.set_ellipsize(Pango.EllipsizeMode.END)
-        featuring_ids = self._track.get_featuring_ids(artist_ids)
-        if featuring_ids:
+        featuring_artist_ids = track.featuring_artist_ids
+        if featuring_artist_ids:
             artists = []
-            for artist_id in featuring_ids:
-                artists.append(Lp().artists.get_name(artist_id))
+            for artist_id in featuring_artist_ids:
+                artists.append(App().artists.get_name(artist_id))
             self._artists_label = Gtk.Label.new(GLib.markup_escape_text(
                                                            ", ".join(artists)))
             self._artists_label.set_use_markup(True)
@@ -85,10 +89,6 @@ class Row(Gtk.ListBoxRow):
         self._num_label.get_style_context().add_class("dim-label")
         self.update_num_label()
         self.__menu_button = Gtk.Button.new()
-        # Here a hack to make old Gtk version support min-height css attribute
-        # min-height = 24px, borders = 2px, we set directly on stack
-        # min-width = 24px, borders = 2px, padding = 8px
-        self.__menu_button.set_size_request(34, 26)
         self.__menu_button.set_relief(Gtk.ReliefStyle.NONE)
         self.__menu_button.get_style_context().add_class("menu-button")
         self.__menu_button.get_style_context().add_class("track-menu-button")
@@ -130,13 +130,6 @@ class Row(Gtk.ListBoxRow):
             else:
                 self._indicator.empty()
 
-    def set_number(self, num):
-        """
-            Set number
-            @param number as int
-        """
-        self.__number = num
-
     def update_duration(self):
         """
             Update duration for row
@@ -149,24 +142,33 @@ class Row(Gtk.ListBoxRow):
         """
             Update position label for row
         """
-        if Lp().player.track_in_queue(self._track):
+        if App().player.track_in_queue(self._track):
             self._num_label.get_style_context().add_class("queued")
-            pos = Lp().player.get_track_position(self._track.id)
+            pos = App().player.get_track_position(self._track.id)
             self._num_label.set_text(str(pos))
-        elif self.__number > 0:
+        elif self._track.number > 0:
             self._num_label.get_style_context().remove_class("queued")
-            self._num_label.set_text(str(self.__number))
+            self._num_label.set_text(str(self._track.number))
         else:
             self._num_label.get_style_context().remove_class("queued")
             self._num_label.set_text("")
 
     @property
-    def id(self):
+    def track(self):
         """
-            Get object id
-            @return Current id as int
+            Get row track
+            @return Track
         """
-        return self._track.id
+        return self._track
+
+#######################
+# PROTECTED           #
+#######################
+    def _get_menu(self):
+        """
+            Return TrackMenu
+        """
+        return TrackMenu(self._track)
 
 #######################
 # PRIVATE             #
@@ -176,11 +178,30 @@ class Row(Gtk.ListBoxRow):
             Play track
             @param widget as Gtk.Widget
         """
-        Lp().player.preview.set_property("uri", self._track.uri)
-        Lp().player.preview.set_state(Gst.State.PLAYING)
+        App().player.preview.set_property("uri", self._track.uri)
+        App().player.preview.set_state(Gst.State.PLAYING)
         self.set_indicator(True, False)
         self.__preview_timeout_id = None
 
+    def __popup_menu(self, widget, xcoordinate=None, ycoordinate=None):
+        """
+            Popup menu for track
+            @param widget as Gtk.Button
+            @param xcoordinate as int (or None)
+            @param ycoordinate as int (or None)
+        """
+        popover = TrackMenuPopover(self._track, self._get_menu())
+        if xcoordinate is not None and ycoordinate is not None:
+            rect = widget.get_allocation()
+            rect.x = xcoordinate
+            rect.y = ycoordinate
+            rect.width = rect.height = 1
+        popover.set_relative_to(widget)
+        popover.set_pointing_to(rect)
+        popover.connect("closed", self.__on_closed)
+        self.get_style_context().add_class("track-menu-selected")
+        popover.show()
+
     def __on_map(self, widget):
         """
             Fix for Gtk < 3.18,
@@ -201,10 +222,10 @@ class Row(Gtk.ListBoxRow):
             @param eventbox as Gtk.EventBox
             @param event as Gdk.EventButton
         """
-        Lp().window.show_artists_albums(self._album.artist_ids)
+        App().window.container.show_artists_albums(self._album.artist_ids)
         return True
 
-    def __on_enter_notify(self, widget, event):
+    def __on_enter_notify_event(self, widget, event):
         """
             Set image on buttons now, speed reason
             @param widget as Gtk.Widget
@@ -213,23 +234,30 @@ class Row(Gtk.ListBoxRow):
         if self.__context_timeout_id is not None:
             GLib.source_remove(self.__context_timeout_id)
             self.__context_timeout_id = None
-        if Lp().settings.get_value("preview-output").get_string() != "":
+        if App().settings.get_value("preview-output").get_string() != "":
             self.__preview_timeout_id = GLib.timeout_add(500,
                                                          self.__play_preview)
         if self.__menu_button.get_image() is None:
             image = Gtk.Image.new_from_icon_name("go-previous-symbolic",
                                                  Gtk.IconSize.MENU)
-            image.set_opacity(0.2)
             self.__menu_button.set_image(image)
-            self.__menu_button.connect("clicked", self.__on_button_clicked)
+            self.__menu_button.connect(
+                                      "button-release-event",
+                                      self.__on_indicator_button_release_event)
             self._indicator.update_button()
 
-    def __on_leave_notify(self, widget, event):
+    def __on_leave_notify_event(self, widget, event):
         """
             Stop preview
             @param widget as Gtk.Widget
             @param event as Gdk.Event
         """
+        def close_indicator():
+            """
+                Simulate a release event
+            """
+            self.__on_indicator_button_release_event(self.__menu_button,
+                                                     event)
         allocation = widget.get_allocation()
         if event.x <= 0 or\
            event.x >= allocation.width or\
@@ -238,61 +266,65 @@ class Row(Gtk.ListBoxRow):
             if self.__context is not None and\
                     self.__context_timeout_id is None:
                 self.__context_timeout_id = GLib.timeout_add(
-                                                      1000,
-                                                      self.__on_button_clicked,
-                                                      self.__menu_button)
-            if Lp().settings.get_value("preview-output").get_string() != "":
+                                     1000,
+                                     close_indicator)
+            if App().settings.get_value("preview-output").get_string() != "":
                 if self.__preview_timeout_id is not None:
                     GLib.source_remove(self.__preview_timeout_id)
                     self.__preview_timeout_id = None
                 self.set_indicator(
-                                Lp().player.current_track.id == self._track.id,
-                                utils.is_loved(self._track.id))
-                Lp().player.preview.set_state(Gst.State.NULL)
+                               App().player.current_track.id == self._track.id,
+                               self._track.loved)
+                App().player.preview.set_state(Gst.State.NULL)
 
-    def __on_button_press(self, widget, event):
+    def __on_button_release_event(self, widget, event):
         """
-            Popup menu for track relative to track row
+            Handle button press event:
+                |_ 1 => activate
+                |_ 2 => queue
+                |_ 3 => menu
             @param widget as Gtk.Widget
-            @param event as Gdk.Event
+            @param event as Gdk.EventButton
         """
-        if self.__context is not None:
-            self.__on_button_clicked(self.__menu_button)
         if event.button == 3:
-            if GLib.getenv("WAYLAND_DISPLAY") != "" and\
-                    self.get_ancestor(Gtk.Popover) is not None:
-                print("https://bugzilla.gnome.org/show_bug.cgi?id=774148")
             window = widget.get_window()
             if window == event.window:
                 self.__popup_menu(widget, event.x, event.y)
             # Happens when pressing button over menu btn
             else:
-                self.__on_button_clicked(self.__menu_button)
-            return True
+                self.__on_indicator_button_release_event(self.__menu_button,
+                                                         event)
         elif event.button == 2:
-            if self._track.id in Lp().player.queue:
-                Lp().player.del_from_queue(self._track.id)
+            if self._track.id in App().player.queue:
+                App().player.del_from_queue(self._track.id)
             else:
-                Lp().player.append_to_queue(self._track.id)
+                App().player.append_to_queue(self._track.id)
+        else:
+            self.activate()
+        return True
 
-    def __on_button_clicked(self, button):
+    def __on_indicator_button_release_event(self, button, event):
         """
             Popup menu for track relative to button
             @param button as Gtk.Button
+            @param event as Gdk.EventButton
         """
+        def on_hide(widget):
+            self.__on_indicator_button_release_event(button, event)
         self.__context_timeout_id = None
         image = self.__menu_button.get_image()
         if self.__context is None:
             image.set_from_icon_name("go-next-symbolic",
                                      Gtk.IconSize.MENU)
             self.__context = ContextWidget(self._track, button)
+            self.__context.connect("hide", on_hide)
             self.__context.set_property("halign", Gtk.Align.END)
             self.__context.show()
             self._duration_label.hide()
             self._grid.insert_next_to(button, Gtk.PositionType.LEFT)
             self._grid.attach_next_to(self.__context, button,
                                       Gtk.PositionType.LEFT, 1, 1)
-            self.set_indicator(Lp().player.current_track.id == self._track.id,
+            self.set_indicator(App().player.current_track.id == self._track.id,
                                False)
         else:
             image.set_from_icon_name("go-previous-symbolic",
@@ -300,27 +332,9 @@ class Row(Gtk.ListBoxRow):
             self.__context.destroy()
             self._duration_label.show()
             self.__context = None
-            self.set_indicator(Lp().player.current_track.id == self._track.id,
-                               utils.is_loved(self._track.id))
-
-    def __popup_menu(self, widget, xcoordinate=None, ycoordinate=None):
-        """
-            Popup menu for track
-            @param widget as Gtk.Button
-            @param xcoordinate as int (or None)
-            @param ycoordinate as int (or None)
-        """
-        popover = TrackMenuPopover(self._track, TrackMenu(self._track))
-        if xcoordinate is not None and ycoordinate is not None:
-            rect = widget.get_allocation()
-            rect.x = xcoordinate
-            rect.y = ycoordinate
-            rect.width = rect.height = 1
-        popover.set_relative_to(widget)
-        popover.set_pointing_to(rect)
-        popover.connect("closed", self.__on_closed)
-        self.get_style_context().add_class("track-menu-selected")
-        popover.show()
+            self.set_indicator(App().player.current_track.id == self._track.id,
+                               self._track.loved)
+        return True
 
     def __on_closed(self, widget):
         """
@@ -350,24 +364,16 @@ class PlaylistRow(Row):
     """
         A track row with album cover
     """
-    __gsignals__ = {
-        "track-moved": (GObject.SignalFlags.RUN_FIRST, None, (int, int, bool))
-    }
 
-    def __init__(self, rowid, num, show_headers):
+    def __init__(self, track, show_headers):
         """
             Init row widget
-            @param rowid as int
-            @param num as int
+            @param track as Track
             @param show headers as bool
         """
-        Row.__init__(self, rowid, num)
+        Row.__init__(self, track, True)
         self.__parent_filter = False
         self.__show_headers = show_headers
-        self._indicator.set_margin_start(5)
-        self._row_widget.set_margin_start(5)
-        self._row_widget.set_margin_top(2)
-        self._row_widget.set_margin_end(5)
         self._grid.insert_row(0)
         self._grid.insert_column(0)
         self._grid.insert_column(1)
@@ -381,7 +387,7 @@ class PlaylistRow(Row):
         box = Gtk.Box()
         box.set_homogeneous(True)
         box.add(self.__cover)
-        box.set_property("width-request", ArtSize.MEDIUM+2)
+        box.set_property("width-request", ArtSize.MEDIUM + 2)
         self._grid.attach(box, 0, 0, 1, 2)
         self.show_all()
         self.__header = Gtk.Grid()
@@ -396,43 +402,29 @@ class PlaylistRow(Row):
             self.__album_artist_label.set_ellipsize(Pango.EllipsizeMode.END)
             self.__album_artist_label.get_style_context().add_class(
                                                                    "dim-label")
-            artist_eventbox = Gtk.EventBox()
-            artist_eventbox.add(self.__album_artist_label)
-            artist_eventbox.connect("realize", self.__on_eventbox_realize)
-            artist_eventbox.connect("button-press-event",
-                                    self.__on_artist_button_press)
-            artist_eventbox.show()
-            self.__header.add(artist_eventbox)
+            self.__header.add(self.__album_artist_label)
         self.__album_label = Gtk.Label.new(self._track.album.name)
         self.__album_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.__album_label.get_style_context().add_class("dim-label")
         self.__album_label.set_hexpand(True)
         self.__album_label.set_property("halign", Gtk.Align.END)
         self.__header.add(self.__album_label)
-        self._num_label.set_property("valign", Gtk.Align.END)
-        self._title_label.set_property("valign", Gtk.Align.END)
-        if self._artists_label is not None:
-            self._artists_label.set_property("valign", Gtk.Align.END)
-        self._duration_label.set_property("valign", Gtk.Align.END)
-        self._indicator.set_property("valign", Gtk.Align.END)
         if self._artists_label is not None:
             self._grid.attach(self.__header, 1, 0, 5, 1)
         else:
             self._grid.attach(self.__header, 1, 0, 4, 1)
-        self.set_indicator(Lp().player.current_track.id == self._track.id,
-                           utils.is_loved(self._track.id))
+        self.set_indicator(App().player.current_track.id == self._track.id,
+                           self._track.loved)
         self.show_headers(self.__show_headers)
         self.drag_source_set(Gdk.ModifierType.BUTTON1_MASK, [],
                              Gdk.DragAction.MOVE)
         self.drag_source_add_text_targets()
-        self.drag_dest_set(Gtk.DestDefaults.DROP | Gtk.DestDefaults.MOTION,
+        self.drag_dest_set(Gtk.DestDefaults.DROP,
                            [], Gdk.DragAction.MOVE)
         self.drag_dest_add_text_targets()
         self.connect("drag-begin", self.__on_drag_begin)
         self.connect("drag-data-get", self.__on_drag_data_get)
         self.connect("drag-data-received", self.__on_drag_data_received)
-        self.connect("drag-motion", self.__on_drag_motion)
-        self.connect("drag-leave", self.__on_drag_leave)
 
     @property
     def filter(self):
@@ -468,7 +460,7 @@ class PlaylistRow(Row):
         self.__show_headers = show
         if show:
             self.__cover.set_tooltip_text(self._track.album.name)
-            surface = Lp().art.get_album_artwork(
+            surface = App().art.get_album_artwork(
                                         self._track.album,
                                         ArtSize.MEDIUM,
                                         self.get_scale_factor())
@@ -481,27 +473,18 @@ class PlaylistRow(Row):
             self.__cover.hide()
             self.__header.hide()
 
+#######################
+# PROTECTED           #
+#######################
+    def _get_menu(self):
+        """
+            Return TrackMenu
+        """
+        return TrackMenu(self._track, True)
+
 #######################
 # PRIVATE             #
 #######################
-    def __on_artist_button_press(self, eventbox, event):
-        """
-            Go to artist page
-            @param eventbox as Gtk.EventBox
-            @param event as Gdk.EventButton
-        """
-        Lp().window.show_artists_albums(self._track.album.artist_ids)
-        return True
-
-    def __on_eventbox_realize(self, eventbox):
-        """
-            Change cursor over eventbox
-            @param eventbox as Gdk.Eventbox
-        """
-        window = eventbox.get_window()
-        if window is not None:
-            window.set_cursor(Gdk.Cursor(Gdk.CursorType.HAND2))
-
     def __on_drag_begin(self, widget, context):
         """
             Set icon
@@ -533,47 +516,25 @@ class PlaylistRow(Row):
             @param info as int
             @param time as int
         """
+        from lollypop.view import View
+        view = widget.get_ancestor(View)
+        if view is not None:
+            view.clear_animation()
         height = self.get_allocated_height()
         if y > height/2:
-            up = False
+            down = True
         else:
-            up = True
+            down = False
         try:
-            src = int(data.get_text())
-            if self._track.id == src:
+            src = data.get_text()
+            if self._track.id == int(src):
                 return
-            self.emit("track-moved", self._track.id, src, up)
+            self.emit("track-moved", self._track.id, src, down)
         except:
-            if len(Lp().window.view.get_ids()) == 1:
-                Lp().playlists.import_uri(Lp().window.view.get_ids()[0],
-                                          data.get_text(), self._track.id, up)
-
-    def __on_drag_motion(self, widget, context, x, y, time):
-        """
-            Add style
-            @param widget as Gtk.Widget
-            @param context as Gdk.DragContext
-            @param x as int
-            @param y as int
-            @param time as int
-        """
-        height = self.get_allocated_height()
-        if y > height/2:
-            self.get_style_context().add_class("drag-up")
-            self.get_style_context().remove_class("drag-down")
-        else:
-            self.get_style_context().remove_class("drag-up")
-            self.get_style_context().add_class("drag-down")
-
-    def __on_drag_leave(self, widget, context, time):
-        """
-            Remove style
-            @param widget as Gtk.Widget
-            @param context as Gdk.DragContext
-            @param time as int
-        """
-        self.get_style_context().remove_class("drag-up")
-        self.get_style_context().remove_class("drag-down")
+            if len(App().window.container.view.get_ids()) == 1:
+                App().playlists.import_uri(
+                                    App().window.container.view.get_ids()[0],
+                                    data.get_text(), self._track.id, down)
 
 
 class TrackRow(Row):
@@ -598,19 +559,27 @@ class TrackRow(Row):
             height = menu_height
         return height
 
-    def __init__(self, rowid, num, artist_ids):
+    def __init__(self, track, dnd):
         """
             Init row widget and show it
-            @param rowid as int
-            @param num as int
-            @param artist_ids as [int]: Allow to tell Row that artist_ids
-                   should not be displayed
+            @param track as Track
+            @param dnd as bool
         """
-        Row.__init__(self, rowid, num, artist_ids)
+        Row.__init__(self, track, dnd)
         self.__parent_filter = False
         self._grid.insert_column(0)
         self._grid.attach(self._indicator, 0, 0, 1, 1)
         self.show_all()
+        if dnd:
+            self.drag_source_set(Gdk.ModifierType.BUTTON1_MASK, [],
+                                 Gdk.DragAction.MOVE)
+            self.drag_source_add_text_targets()
+            self.drag_dest_set(Gtk.DestDefaults.DROP,
+                               [], Gdk.DragAction.MOVE)
+            self.drag_dest_add_text_targets()
+            self.connect("drag-begin", self.__on_drag_begin)
+            self.connect("drag-data-get", self.__on_drag_data_get)
+            self.connect("drag-data-received", self.__on_drag_data_received)
 
     @property
     def filter(self):
@@ -631,9 +600,60 @@ class TrackRow(Row):
             True if filtered by parent
         """
         return self.__parent_filter
+
 #######################
 # PRIVATE             #
 #######################
+    def __on_drag_begin(self, widget, context):
+        """
+            Set icon and update view padding
+            @param widget as Gtk.Widget
+            @param context as Gdk.DragContext
+        """
+        widget.drag_source_set_icon_name("emblem-music-symbolic")
+
+    def __on_drag_data_get(self, widget, context, data, info, time):
+        """
+            Send track id
+            @param widget as Gtk.Widget
+            @param context as Gdk.DragContext
+            @param data as Gtk.SelectionData
+            @param info as int
+            @param time as int
+        """
+        track = "t:%s:%s" % (self._track, self._track.album)
+        data.set_text(track, len(track))
+
+    def __on_drag_data_received(self, widget, context, x, y, data, info, time):
+        """
+            Move track
+            @param widget as Gtk.Widget
+            @param context as Gdk.DragContext
+            @param x as int
+            @param y as int
+            @param data as Gtk.SelectionData
+            @param info as int
+            @param time as int
+        """
+        from lollypop.view import View
+        view = widget.get_ancestor(View)
+        if view is not None:
+            view.clear_animation()
+        height = self.get_allocated_height()
+        if y > height/2:
+            down = True
+        else:
+            down = False
+        try:
+            (type_id, object_str, album_str) = data.get_text().split(":")
+            if object_str == str(self._track):
+                pass
+            elif type_id == "t":
+                self.emit("track-moved", object_str, album_str, down)
+            elif type_id == "a":
+                self.emit("album-moved", object_str, down)
+        except Exception as e:
+            print("Row::__on_drag_data_received():", e)
 
 
 class TracksWidget(Gtk.ListBox):
@@ -642,7 +662,8 @@ class TracksWidget(Gtk.ListBox):
     """
 
     __gsignals__ = {
-        "activated": (GObject.SignalFlags.RUN_FIRST, None, (int,))
+        "activated": (GObject.SignalFlags.RUN_FIRST,
+                      None, (GObject.TYPE_PYOBJECT,))
     }
 
     def __init__(self, dnd=False):
@@ -652,12 +673,12 @@ class TracksWidget(Gtk.ListBox):
         """
         Gtk.ListBox.__init__(self)
         self.connect("destroy", self.__on_destroy)
-        self.__queue_signal_id = Lp().player.connect("queue-changed",
-                                                     self.__on_queue_changed)
-        self.__loved_signal_id1 = Lp().playlists.connect(
+        self.__queue_signal_id = App().player.connect("queue-changed",
+                                                      self.__on_queue_changed)
+        self.__loved_signal_id1 = App().playlists.connect(
                                               "playlist-add",
                                               self.__on_loved_playlist_changed)
-        self.__loved_signal_id2 = Lp().playlists.connect(
+        self.__loved_signal_id2 = App().playlists.connect(
                                               "playlist-del",
                                               self.__on_loved_playlist_changed)
         self.connect("row-activated", self.__on_activate)
@@ -665,7 +686,7 @@ class TracksWidget(Gtk.ListBox):
         self.set_property("hexpand", True)
         self.set_property("selection-mode", Gtk.SelectionMode.NONE)
         if dnd:
-            self.drag_dest_set(Gtk.DestDefaults.DROP | Gtk.DestDefaults.MOTION,
+            self.drag_dest_set(Gtk.DestDefaults.DROP,
                                [], Gdk.DragAction.MOVE)
             self.drag_dest_add_text_targets()
             self.connect("drag-data-received", self.__on_drag_data_received)
@@ -676,12 +697,11 @@ class TracksWidget(Gtk.ListBox):
             @param previous album id as int
         """
         for child in self.get_children():
-            track = Track(child.id)
-            if track.album.id == prev_album_id:
+            if child.track.album.id == prev_album_id:
                 child.show_headers(False)
             else:
                 child.show_headers(True)
-            prev_album_id = track.album.id
+            prev_album_id = child.track.album.id
 
     def update_indexes(self, start):
         """
@@ -689,7 +709,7 @@ class TracksWidget(Gtk.ListBox):
             @param start index as int
         """
         for row in self.get_children():
-            row.set_number(start)
+            row.track.set_number(start)
             row.update_num_label()
             start += 1
 
@@ -699,8 +719,8 @@ class TracksWidget(Gtk.ListBox):
             @param track id as int
         """
         for row in self.get_children():
-            row.set_indicator(row.id == track_id,
-                              utils.is_loved(row.id))
+            row.set_indicator(row.track.id == track_id,
+                              Track(track_id).loved)
 
     def update_duration(self, track_id):
         """
@@ -735,13 +755,18 @@ class TracksWidget(Gtk.ListBox):
             @param time as int
         """
         try:
+            from lollypop.view import View
+            view = widget.get_ancestor(View)
+            if view is not None:
+                view.clear_animation()
             value = int(data.get_text())
             bottom_row = self.get_children()[-1]
             bottom_row.emit("track-moved", bottom_row.id, value, False)
         except:
-            if len(Lp().window.view.get_ids()) == 1:
-                Lp().playlists.import_uri(Lp().window.view.get_ids()[0],
-                                          data.get_text())
+            if len(App().window.container.view.get_ids()) == 1:
+                App().playlists.import_uri(
+                                      App().window.container.view.get_ids()[0],
+                                      data.get_text())
 
     def __on_queue_changed(self, unused):
         """
@@ -761,11 +786,10 @@ class TracksWidget(Gtk.ListBox):
         """
         if playlist_id != Type.LOVED:
             return
-
         for row in self.get_children():
-            if track_id == row.id:
-                row.set_indicator(track_id == Lp().player.current_track.id,
-                                  utils.is_loved(track_id))
+            if track_id == row.track.id:
+                row.set_indicator(track_id == App().player.current_track.id,
+                                  Track(track_id).loved)
 
     def __on_destroy(self, widget):
         """
@@ -773,13 +797,13 @@ class TracksWidget(Gtk.ListBox):
             @param widget as Gtk.Widget
         """
         if self.__queue_signal_id is not None:
-            Lp().player.disconnect(self.__queue_signal_id)
+            App().player.disconnect(self.__queue_signal_id)
             self.__queue_signal_id = None
         if self.__loved_signal_id1 is not None:
-            Lp().playlists.disconnect(self.__loved_signal_id1)
+            App().playlists.disconnect(self.__loved_signal_id1)
             self.__loved_signal_id1 = None
         if self.__loved_signal_id2 is not None:
-            Lp().playlists.disconnect(self.__loved_signal_id2)
+            App().playlists.disconnect(self.__loved_signal_id2)
             self.__loved_signal_id2 = None
 
     def __on_activate(self, widget, row):
@@ -788,4 +812,4 @@ class TracksWidget(Gtk.ListBox):
             @param widget as TracksWidget
             @param row as TrackRow
         """
-        self.emit("activated", row.id)
+        self.emit("activated", row.track)
