@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -12,12 +12,11 @@
 
 from gi.repository import Gtk, Gio, Gdk, GLib, Gst
 
-from gettext import gettext as _
 from lollypop.container import Container
-from lollypop.define import Lp, WindowSize
+from lollypop.define import App, WindowSize
 from lollypop.toolbar import Toolbar
 from lollypop.helper_task import TaskHelper
-from lollypop.utils import is_unity, set_loved, is_loved
+from lollypop.utils import is_unity
 
 
 class Window(Gtk.ApplicationWindow, Container):
@@ -29,28 +28,28 @@ class Window(Gtk.ApplicationWindow, Container):
         """
             Init window
         """
-        Container.__init__(self)
         self.__signal1 = None
         self.__signal2 = None
         self.__timeout = None
+        self.__miniplayer = None
         self.__mediakeys = None
         self.__media_keys_busnames = []
         self.__was_maximized = False
         Gtk.ApplicationWindow.__init__(self,
-                                       application=Lp(),
+                                       application=App(),
                                        title="Lollypop",
                                        icon_name="org.gnome.Lollypop")
         self.connect("hide", self.__on_hide)
-        Lp().player.connect("current-changed", self.__on_current_changed)
+        App().player.connect("current-changed", self.__on_current_changed)
         self.__timeout_configure = None
         seek_action = Gio.SimpleAction.new("seek",
                                            GLib.VariantType.new("i"))
         seek_action.connect("activate", self.__on_seek_action)
-        Lp().add_action(seek_action)
+        App().add_action(seek_action)
         player_action = Gio.SimpleAction.new("shortcut",
                                              GLib.VariantType.new("s"))
         player_action.connect("activate", self.__on_player_action)
-        Lp().add_action(player_action)
+        App().add_action(player_action)
 
         self.__setup_global_shortcuts()
 
@@ -61,12 +60,11 @@ class Window(Gtk.ApplicationWindow, Container):
         self.__main_stack.show()
 
         self.__setup_content()
-        self.setup_window()
         # FIXME Remove this, handled by MPRIS in GNOME 3.26
         self.__setup_media_keys()
         self.__enabled_shortcuts = False
         self.enable_global_shortcuts(True)
-
+        self.set_auto_startup_notification(False)
         self.connect("destroy", self.__on_destroyed_window)
         self.connect("realize", self.__on_realize)
 
@@ -87,45 +85,46 @@ class Window(Gtk.ApplicationWindow, Container):
         self.__enabled_shortcuts = enable
         if enable:
             if Gtk.Widget.get_default_direction() == Gtk.TextDirection.RTL:
-                Lp().set_accels_for_action("app.seek(10)", ["Left"])
-                Lp().set_accels_for_action("app.seek(20)", ["<Control>Left"])
-                Lp().set_accels_for_action("app.seek(-10)", ["Right"])
-                Lp().set_accels_for_action("app.seek(-20)", ["<Control>Right"])
+                App().set_accels_for_action("app.seek(10)", ["Left"])
+                App().set_accels_for_action("app.seek(20)", ["<Control>Left"])
+                App().set_accels_for_action("app.seek(-10)", ["Right"])
+                App().set_accels_for_action("app.seek(-20)",
+                                            ["<Control>Right"])
             else:
-                Lp().set_accels_for_action("app.seek(10)", ["Right"])
-                Lp().set_accels_for_action("app.seek(20)", ["<Control>Right"])
-                Lp().set_accels_for_action("app.seek(-10)", ["Left"])
-                Lp().set_accels_for_action("app.seek(-20)", ["<Control>Left"])
+                App().set_accels_for_action("app.seek(10)", ["Right"])
+                App().set_accels_for_action("app.seek(20)", ["<Control>Right"])
+                App().set_accels_for_action("app.seek(-10)", ["Left"])
+                App().set_accels_for_action("app.seek(-20)", ["<Control>Left"])
 
-            Lp().set_accels_for_action("app.shortcut::play_pause",
-                                       ["space", "c"])
-            Lp().set_accels_for_action("app.shortcut::play", ["x"])
-            Lp().set_accels_for_action("app.shortcut::stop", ["v"])
-            Lp().set_accels_for_action("app.shortcut::next", ["n"])
-            Lp().set_accels_for_action("app.shortcut::prev", ["p"])
-            Lp().set_accels_for_action("app.shortcut::loved", ["l"])
+            App().set_accels_for_action("app.shortcut::play_pause",
+                                        ["space", "c"])
+            App().set_accels_for_action("app.shortcut::play", ["x"])
+            App().set_accels_for_action("app.shortcut::stop", ["v"])
+            App().set_accels_for_action("app.shortcut::next", ["n"])
+            App().set_accels_for_action("app.shortcut::prev", ["p"])
+            App().set_accels_for_action("app.shortcut::loved", ["l"])
         else:
-            Lp().set_accels_for_action("app.seek(10)", [None])
-            Lp().set_accels_for_action("app.seek(20)", [None])
-            Lp().set_accels_for_action("app.seek(-10)", [None])
-            Lp().set_accels_for_action("app.seek(-20)", [None])
-            Lp().set_accels_for_action("app.shortcut::play_pause", [None])
-            Lp().set_accels_for_action("app.shortcut::play", [None])
-            Lp().set_accels_for_action("app.shortcut::stop", [None])
-            Lp().set_accels_for_action("app.shortcut::play_pause", [None])
-            Lp().set_accels_for_action("app.shortcut::play", [None])
-            Lp().set_accels_for_action("app.shortcut::stop", [None])
-            Lp().set_accels_for_action("app.shortcut::next", [None])
-            Lp().set_accels_for_action("app.shortcut::next_album", [None])
-            Lp().set_accels_for_action("app.shortcut::prev", [None])
-            Lp().set_accels_for_action("app.shortcut::loved", [None])
+            App().set_accels_for_action("app.seek(10)", [None])
+            App().set_accels_for_action("app.seek(20)", [None])
+            App().set_accels_for_action("app.seek(-10)", [None])
+            App().set_accels_for_action("app.seek(-20)", [None])
+            App().set_accels_for_action("app.shortcut::play_pause", [None])
+            App().set_accels_for_action("app.shortcut::play", [None])
+            App().set_accels_for_action("app.shortcut::stop", [None])
+            App().set_accels_for_action("app.shortcut::play_pause", [None])
+            App().set_accels_for_action("app.shortcut::play", [None])
+            App().set_accels_for_action("app.shortcut::stop", [None])
+            App().set_accels_for_action("app.shortcut::next", [None])
+            App().set_accels_for_action("app.shortcut::next_album", [None])
+            App().set_accels_for_action("app.shortcut::prev", [None])
+            App().set_accels_for_action("app.shortcut::loved", [None])
 
     def setup_window(self):
         """
             Setup window position and size, callbacks
         """
         self.__setup_pos_size("window")
-        if Lp().settings.get_value("window-maximized"):
+        if App().settings.get_value("window-maximized"):
             self.maximize()
 
         if self.__signal1 is None:
@@ -141,15 +140,33 @@ class Window(Gtk.ApplicationWindow, Container):
         """
         size = self.get_size()
         self.__toolbar.set_content_width(size[0])
-        self.__show_miniplayer(size[0] < WindowSize.MEDIUM)
-        self.__show_subtoolbar(size[0] < WindowSize.MONSTER and
-                               size[0] > WindowSize.MEDIUM)
+        if size[0] < WindowSize.BIG:
+            self.__show_miniplayer(True)
+            self.__container.paned_stack(True)
+            self.__main_stack.show()
+            if self.__miniplayer is not None:
+                self.__miniplayer.set_vexpand(False)
+            self.__toolbar.title.hide()
+            self.__toolbar.info.hide()
+            self.__toolbar.end.set_minimal(True)
+        else:
+            self.__container.paned_stack(False)
+            self.__main_stack.show()
+            self.__show_miniplayer(False)
+            self.__toolbar.title.show()
+            self.__toolbar.info.show()
+            self.__toolbar.end.set_minimal(False)
+        if size[1] < WindowSize.MEDIUM:
+            if self.__miniplayer is not None and\
+                    self.__miniplayer.is_visible():
+                self.__main_stack.hide()
+                self.__miniplayer.set_vexpand(True)
 
     def set_mini(self):
         """
             Set mini player on/off
         """
-        if Lp().player.current_track.id is None:
+        if App().player.current_track.id is None:
             return
         was_maximized = self.is_maximized()
         if self.__main_stack.get_visible_child_name() == "main":
@@ -177,11 +194,21 @@ class Window(Gtk.ApplicationWindow, Container):
             @param widget as Gtk.Widget
             @param event as Gdk.event
         """
-        if event.type == Gdk.EventType.FOCUS_CHANGE and self.view is not None:
-            self.view.disable_overlay()
-            Lp().player.preview.set_state(Gst.State.NULL)
+        if event.type == Gdk.EventType.FOCUS_CHANGE and\
+                self.__container.view is not None:
+            if hasattr(self.__container.view, "disable_overlay"):
+                self.__container.view.disable_overlay()
+            App().player.preview.set_state(Gst.State.NULL)
         Gtk.ApplicationWindow.do_event(self, event)
 
+    @property
+    def container(self):
+        """
+            Get container
+            @return Container
+        """
+        return self.__container
+
 ############
 # Private  #
 ############
@@ -189,70 +216,41 @@ class Window(Gtk.ApplicationWindow, Container):
         """
             Setup global shortcuts
         """
-        Lp().set_accels_for_action("app.shortcut::locked", ["<Control>l"])
-        Lp().set_accels_for_action("app.shortcut::filter", ["<Control>i"])
-        Lp().set_accels_for_action("app.shortcut::volume", ["<Alt>v"])
-        Lp().set_accels_for_action("app.shortcut::next_album", ["<Control>n"])
-        Lp().set_accels_for_action("app.shortcut::show_genres", ["<Control>g"])
-        Lp().set_accels_for_action("app.shortcut::hide_pane", ["<Control>h"])
-        Lp().set_accels_for_action("app.update_db", ["<Control>u"])
-        Lp().set_accels_for_action("app.settings", ["<Control>s"])
-        Lp().set_accels_for_action("app.fullscreen", ["F11", "F7"])
-        Lp().set_accels_for_action("app.mini", ["<Control>m"])
-        Lp().set_accels_for_action("app.about", ["F3"])
-        Lp().set_accels_for_action("app.shortcuts", ["F2"])
-        Lp().set_accels_for_action("app.help", ["F1"])
-        Lp().set_accels_for_action("app.quit", ["<Control>q"])
+        App().set_accels_for_action("app.shortcut::locked", ["<Control>l"])
+        App().set_accels_for_action("app.shortcut::filter", ["<Control>i"])
+        App().set_accels_for_action("app.shortcut::volume", ["<Alt>v"])
+        App().set_accels_for_action("app.shortcut::next_album", ["<Control>n"])
+        App().set_accels_for_action("app.shortcut::show_genres",
+                                    ["<Control>g"])
+        App().set_accels_for_action("app.shortcut::hide_paned", ["<Control>h"])
+        App().set_accels_for_action("app.update_db", ["<Control>u"])
+        App().set_accels_for_action("app.settings", ["<Control>s"])
+        App().set_accels_for_action("app.fullscreen", ["F11", "F7"])
+        App().set_accels_for_action("app.mini", ["<Control>m"])
+        App().set_accels_for_action("app.about", ["F3"])
+        App().set_accels_for_action("app.shortcuts", ["F2"])
+        App().set_accels_for_action("app.help", ["F1"])
+        App().set_accels_for_action("app.quit", ["<Control>q"])
 
-    def __show_subtoolbar(self, show):
+    def __show_miniplayer(self, show):
         """
             Show/hide subtoolbar
             @param show as bool
         """
-        is_visible = self.__subtoolbar.is_visible()
-        if show and not is_visible:
+        if show and self.__miniplayer is None:
             from lollypop.miniplayer import MiniPlayer
-            mini = MiniPlayer()
-            mini.show()
-            self.__subtoolbar.add(mini)
-            self.__subtoolbar.show()
-        elif not show and is_visible:
-            children = self.__subtoolbar.get_children()
-            if children:
-                children[0].destroy()
-            self.__subtoolbar.hide()
-
-    def __show_miniplayer(self, show):
-        """
-            Show/hide miniplayer
-            @param show as bool
-        """
-        mini = self.__main_stack.get_child_by_name("mini")
-        if show:
-            if mini is not None:
-                if self.__timeout is not None:
-                    GLib.source_remove(self.__timeout)
-            else:
-                from lollypop.miniplayer import MiniPlayer
-                mini = MiniPlayer()
-                self.__main_stack.add_named(mini, "mini")
-            self.__timeout = None
-            mini.show()
-            self.__main_stack.set_visible_child_name("mini")
-            self.__toolbar.set_show_close_button(False)
-        elif mini is not None and not show and self.__timeout is None:
-            self.__main_stack.set_visible_child_name("main")
-            self.__toolbar.set_show_close_button(
-                                not Lp().settings.get_value("disable-csd") and
-                                not is_unity())
-            self.__timeout = GLib.timeout_add(1000, mini.destroy)
+            self.__miniplayer = MiniPlayer(self.get_size()[0])
+            self.__vgrid.add(self.__miniplayer)
+        elif not show and self.__miniplayer is not None:
+            self.__miniplayer.destroy()
+            self.__miniplayer = None
 
     def __setup_pos_size(self, name):
         """
             Set window pos and size based on name
             @param name as str
         """
-        size_setting = Lp().settings.get_value("%s-size" % name)
+        size_setting = App().settings.get_value("%s-size" % name)
         if len(size_setting) == 2 and\
            isinstance(size_setting[0], int) and\
            isinstance(size_setting[1], int):
@@ -269,7 +267,7 @@ class Window(Gtk.ApplicationWindow, Container):
             Set window position
             @param name as str
         """
-        position_setting = Lp().settings.get_value("%s-position" % name)
+        position_setting = App().settings.get_value("%s-position" % name)
         if len(position_setting) == 2 and\
            isinstance(position_setting[0], int) and\
            isinstance(position_setting[1], int):
@@ -292,7 +290,7 @@ class Window(Gtk.ApplicationWindow, Container):
         if self.__media_keys_busnames:
             bus_name = self.__media_keys_busnames.pop(0)
             try:
-                bus = Lp().get_dbus_connection()
+                bus = App().get_dbus_connection()
                 Gio.DBusProxy.new(
                     bus,
                     Gio.DBusProxyFlags.DO_NOT_LOAD_PROPERTIES,
@@ -342,40 +340,39 @@ class Window(Gtk.ApplicationWindow, Container):
         app, action = param.unpack()
         if app == "org.gnome.Lollypop":
             if action == "Play":
-                Lp().player.play_pause()
+                App().player.play_pause()
             elif action == "Next":
-                Lp().player.next()
+                App().player.next()
             elif action == "Stop":
-                Lp().player.stop()
+                App().player.stop()
             elif action == "Previous":
-                Lp().player.prev()
+                App().player.prev()
 
     def __setup_content(self):
         """
             Setup window content
         """
-        vgrid = Gtk.Grid()
-        vgrid.set_orientation(Gtk.Orientation.VERTICAL)
-        vgrid.show()
+        self.__container = Container()
+        self.__container.show()
+        self.__vgrid = Gtk.Grid()
+        self.__vgrid.set_orientation(Gtk.Orientation.VERTICAL)
+        self.__vgrid.show()
         self.__toolbar = Toolbar()
         self.__toolbar.show()
-        self.__subtoolbar = Gtk.Grid()
-        if Lp().settings.get_value("disable-csd") or is_unity():
-            vgrid.add(self.__toolbar)
+        if App().settings.get_value("disable-csd") or is_unity():
+            self.__vgrid.add(self.__toolbar)
         else:
             self.set_titlebar(self.__toolbar)
             self.__toolbar.set_show_close_button(
-                                    not Lp().settings.get_value("disable-csd"))
-        vgrid.add(self.__main_stack)
-        vgrid.add(self.__subtoolbar)
-        self.add(vgrid)
-        self.__main_stack.add_named(self._paned_main_list, "main")
+                                   not App().settings.get_value("disable-csd"))
+        self.__vgrid.add(self.__main_stack)
+        self.add(self.__vgrid)
+        self.__main_stack.add_named(self.__container, "main")
         self.__main_stack.set_visible_child_name("main")
         self.drag_dest_set(Gtk.DestDefaults.DROP | Gtk.DestDefaults.MOTION,
                            [], Gdk.DragAction.MOVE)
         self.drag_dest_add_text_targets()
         self.connect("drag-data-received", self.__on_drag_data_received)
-        self.connect("drag-motion", self.__on_drag_motion)
         self.connect("drag-leave", self.__on_drag_leave)
 
     def __on_drag_data_received(self, widget, context, x, y, data, info, time):
@@ -389,29 +386,21 @@ class Window(Gtk.ApplicationWindow, Container):
             @param info as int
             @param time as int
         """
-        from lollypop.collectionimporter import CollectionImporter
-        importer = CollectionImporter()
-        uris = data.get_text().strip("\n").split("\r")
-        task_helper = TaskHelper()
-        task_helper.run(importer.add, uris, callback=(self.update_db,))
-
-    def __on_drag_motion(self, widget, context, x, y, time):
-        """
-            Add style
-            @param widget as Gtk.Widget
-            @param context as Gdk.DragContext
-            @param x as int
-            @param y as int
-            @param time as int
-        """
-        import_widget = self.__main_stack.get_child_by_name("import")
-        if import_widget is None:
-            import_widget = Gtk.Label()
-            import_widget.set_markup(_("<span size='xx-large'>"
-                                       "<b>Import music</b></span>"))
-            import_widget.show()
-            self.__main_stack.add_named(import_widget, "import")
-        self.__main_stack.set_visible_child_name("import")
+        try:
+            from lollypop.collectionimporter import CollectionImporter
+            from urllib.parse import urlparse
+            importer = CollectionImporter()
+            uris = []
+            for uri in data.get_text().strip("\n").split("\r"):
+                parsed = urlparse(uri)
+                if parsed.scheme in ["file", "sftp", "smb", "webdav"]:
+                    uris.append(uri)
+            if uris:
+                task_helper = TaskHelper()
+                task_helper.run(importer.add, uris,
+                                callback=(App().scanner.update,))
+        except:
+            pass
 
     def __on_drag_leave(self, widget, context, time):
         """
@@ -457,28 +446,30 @@ class Window(Gtk.ApplicationWindow, Container):
         """
         self.__timeout_configure = None
         size = widget.get_size()
-        if size[0] > WindowSize.MEDIUM:
+        if self.__main_stack.is_visible():
             name = "window"
         else:
             name = "mini"
-        Lp().settings.set_value("%s-size" % name,
-                                GLib.Variant("ai", [size[0], size[1]]))
+        if self.__miniplayer is not None:
+            self.__miniplayer.update_cover(size[0])
+        App().settings.set_value("%s-size" % name,
+                                 GLib.Variant("ai", [size[0], size[1]]))
 
         position = widget.get_position()
-        Lp().settings.set_value("%s-position" % name,
-                                GLib.Variant("ai",
-                                             [position[0], position[1]]))
+        App().settings.set_value("%s-position" % name,
+                                 GLib.Variant("ai",
+                                              [position[0], position[1]]))
 
     def __on_window_state_event(self, widget, event):
         """
             Save maximised state
         """
-        Lp().settings.set_boolean("window-maximized",
-                                  "GDK_WINDOW_STATE_MAXIMIZED" in
-                                  event.new_window_state.value_names)
-        # FIXME Remove this, handled by MPRIS in GNOME 3.26
+        App().settings.set_boolean("window-maximized",
+                                   "GDK_WINDOW_STATE_MAXIMIZED" in
+                                   event.new_window_state.value_names)
         if event.changed_mask & Gdk.WindowState.FOCUSED and \
            event.new_window_state & Gdk.WindowState.FOCUSED:
+            # FIXME Remove this, handled by MPRIS in GNOME 3.26
             self.__grab_media_keys()
 
     def __on_destroyed_window(self, widget):
@@ -488,16 +479,8 @@ class Window(Gtk.ApplicationWindow, Container):
         """
         if self.__was_maximized and\
            self.__main_stack.get_visible_child_name() == "mini":
-            Lp().settings.set_boolean("window-maximized", True)
-        main_pos = self._paned_main_list.get_position()
-        listview_pos = self._paned_list_view.get_position()
-        listview_pos = listview_pos if listview_pos > 100 else 100
-        Lp().settings.set_value("paned-mainlist-width",
-                                GLib.Variant("i",
-                                             main_pos))
-        Lp().settings.set_value("paned-listview-width",
-                                GLib.Variant("i",
-                                             listview_pos))
+            App().settings.set_boolean("window-maximized", True)
+        self.__container.save_internals()
 
     def __on_seek_action(self, action, param):
         """
@@ -506,14 +489,14 @@ class Window(Gtk.ApplicationWindow, Container):
             @param param as GLib.Variant
         """
         seconds = param.get_int32()
-        position = Lp().player.position
+        position = App().player.position
         seek = position / Gst.SECOND + seconds
         if seek < 0:
             seek = 0
-        if seek > Lp().player.current_track.duration:
-            seek = Lp().player.current_track.duration - 2
-        Lp().player.seek(seek)
-        if Lp().player.current_track.id is not None:
+        if seek > App().player.current_track.duration:
+            seek = App().player.current_track.duration - 2
+        App().player.seek(seek)
+        if App().player.current_track.id is not None:
             self.__toolbar.update_position(seek)
 
     def __on_player_action(self, action, param):
@@ -524,63 +507,66 @@ class Window(Gtk.ApplicationWindow, Container):
         """
         string = param.get_string()
         if string == "play_pause":
-            Lp().player.play_pause()
+            App().player.play_pause()
         elif string == "play":
-            Lp().player.play()
+            App().player.play()
         elif string == "stop":
-            Lp().player.stop()
+            App().player.stop()
         elif string == "next":
-            Lp().player.next()
+            App().player.next()
         elif string == "next_album":
-            Lp().player.skip_album()
+            App().player.skip_album()
         elif string == "prev":
-            Lp().player.prev()
+            App().player.prev()
         elif string == "locked":
-            Lp().player.lock()
-        elif string == "hide_pane":
-            self._hide_pane()
+            App().player.lock()
+        elif string == "hide_paned":
+            self.__container.hide_paned()
         elif string == "filter":
             if self.view is not None:
                 self.view.set_search_mode()
         elif string == "volume":
-            self.__toolbar.show_hide_volume_control()
+            self.__toolbar.title.show_hide_volume_control()
         elif string == "show_genres":
-            state = not Lp().settings.get_value("show-genres")
-            Lp().settings.set_value("show-genres",
-                                    GLib.Variant("b", state))
-            Lp().window.show_genres(state)
+            state = not App().settings.get_value("show-genres")
+            App().settings.set_value("show-genres",
+                                     GLib.Variant("b", state))
+            self.__container.show_genres(state)
         elif string == "loved":
-            if Lp().player.current_track.id is not None and\
-                    Lp().player.current_track.id >= 0:
-                isloved = is_loved(Lp().player.current_track.id)
-                set_loved(Lp().player.current_track.id, not isloved)
-                if Lp().notify is not None:
-                    if isloved:
+            track = App().player.current_track
+            if track.id >= 0:
+                is_loved = track.loved()
+                track.set_loved(not is_loved)
+                if App().notify is not None:
+                    if is_loved:
                         heart = "♡"
                     else:
                         heart = "❤"
-                    Lp().notify.send("%s - %s: %s" % (
-                                ", ".join(Lp().player.current_track.artists),
-                                Lp().player.current_track.name,
-                                heart))
+                    App().notify.send("%s - %s: %s" %
+                                      (", ".join(track.artists),
+                                       track.name,
+                                       heart))
 
     def __on_realize(self, widget):
         """
             Run scanner on realize
             @param widget as Gtk.Widget
         """
-        if Lp().settings.get_value("auto-update") or Lp().tracks.is_empty():
+        self.setup_window()
+        if App().settings.get_value("auto-update") or App().tracks.is_empty():
             # Delayed, make python segfault on sys.exit() otherwise
             # No idea why, maybe scanner using Gstpbutils before Gstreamer
             # initialisation is finished...
-            GLib.timeout_add(2000, self.update_db)
+            GLib.timeout_add(2000, App().scanner.update)
+        GLib.idle_add(self.__container.init_list_one)
+        GLib.idle_add(self.__container.restore_view_state)
 
     def __on_current_changed(self, player):
         """
             Update toolbar
             @param player as Player
         """
-        if Lp().player.current_track.id is None:
+        if App().player.current_track.id is None:
             self.set_title("Lollypop")
         else:
             self.set_title(", ".join(player.current_track.artists) + " - " +
